@@ -203,7 +203,7 @@ class core_backup_renderer extends plugin_renderer_base {
                         $table->data = array();
                     }
                     $name = get_string('pluginname', $activity->modulename);
-                    $icon = new image_icon('icon', '', $activity->modulename, ['class' => 'iconlarge icon-pre']);
+                    $icon = new image_icon('monologo', '', $activity->modulename, ['class' => 'iconlarge icon-pre']);
                     $table->data[] = array(
                         $this->output->render($icon).$name,
                         $activity->title,
@@ -553,8 +553,7 @@ class core_backup_renderer extends plugin_renderer_base {
     }
 
     /**
-     * Displays a continue button, overriding core renderer method of the same in order
-     * to override submission method of the button form
+     * Displays a continue button
      *
      * @param string|moodle_url $url
      * @param string $method
@@ -567,6 +566,7 @@ class core_backup_renderer extends plugin_renderer_base {
         if ($method != 'post') {
             $method = 'get';
         }
+        $url->param('sesskey', sesskey());
         $button = new single_button($url, get_string('continue'), $method, true);
         $button->class = 'continuebutton';
         return $this->render($button);
@@ -1022,7 +1022,7 @@ class core_backup_renderer extends plugin_renderer_base {
         $tabledata = array();
 
         // Get all in progress course copies for this user.
-        $copies = \copy_helper::get_copies($userid, $courseid);
+        $copies = \core_backup\copy\copy::get_copies($userid, $courseid);
 
         foreach ($copies as $copy) {
             $sourceurl = new \moodle_url('/course/view.php', array('id' => $copy->sourceid));
@@ -1030,7 +1030,7 @@ class core_backup_renderer extends plugin_renderer_base {
             $tablerow = array(
                 html_writer::link($sourceurl, $copy->source),
                 $copy->destination,
-                userdate($copy->timecreated),
+                userdate($copy->time),
                 get_string($copy->operation),
                 $this->get_status_display($copy->status, $copy->backupid, $copy->restoreid, $copy->operation)
             );
