@@ -17,14 +17,55 @@
  * Various actions on modules and sections in the editing mode - hiding, duplicating, deleting, etc.
  *
  * @module     core_course/actions
- * @package    core
  * @copyright  2016 Marina Glancy
  * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
  * @since      3.3
  */
-define(['jquery', 'core/ajax', 'core/templates', 'core/notification', 'core/str', 'core/url', 'core/yui',
-        'core/modal_factory', 'core/modal_events', 'core/key_codes'],
-    function($, ajax, templates, notification, str, url, Y, ModalFactory, ModalEvents, KeyCodes) {
+define(
+    [
+        'jquery',
+        'core/ajax',
+        'core/templates',
+        'core/notification',
+        'core/str',
+        'core/url',
+        'core/yui',
+        'core/modal_factory',
+        'core/modal_events',
+        'core/key_codes',
+        'core/log',
+        'core_courseformat/courseeditor',
+        'core/event_dispatcher',
+        'core_course/events'
+    ],
+    function(
+        $,
+        ajax,
+        templates,
+        notification,
+        str,
+        url,
+        Y,
+        ModalFactory,
+        ModalEvents,
+        KeyCodes,
+        log,
+        editor,
+        EventDispatcher,
+        CourseEvents
+    ) {
+
+        // Eventually, core_courseformat/local/content/actions will handle all actions for
+        // component compatible formats and the default actions.js won't be necessary anymore.
+        // Meanwhile, we filter the migrated actions.
+        const componentActions = ['moveSection', 'moveCm', 'addSection', 'deleteSection'];
+
+        // The course reactive instance.
+        const courseeditor = editor.getCurrentCourseEditor();
+
+        // The current course format name (loaded on init).
+        let formatname;
+
         var CSS = {
             EDITINPROGRESS: 'editinprogress',
             SECTIONDRAGGABLE: 'sectiondraggable',
@@ -38,7 +79,8 @@ define(['jquery', 'core/ajax', 'core/templates', 'core/notification', 'core/str'
             TOGGLE: '.toggle-display,.dropdown-toggle',
             SECTIONLI: 'li.section',
             SECTIONACTIONMENU: '.section_action_menu',
-            ADDSECTIONS: '#changenumsections [data-add-sections]'
+            ADDSECTIONS: '.changenumsections [data-add-sections]',
+            SECTIONBADGES: '[data-region="sectionbadges"]',
         };
 
         Y.use('moodle-course-coursebase', function() {
@@ -48,6 +90,29 @@ define(['jquery', 'core/ajax', 'core/templates', 'core/notification', 'core/str'
             }
         });
 
+        /**
+         * Dispatch event wrapper.
+         *
+         * Old jQuery events will be replaced by native events gradually.
+         *
+         * @method dispatchEvent
+         * @param {String} eventName The name of the event
+         * @param {Object} detail Any additional details to pass into the eveent
+         * @param {Node|HTMLElement} container The point at which to dispatch the event
+         * @param {Object} options
+         * @param {Boolean} options.bubbles Whether to bubble up the DOM
+         * @param {Boolean} options.cancelable Whether preventDefault() can be called
+         * @param {Boolean} options.composed Whether the event can bubble across the ShadowDOM boundary
+         * @returns {CustomEvent}
+         */
+        const dispatchEvent = function(eventName, detail, container, options) {
+            // Most actions still uses jQuery node instead of regular HTMLElement.
+            if (!(container instanceof Element) && container.get !== undefined) {
+                container = container.get(0);
+            }
+            return EventDispatcher.dispatchEvent(eventName, detail, container, options);
+        };
+
         /**
          * Wrapper for Y.Moodle.core_course.util.cm.getId
          *
@@ -55,9 +120,15 @@ define(['jquery', 'core/ajax', 'core/templates', 'core/notification', 'core/str'
          * @returns {Integer}
          */
         var getModuleId = function(element) {
-            var id;
+            // Check if we have a data-id first.
+            const item = element.get(0);
+            if (item.dataset.id) {
+                return item.dataset.id;
+            }
+            // Use YUI way if data-id is not present.
+            let id;
             Y.use('moodle-course-util', function(Y) {
-                id = Y.Moodle.core_course.util.cm.getId(Y.Node(element.get(0)));
+                id = Y.Moodle.core_course.util.cm.getId(Y.Node(item));
             });
             return id;
         };
@@ -73,6 +144,12 @@ define(['jquery', 'core/ajax', 'core/templates', 'core/notification', 'core/str'
             Y.use('moodle-course-util', function(Y) {
                 name = Y.Moodle.core_course.util.cm.getName(Y.Node(element.get(0)));
             });
+            // Check if we have the name in the course state.
+            const state = courseeditor.state;
+            const cmid = getModuleId(element);
+            if (!name && state && cmid) {
+                name = state.cm.get(cmid)?.name;
+            }
             return name;
         };
 
@@ -88,6 +165,10 @@ define(['jquery', 'core/ajax', 'core/templates', 'core/notification', 'core/str'
             if (actionarea) {
                 var spinner = M.util.add_spinner(Y, Y.Node(actionarea));
                 spinner.show();
+                // Lock the activity state element.
+                if (activity.data('id') !== undefined) {
+                    courseeditor.dispatch('cmLock', [activity.data('id')], true);
+                }
                 return spinner;
             }
             return null;
@@ -105,6 +186,10 @@ define(['jquery', 'core/ajax', 'core/templates', 'core/notification', 'core/str'
             if (actionarea) {
                 var spinner = M.util.add_spinner(Y, Y.Node(actionarea));
                 spinner.show();
+                // Lock the section state element.
+                if (sectionelement.data('id') !== undefined) {
+                    courseeditor.dispatch('sectionLock', [sectionelement.data('id')], true);
+                }
                 return spinner;
             }
             return null;
@@ -117,7 +202,13 @@ define(['jquery', 'core/ajax', 'core/templates', 'core/notification', 'core/str'
          * @returns {Node}
          */
         var addSectionLightbox = function(sectionelement) {
-            var lightbox = M.util.add_lightbox(Y, Y.Node(sectionelement.get(0)));
+            const item = sectionelement.get(0);
+            var lightbox = M.util.add_lightbox(Y, Y.Node(item));
+            if (item.dataset.for == 'section' && item.dataset.id) {
+                courseeditor.dispatch('sectionLock', [item.dataset.id], true);
+                lightbox.setAttribute('data-state', 'section');
+                lightbox.setAttribute('data-state-id', item.dataset.id);
+            }
             lightbox.show();
             return lightbox;
         };
@@ -135,6 +226,11 @@ define(['jquery', 'core/ajax', 'core/templates', 'core/notification', 'core/str'
                 if (spinner) {
                     spinner.hide();
                 }
+                // Unlock the state element.
+                if (element.data('id') !== undefined) {
+                    const mutation = (element.data('for') === 'section') ? 'sectionLock' : 'cmLock';
+                    courseeditor.dispatch(mutation, [element.data('id')], false);
+                }
             }, delay);
         };
 
@@ -148,6 +244,14 @@ define(['jquery', 'core/ajax', 'core/templates', 'core/notification', 'core/str'
             if (lightbox) {
                 window.setTimeout(function() {
                     lightbox.hide();
+                    // Unlock state if necessary.
+                    if (lightbox.getAttribute('data-state')) {
+                        courseeditor.dispatch(
+                            `${lightbox.getAttribute('data-state')}Lock`,
+                            [lightbox.getAttribute('data-state-id')],
+                            false
+                        );
+                    }
                 }, delay);
             }
         };
@@ -205,6 +309,7 @@ define(['jquery', 'core/ajax', 'core/templates', 'core/notification', 'core/str'
                     foundElement = this;
                     return false; // Returning false in .each() is equivalent to "break;" inside the loop in php.
                 }
+                return true;
             });
             return foundElement;
         };
@@ -235,6 +340,7 @@ define(['jquery', 'core/ajax', 'core/templates', 'core/notification', 'core/str'
                 .done(function(data) {
                     var elementToFocus = findNextFocusable(moduleElement);
                     moduleElement.replaceWith(data);
+                    let affectedids = [];
                     // Initialise action menu for activity(ies) added as a result of this.
                     $('<div>' + data + '</div>').find(SELECTOR.ACTIVITYLI).each(function(index) {
                         initActionMenu($(this).attr('id'));
@@ -242,6 +348,8 @@ define(['jquery', 'core/ajax', 'core/templates', 'core/notification', 'core/str'
                             focusActionItem($(this).attr('id'), action);
                             elementToFocus = null;
                         }
+                        // Save any activity id in cmids.
+                        affectedids.push(getModuleId($(this)));
                     });
                     // In case of activity deletion focus the next focusable element.
                     if (elementToFocus) {
@@ -252,6 +360,10 @@ define(['jquery', 'core/ajax', 'core/templates', 'core/notification', 'core/str'
                     removeLightbox(lightbox, 400);
                     // Trigger event that can be observed by course formats.
                     moduleElement.trigger($.Event('coursemoduleedited', {ajaxreturn: data, action: action}));
+
+                    // Modify cm state.
+                    courseeditor.dispatch('legacyActivityAction', action, cmid, affectedids);
+
                 }).fail(function(ex) {
                     // Remove spinner and lightbox.
                     removeSpinner(moduleElement, spinner);
@@ -270,24 +382,109 @@ define(['jquery', 'core/ajax', 'core/templates', 'core/notification', 'core/str'
          *
          * Used after d&d of the module to another section
          *
-         * @param {JQuery} activityElement
+         * @param {JQuery|Element} element
          * @param {Number} cmid
          * @param {Number} sectionreturn
+         * @return {Promise} the refresh promise
          */
-        var refreshModule = function(activityElement, cmid, sectionreturn) {
+        var refreshModule = function(element, cmid, sectionreturn) {
+
+            if (sectionreturn === undefined) {
+                sectionreturn = courseeditor.sectionReturn;
+            }
+
+            const activityElement = $(element);
             var spinner = addActivitySpinner(activityElement);
             var promises = ajax.call([{
                 methodname: 'core_course_get_module',
                 args: {id: cmid, sectionreturn: sectionreturn}
             }], true);
 
-            $.when.apply($, promises)
-                .done(function(data) {
-                    removeSpinner(activityElement, spinner, 400);
-                    replaceActivityHtmlWith(data);
-                }).fail(function() {
-                    removeSpinner(activityElement, spinner);
-                });
+            return new Promise((resolve, reject) => {
+                $.when.apply($, promises)
+                    .done(function(data) {
+                        removeSpinner(activityElement, spinner, 400);
+                        replaceActivityHtmlWith(data);
+                        resolve(data);
+                    }).fail(function() {
+                        removeSpinner(activityElement, spinner);
+                        reject();
+                    });
+            });
+        };
+
+        /**
+         * Requests html for the section via WS core_course_edit_section and updates the section on the course page
+         *
+         * @param {JQuery|Element} element
+         * @param {Number} sectionid
+         * @param {Number} sectionreturn
+         * @return {Promise} the refresh promise
+         */
+        var refreshSection = function(element, sectionid, sectionreturn) {
+
+            if (sectionreturn === undefined) {
+                sectionreturn = courseeditor.sectionReturn;
+            }
+
+            const sectionElement = $(element);
+            const action = 'refresh';
+            const promises = ajax.call([{
+                methodname: 'core_course_edit_section',
+                args: {id: sectionid, action, sectionreturn},
+            }], true);
+
+            var spinner = addSectionSpinner(sectionElement);
+            return new Promise((resolve, reject) => {
+                $.when.apply($, promises)
+                    .done(dataencoded => {
+
+                        removeSpinner(sectionElement, spinner);
+                        const data = $.parseJSON(dataencoded);
+
+                        const newSectionElement = $(data.content);
+                        sectionElement.replaceWith(newSectionElement);
+
+                        // Init modules menus.
+                        $(`${SELECTOR.SECTIONLI}#${sectionid} ${SELECTOR.ACTIVITYLI}`).each(
+                            (index, activity) => {
+                                initActionMenu(activity.data('id'));
+                            }
+                        );
+
+                        // Trigger event that can be observed by course formats.
+                        const event = dispatchEvent(
+                            CourseEvents.sectionRefreshed,
+                            {
+                                ajaxreturn: data,
+                                action: action,
+                                newSectionElement: newSectionElement.get(0),
+                            },
+                            newSectionElement
+                        );
+
+                        if (!event.defaultPrevented) {
+                            defaultEditSectionHandler(
+                                newSectionElement, $(SELECTOR.SECTIONLI + '#' + sectionid),
+                                data,
+                                formatname,
+                                sectionid
+                            );
+                        }
+                        resolve(data);
+                    }).fail(ex => {
+                        // Trigger event that can be observed by course formats.
+                        const event = dispatchEvent(
+                            'coursesectionrefreshfailed',
+                            {exception: ex, action: action},
+                            sectionElement
+                        );
+                        if (!event.defaultPrevented) {
+                            notification.exception(ex);
+                        }
+                        reject();
+                    });
+            });
         };
 
         /**
@@ -306,7 +503,7 @@ define(['jquery', 'core/ajax', 'core/templates', 'core/notification', 'core/str'
                     name: modulename
                 };
                 str.get_strings([
-                    {key: 'confirm'},
+                    {key: 'confirm', component: 'core'},
                     {key: modulename === null ? 'deletechecktype' : 'deletechecktypename', param: plugindata},
                     {key: 'yes'},
                     {key: 'no'}
@@ -341,13 +538,11 @@ define(['jquery', 'core/ajax', 'core/templates', 'core/notification', 'core/str'
          * @param {String} image new image name ("i/show", "i/hide", etc.)
          * @param {String} stringname new string for the action menu item
          * @param {String} stringcomponent
-         * @param {String} titlestr not used
-         * @param {String} titlecomponent not used
          * @param {String} newaction new value for data-action attribute of the link
          * @return {Promise} promise which is resolved when the replacement has completed
          */
         var replaceActionItem = function(actionitem, image, stringname,
-                                           stringcomponent, titlestr, titlecomponent, newaction) {
+                                           stringcomponent, newaction) {
 
             var stringRequests = [{key: stringname, component: stringcomponent}];
             // Do not provide an icon with duplicate, different text to the menu item.
@@ -380,18 +575,21 @@ define(['jquery', 'core/ajax', 'core/templates', 'core/notification', 'core/str'
          * @param {JQuery} actionItem
          * @param {Object} data
          * @param {String} courseformat
+         * @param {Number} sectionid
          */
-        var defaultEditSectionHandler = function(sectionElement, actionItem, data, courseformat) {
+        var defaultEditSectionHandler = function(sectionElement, actionItem, data, courseformat, sectionid) {
             var action = actionItem.attr('data-action');
             if (action === 'hide' || action === 'show') {
                 if (action === 'hide') {
                     sectionElement.addClass('hidden');
+                    setSectionBadge(sectionElement[0], 'hiddenfromstudents', true);
                     replaceActionItem(actionItem, 'i/show',
-                        'showfromothers', 'format_' + courseformat, null, null, 'show');
+                        'showfromothers', 'format_' + courseformat, 'show');
                 } else {
+                    setSectionBadge(sectionElement[0], 'hiddenfromstudents', false);
                     sectionElement.removeClass('hidden');
                     replaceActionItem(actionItem, 'i/hide',
-                        'hidefromothers', 'format_' + courseformat, null, null, 'hide');
+                        'hidefromothers', 'format_' + courseformat, 'hide');
                 }
                 // Replace the modules with new html (that indicates that they are now hidden or not hidden).
                 if (data.modules !== undefined) {
@@ -403,22 +601,57 @@ define(['jquery', 'core/ajax', 'core/templates', 'core/notification', 'core/str'
                 if (data.section_availability !== undefined) {
                     sectionElement.find('.section_availability').first().replaceWith(data.section_availability);
                 }
+                // Modify course state.
+                const section = courseeditor.state.section.get(sectionid);
+                if (section !== undefined) {
+                    courseeditor.dispatch('sectionState', [sectionid]);
+                }
             } else if (action === 'setmarker') {
                 var oldmarker = $(SELECTOR.SECTIONLI + '.current'),
                     oldActionItem = oldmarker.find(SELECTOR.SECTIONACTIONMENU + ' ' + 'a[data-action=removemarker]');
                 oldmarker.removeClass('current');
                 replaceActionItem(oldActionItem, 'i/marker',
-                    'highlight', 'core', 'markthistopic', 'core', 'setmarker');
+                    'highlight', 'core', 'setmarker');
                 sectionElement.addClass('current');
                 replaceActionItem(actionItem, 'i/marked',
-                    'highlightoff', 'core', 'markedthistopic', 'core', 'removemarker');
+                    'highlightoff', 'core', 'removemarker');
+                courseeditor.dispatch('legacySectionAction', action, sectionid);
+                setSectionBadge(sectionElement[0], 'iscurrent', true);
             } else if (action === 'removemarker') {
                 sectionElement.removeClass('current');
                 replaceActionItem(actionItem, 'i/marker',
-                    'highlight', 'core', 'markthistopic', 'core', 'setmarker');
+                    'highlight', 'core', 'setmarker');
+                courseeditor.dispatch('legacySectionAction', action, sectionid);
+                setSectionBadge(sectionElement[0], 'iscurrent', false);
             }
         };
 
+        /**
+         * Get the focused element path in an activity if any.
+         *
+         * This method is used to restore focus when the activity HTML is refreshed.
+         * Only the main course editor elements can be refocused as they are always present
+         * even if the activity content changes.
+         *
+         * @param {String} id the element id the activity element
+         * @return {String|undefined} the inner path of the focused element or undefined
+         */
+        const getActivityFocusedElement = function(id) {
+            const element = document.getElementById(id);
+            if (!element || !element.contains(document.activeElement)) {
+                return undefined;
+            }
+            // Check if the actions menu toggler is focused.
+            if (element.querySelector(SELECTOR.ACTIONAREA).contains(document.activeElement)) {
+                return `${SELECTOR.ACTIONAREA} [tabindex="0"]`;
+            }
+            // Return the current element id if any.
+            if (document.activeElement.id) {
+                return `#${document.activeElement.id}`;
+            }
+            return undefined;
+        };
+
         /**
          * Replaces the course module with the new html (used to update module after it was edited or its visibility was changed).
          *
@@ -428,10 +661,18 @@ define(['jquery', 'core/ajax', 'core/templates', 'core/notification', 'core/str'
             $('<div>' + activityHTML + '</div>').find(SELECTOR.ACTIVITYLI).each(function() {
                 // Extract id from the new activity html.
                 var id = $(this).attr('id');
+                // Check if the current focused element is inside the activity.
+                let focusedPath = getActivityFocusedElement(id);
                 // Find the existing element with the same id and replace its contents with new html.
                 $(SELECTOR.ACTIVITYLI + '#' + id).replaceWith(activityHTML);
                 // Initialise action menu.
                 initActionMenu(id);
+                // Re-focus the previous elements.
+                if (focusedPath) {
+                    const newItem = document.getElementById(id);
+                    newItem.querySelector(focusedPath)?.focus();
+                }
+
             });
         };
 
@@ -442,10 +683,17 @@ define(['jquery', 'core/ajax', 'core/templates', 'core/notification', 'core/str'
          * @param {Nunmber} sectionid
          * @param {JQuery} target the element (menu item) that was clicked
          * @param {String} courseformat
+         * @return {boolean} true the action call is sent to the server or false if it is ignored.
          */
         var editSection = function(sectionElement, sectionid, target, courseformat) {
             var action = target.attr('data-action'),
                 sectionreturn = target.attr('data-sectionreturn') ? target.attr('data-sectionreturn') : 0;
+
+            // Filter direct component handled actions.
+            if (courseeditor.supportComponents && componentActions.includes(action)) {
+                return false;
+            }
+
             var spinner = addSectionSpinner(sectionElement);
             var promises = ajax.call([{
                 methodname: 'core_course_edit_section',
@@ -463,7 +711,7 @@ define(['jquery', 'core/ajax', 'core/templates', 'core/notification', 'core/str'
                     var e = $.Event('coursesectionedited', {ajaxreturn: data, action: action});
                     sectionElement.trigger(e);
                     if (!e.isDefaultPrevented()) {
-                        defaultEditSectionHandler(sectionElement, target, data, courseformat);
+                        defaultEditSectionHandler(sectionElement, target, data, courseformat, sectionid);
                     }
                 }).fail(function(ex) {
                     // Remove spinner and lightbox.
@@ -476,6 +724,33 @@ define(['jquery', 'core/ajax', 'core/templates', 'core/notification', 'core/str'
                         notification.exception(ex);
                     }
                 });
+            return true;
+        };
+
+        /**
+         * Sets the section badge in the section header.
+         *
+         * @param {JQuery} sectionElement section element we perform action on
+         * @param {String} badgetype the type of badge this is for
+         * @param {bool} add true to add, false to remove
+         */
+        var setSectionBadge = function(sectionElement, badgetype, add) {
+            const sectionbadges = sectionElement.querySelector(SELECTOR.SECTIONBADGES);
+            if (!sectionbadges) {
+                return;
+            }
+            const badge = sectionbadges.querySelector('[data-type="' + badgetype + '"]');
+            if (!badge) {
+                return;
+            }
+            if (add) {
+                document.querySelectorAll('[data-type="' + badgetype + '"]').forEach((b) => {
+                    b.classList.add('d-none');
+                });
+                badge.classList.remove('d-none');
+            } else {
+                badge.classList.add('d-none');
+            }
         };
 
         // Register a function to be executed after D&D of an activity.
@@ -490,10 +765,136 @@ define(['jquery', 'core/ajax', 'core/templates', 'core/notification', 'core/str'
                         var sectionreturn = mainelement.find('.' + CSS.EDITINGMOVE).attr('data-sectionreturn');
                         refreshModule(mainelement, cmid, sectionreturn);
                     }
-                }
+                },
+                /**
+                 * Update the course state when some cm is moved via YUI.
+                 * @param {*} params
+                 */
+                updateMovedCmState: (params) => {
+                    const state = courseeditor.state;
+
+                    // Update old section.
+                    const cm = state.cm.get(params.cmid);
+                    if (cm !== undefined) {
+                        courseeditor.dispatch('sectionState', [cm.sectionid]);
+                    }
+                    // Update cm state.
+                    courseeditor.dispatch('cmState', [params.cmid]);
+                },
+                /**
+                 * Update the course state when some section is moved via YUI.
+                 */
+                updateMovedSectionState: () => {
+                    courseeditor.dispatch('courseState');
+                },
             });
         });
 
+        // From Moodle 4.0 all edit actions are being re-implemented as state mutation.
+        // This means all method from this "actions" module will be deprecated when all the course
+        // interface is migrated to reactive components.
+        // Most legacy actions did not provide enough information to regenarate the course so they
+        // use the mutations courseState, sectionState and cmState to get the updated state from
+        // the server. However, some activity actions where we can prevent an extra webservice
+        // call by implementing an adhoc mutation.
+        courseeditor.addMutations({
+            /**
+             * Compatibility function to update Moodle 4.0 course state using legacy actions.
+             *
+             * This method only updates some actions which does not require to use cmState mutation
+             * to get updated data form the server.
+             *
+             * @param {Object} statemanager the current state in read write mode
+             * @param {String} action the performed action
+             * @param {Number} cmid the affected course module id
+             * @param {Array} affectedids all affected cm ids (for duplicate action)
+             */
+            legacyActivityAction: function(statemanager, action, cmid, affectedids) {
+
+                const state = statemanager.state;
+                const cm = state.cm.get(cmid);
+                if (cm === undefined) {
+                    return;
+                }
+                const section = state.section.get(cm.sectionid);
+                if (section === undefined) {
+                    return;
+                }
+
+                // Send the element is locked.
+                courseeditor.dispatch('cmLock', [cm.id], true);
+
+                // Now we do the real mutation.
+                statemanager.setReadOnly(false);
+
+                // This unlocked will take effect when the read only is restored.
+                cm.locked = false;
+
+                switch (action) {
+                    case 'delete':
+                        // Remove from section.
+                        section.cmlist = section.cmlist.reduce(
+                            (cmlist, current) => {
+                                if (current != cmid) {
+                                    cmlist.push(current);
+                                }
+                                return cmlist;
+                            },
+                            []
+                        );
+                        // Delete form list.
+                        state.cm.delete(cmid);
+                        break;
+
+                    case 'hide':
+                    case 'show':
+                    case 'duplicate':
+                        courseeditor.dispatch('cmState', affectedids);
+                        break;
+                }
+                statemanager.setReadOnly(true);
+            },
+            legacySectionAction: function(statemanager, action, sectionid) {
+
+                const state = statemanager.state;
+                const section = state.section.get(sectionid);
+                if (section === undefined) {
+                    return;
+                }
+
+                // Send the element is locked. Reactive events are only triggered when the state
+                // read only mode is restored. We want to notify the interface the element is
+                // locked so we need to do a quick lock operation before performing the rest
+                // of the mutation.
+                statemanager.setReadOnly(false);
+                section.locked = true;
+                statemanager.setReadOnly(true);
+
+                // Now we do the real mutation.
+                statemanager.setReadOnly(false);
+
+                // This locked will take effect when the read only is restored.
+                section.locked = false;
+
+                switch (action) {
+                    case 'setmarker':
+                        // Remove previous marker.
+                        state.section.forEach((current) => {
+                            if (current.id != sectionid) {
+                                current.current = false;
+                            }
+                        });
+                        section.current = true;
+                        break;
+
+                    case 'removemarker':
+                        section.current = false;
+                        break;
+                }
+                statemanager.setReadOnly(true);
+            },
+        });
+
         return /** @alias module:core_course/actions */ {
 
             /**
@@ -504,6 +905,8 @@ define(['jquery', 'core/ajax', 'core/templates', 'core/notification', 'core/str'
              */
             initCoursePage: function(courseformat) {
 
+                formatname = courseformat;
+
                 // Add a handler for course module actions.
                 $('body').on('click keypress', SELECTOR.ACTIVITYLI + ' ' +
                         SELECTOR.ACTIVITYACTION + '[data-action]', function(e) {
@@ -554,22 +957,49 @@ define(['jquery', 'core/ajax', 'core/templates', 'core/notification', 'core/str'
                     var actionItem = $(this),
                         sectionElement = actionItem.closest(SELECTOR.SECTIONLI),
                         sectionId = actionItem.closest(SELECTOR.SECTIONACTIONMENU).attr('data-sectionid');
-                    e.preventDefault();
+
+                    let isExecuted = true;
                     if (actionItem.attr('data-confirm')) {
                         // Action requires confirmation.
                         confirmEditSection(actionItem.attr('data-confirm'), function() {
-                            editSection(sectionElement, sectionId, actionItem, courseformat);
+                            isExecuted = editSection(sectionElement, sectionId, actionItem, courseformat);
                         });
                     } else {
-                        editSection(sectionElement, sectionId, actionItem, courseformat);
+                        isExecuted = editSection(sectionElement, sectionId, actionItem, courseformat);
+                    }
+                    // Prevent any other module from capturing the action if it is already in execution.
+                    if (isExecuted) {
+                        e.preventDefault();
                     }
                 });
 
+                // The section and activity names are edited using inplace editable.
+                // The "update" jQuery event must be captured in order to update the course state.
+                $('body').on('updated', `${SELECTOR.SECTIONLI} [data-inplaceeditable]`, function(e) {
+                    if (e.ajaxreturn && e.ajaxreturn.itemid) {
+                        const state = courseeditor.state;
+                        const section = state.section.get(e.ajaxreturn.itemid);
+                        if (section !== undefined) {
+                            courseeditor.dispatch('sectionState', [e.ajaxreturn.itemid]);
+                        }
+                    }
+                });
+                $('body').on('updated', `${SELECTOR.ACTIVITYLI} [data-inplaceeditable]`, function(e) {
+                    if (e.ajaxreturn && e.ajaxreturn.itemid) {
+                        courseeditor.dispatch('cmState', [e.ajaxreturn.itemid]);
+                    }
+                });
+
+                // Component-based formats don't use modals to create sections.
+                if (courseeditor.supportComponents && componentActions.includes('addSection')) {
+                    return;
+                }
+
                 // Add a handler for "Add sections" link to ask for a number of sections to add.
                 str.get_string('numberweeks').done(function(strNumberSections) {
                     var trigger = $(SELECTOR.ADDSECTIONS),
                         modalTitle = trigger.attr('data-add-sections'),
-                        newSections = trigger.attr('new-sections');
+                        newSections = trigger.attr('data-new-sections');
                     var modalBody = $('<div><label for="add_section_numsections"></label> ' +
                         '<input id="add_section_numsections" type="number" min="1" max="' + newSections + '" value="1"></div>');
                     modalBody.find('label').html(strNumberSections);
@@ -610,19 +1040,22 @@ define(['jquery', 'core/ajax', 'core/templates', 'core/notification', 'core/str'
              *
              * This method can be used by course formats in their listener to the coursesectionedited event
              *
+             * @deprecated since Moodle 3.9
              * @param {JQuery} sectionelement
              * @param {String} selector CSS selector inside the section element, for example "a[data-action=show]"
              * @param {String} image new image name ("i/show", "i/hide", etc.)
              * @param {String} stringname new string for the action menu item
              * @param {String} stringcomponent
-             * @param {String} titlestr string for "title" attribute (if different from stringname)
-             * @param {String} titlecomponent
              * @param {String} newaction new value for data-action attribute of the link
              */
             replaceSectionActionItem: function(sectionelement, selector, image, stringname,
-                                                    stringcomponent, titlestr, titlecomponent, newaction) {
+                                                    stringcomponent, newaction) {
+                log.debug('replaceSectionActionItem() is deprecated and will be removed.');
                 var actionitem = sectionelement.find(SELECTOR.SECTIONACTIONMENU + ' ' + selector);
-                replaceActionItem(actionitem, image, stringname, stringcomponent, titlestr, titlecomponent, newaction);
-            }
+                replaceActionItem(actionitem, image, stringname, stringcomponent, newaction);
+            },
+            // Method to refresh a module.
+            refreshModule,
+            refreshSection,
         };
     });
