@@ -27,6 +27,7 @@ require_once($CFG->libdir.'/completionlib.php');
 require_once($CFG->libdir.'/gradelib.php');
 require_once($CFG->libdir.'/plagiarismlib.php');
 
+use core\content\export\exporters\component_exporter;
 use core_grades\component_gradeitems;
 
 /**
@@ -212,7 +213,6 @@ abstract class moodleform_mod extends moodleform {
         $this->_features->showdescription   = plugin_supports('mod', $this->_modname, FEATURE_SHOW_DESCRIPTION, false);
         $this->_features->gradecat          = ($this->_features->outcomes or $this->_features->hasgrades);
         $this->_features->advancedgrading   = plugin_supports('mod', $this->_modname, FEATURE_ADVANCED_GRADING, false);
-        $this->_features->hasnoview         = plugin_supports('mod', $this->_modname, FEATURE_NO_VIEW_LINK, false);
         $this->_features->canrescale = (component_callback_exists('mod_' . $this->_modname, 'rescale_activity_grades') !== false);
     }
 
@@ -371,6 +371,15 @@ abstract class moodleform_mod extends moodleform {
                 if ($mform->elementExists('completionusegrade')) {
                     $mform->freeze('completionusegrade');
                 }
+                if ($mform->elementExists('completionpassgrade')) {
+                    $mform->freeze('completionpassgrade');
+
+                    // Has the completion pass grade completion criteria been set?
+                    // If it has then we shouldn't change the gradepass field.
+                    if ($mform->exportValue('completionpassgrade')) {
+                        $mform->freeze('gradepass');
+                    }
+                }
                 if ($mform->elementExists('completiongradeitemnumber')) {
                     $mform->freeze('completiongradeitemnumber');
                 }
@@ -387,10 +396,11 @@ abstract class moodleform_mod extends moodleform {
     // form verification
     function validation($data, $files) {
         global $COURSE, $DB, $CFG;
+        $errors = parent::validation($data, $files);
 
         $mform =& $this->_form;
 
-        $errors = parent::validation($data, $files);
+        $errors = array();
 
         if ($mform->elementExists('name')) {
             $name = trim($data['name']);
@@ -468,6 +478,28 @@ abstract class moodleform_mod extends moodleform {
                     );
                 }
             }
+
+            if (isset($data['completionpassgrade']) && $data['completionpassgrade']) {
+                // We need to check whether there's a valid gradepass value.
+                // This can either be in completiongradeitemnumber when there are multiple options OR,
+                // The first grade item if completionusegrade is specified.
+                $validategradepass = false;
+                if (isset($data['completiongradeitemnumber'])) {
+                    if ($data['completiongradeitemnumber'] == (string)$itemnumber) {
+                        $validategradepass = true;
+                    }
+                } else if (isset($data['completionusegrade']) && $data['completionusegrade']) {
+                    $validategradepass = true;
+                }
+
+                // Confirm gradepass is a valid non-zero value.
+                if ($validategradepass && (!isset($data[$gradepassfieldname]) || grade_floatval($data[$gradepassfieldname]) == 0)) {
+                    $errors['completionpassgrade'] = get_string(
+                        'activitygradetopassnotset',
+                        'completion'
+                    );
+                }
+            }
         }
 
         // Completion: Don't let them choose automatic completion without turning
@@ -482,7 +514,7 @@ abstract class moodleform_mod extends moodleform {
             $rulesenabled = !empty($data['completionview']);
 
             // Use grade to complete (only one grade item).
-            $rulesenabled = $rulesenabled || !empty($data['completionusegrade']);
+            $rulesenabled = $rulesenabled || !empty($data['completionusegrade']) || !empty($data['completionpassgrade']);
 
             // Use grade to complete (specific grade item).
             if (!$rulesenabled && isset($data['completiongradeitemnumber'])) {
@@ -574,10 +606,7 @@ abstract class moodleform_mod extends moodleform {
         $mform->addElement('header', 'modstandardelshdr', get_string('modstandardels', 'form'));
 
         $section = get_fast_modinfo($COURSE)->get_section_info($this->_section);
-        $allowstealth =
-            !empty($CFG->allowstealth) &&
-            $this->courseformat->allow_stealth_module_visibility($this->_cm, $section) &&
-            !$this->_features->hasnoview;
+        $allowstealth = !empty($CFG->allowstealth) && $this->courseformat->allow_stealth_module_visibility($this->_cm, $section);
         if ($allowstealth && $section->visible) {
             $modvisiblelabel = 'modvisiblewithstealth';
         } else if ($section->visible) {
@@ -609,6 +638,22 @@ abstract class moodleform_mod extends moodleform {
             $mform->addHelpButton('groupmode', 'groupmode', 'group');
         }
 
+        if ($CFG->downloadcoursecontentallowed) {
+                $choices = [
+                    DOWNLOAD_COURSE_CONTENT_DISABLED => get_string('no'),
+                    DOWNLOAD_COURSE_CONTENT_ENABLED => get_string('yes'),
+                ];
+                $mform->addElement('select', 'downloadcontent', get_string('downloadcontent', 'course'), $choices);
+                $downloadcontentdefault = $this->_cm->downloadcontent ?? DOWNLOAD_COURSE_CONTENT_ENABLED;
+                $mform->addHelpButton('downloadcontent', 'downloadcontent', 'course');
+                if (has_capability('moodle/course:configuredownloadcontent', $this->get_context())) {
+                    $mform->setDefault('downloadcontent', $downloadcontentdefault);
+                } else {
+                    $mform->hardFreeze('downloadcontent');
+                    $mform->setConstant('downloadcontent', $downloadcontentdefault);
+                }
+        }
+
         if ($this->_features->groupings) {
             // Groupings selector - used to select grouping for groups in activity.
             $options = array();
@@ -727,16 +772,28 @@ abstract class moodleform_mod extends moodleform {
                     $mform->hideIf('completionusegrade', 'completion', 'ne', COMPLETION_TRACKING_AUTOMATIC);
                     $mform->addHelpButton('completionusegrade', 'completionusegrade', 'completion');
 
+                    // Complete if the user has reached the pass grade.
+                    $mform->addElement(
+                        'checkbox',
+                        'completionpassgrade', null,
+                        get_string('completionpassgrade_desc', 'completion')
+                    );
+                    $mform->hideIf('completionpassgrade', 'completion', 'ne', COMPLETION_TRACKING_AUTOMATIC);
+                    $mform->disabledIf('completionpassgrade', 'completionusegrade', 'notchecked');
+                    $mform->addHelpButton('completionpassgrade', 'completionpassgrade', 'completion');
+
                     // The disabledIf logic differs between ratings and other grade items due to different field types.
                     if ($this->_features->rating) {
                         // If using the rating system, there is no grade unless ratings are enabled.
                         $mform->disabledIf('completionusegrade', 'assessed', 'eq', 0);
+                        $mform->disabledIf('completionpassgrade', 'assessed', 'eq', 0);
                     } else {
                         // All other field types use the '$gradefieldname' field's modgrade_type.
                         $itemnumbers = array_keys($itemnames);
                         $itemnumber = array_shift($itemnumbers);
                         $gradefieldname = component_gradeitems::get_field_name_for_itemnumber($component, $itemnumber, 'grade');
                         $mform->disabledIf('completionusegrade', "{$gradefieldname}[modgrade_type]", 'eq', 'none');
+                        $mform->disabledIf('completionpassgrade', "{$gradefieldname}[modgrade_type]", 'eq', 'none');
                     }
                 } else if (count($itemnames) > 1) {
                     // There are multiple grade items in this activity.
@@ -755,6 +812,16 @@ abstract class moodleform_mod extends moodleform {
                         $options
                     );
                     $mform->hideIf('completiongradeitemnumber', 'completion', 'ne', COMPLETION_TRACKING_AUTOMATIC);
+
+                    // Complete if the user has reached the pass grade.
+                    $mform->addElement(
+                        'checkbox',
+                        'completionpassgrade', null,
+                        get_string('completionpassgrade_desc', 'completion')
+                    );
+                    $mform->hideIf('completionpassgrade', 'completion', 'ne', COMPLETION_TRACKING_AUTOMATIC);
+                    $mform->disabledIf('completionpassgrade', 'completiongradeitemnumber', 'eq', '');
+                    $mform->addHelpButton('completionpassgrade', 'completionpassgrade', 'completion');
                 }
             }
 
@@ -892,9 +959,10 @@ abstract class moodleform_mod extends moodleform {
         }
 
         // Grade to pass.
-        $mform->addElement('float', $gradepassfieldname, get_string('gradepass', 'grades'));
+        $mform->addElement('text', $gradepassfieldname, get_string('gradepass', 'grades'));
         $mform->addHelpButton($gradepassfieldname, 'gradepass', 'grades');
         $mform->setDefault($gradepassfieldname, '');
+        $mform->setType($gradepassfieldname, PARAM_RAW);
         $mform->hideIf($gradepassfieldname, $assessedfieldname, 'eq', '0');
         $mform->hideIf($gradepassfieldname, "{$scalefieldname}[modgrade_type]", 'eq', 'none');
     }
@@ -1068,9 +1136,10 @@ abstract class moodleform_mod extends moodleform {
             }
 
             // Grade to pass.
-            $mform->addElement('float', $gradepassfieldname, get_string($gradepassfieldname, 'grades'));
+            $mform->addElement('text', $gradepassfieldname, get_string($gradepassfieldname, 'grades'));
             $mform->addHelpButton($gradepassfieldname, $gradepassfieldname, 'grades');
             $mform->setDefault($gradepassfieldname, '');
+            $mform->setType($gradepassfieldname, PARAM_RAW);
             $mform->hideIf($gradepassfieldname, "{$gradefieldname}[modgrade_type]", 'eq', 'none');
         }
     }
@@ -1138,6 +1207,10 @@ abstract class moodleform_mod extends moodleform {
 
         $mform = $this->_form;
 
+        $mform->addElement('checkbox', 'coursecontentnotification', get_string('coursecontentnotification', 'course'));
+        $mform->addHelpButton('coursecontentnotification', 'coursecontentnotification', 'course');
+        $mform->closeHeaderBefore('coursecontentnotification');
+
         // elements in a row need a group
         $buttonarray = array();
 
@@ -1157,7 +1230,6 @@ abstract class moodleform_mod extends moodleform {
 
         $mform->addGroup($buttonarray, 'buttonar', '', array(' '), false);
         $mform->setType('buttonar', PARAM_RAW);
-        $mform->closeHeaderBefore('buttonar');
     }
 
     /**
