@@ -22,6 +22,10 @@
  * @license http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
  */
 
+use enrol_lti\data_connector;
+use enrol_lti\local\ltiadvantage\repository\resource_link_repository;
+use IMSGlobal\LTI\ToolProvider\ToolConsumer;
+
 defined('MOODLE_INTERNAL') || die();
 
 /**
@@ -106,6 +110,11 @@ class enrol_lti_plugin extends enrol_plugin {
             $data->$field = $value;
         }
 
+        // LTI Advantage: make a unique identifier for the published resource.
+        if (empty($data->ltiversion) || $data->ltiversion == 'LTI-1p3') {
+            $data->uuid = \core\uuid::generate();
+        }
+
         $DB->insert_record('enrol_lti_tools', $data);
 
         return $instanceid;
@@ -139,6 +148,11 @@ class enrol_lti_plugin extends enrol_plugin {
             $tool->$field = $value;
         }
 
+        // LTI Advantage: make a unique identifier for the published resource.
+        if ($tool->ltiversion == 'LTI-1p3' && empty($tool->uuid)) {
+            $tool->uuid = \core\uuid::generate();
+        }
+
         return $DB->update_record('enrol_lti_tools', $tool);
     }
 
@@ -154,9 +168,28 @@ class enrol_lti_plugin extends enrol_plugin {
         // Get the tool associated with this instance.
         $tool = $DB->get_record('enrol_lti_tools', array('enrolid' => $instance->id), 'id', MUST_EXIST);
 
+        // LTI Advantage: delete any resource_link and user_resource_link mappings.
+        $resourcelinkrepo = new resource_link_repository();
+        $resourcelinkrepo->delete_by_resource($tool->id);
+
         // Delete any users associated with this tool.
         $DB->delete_records('enrol_lti_users', array('toolid' => $tool->id));
 
+        // Get tool and consumer mappings.
+        $rsmapping = $DB->get_recordset('enrol_lti_tool_consumer_map', array('toolid' => $tool->id));
+
+        // Delete consumers that are linked to this tool and their related data.
+        $dataconnector = new data_connector();
+        foreach ($rsmapping as $mapping) {
+            $consumer = new ToolConsumer(null, $dataconnector);
+            $consumer->setRecordId($mapping->consumerid);
+            $dataconnector->deleteToolConsumer($consumer);
+        }
+        $rsmapping->close();
+
+        // Delete mapping records.
+        $DB->delete_records('enrol_lti_tool_consumer_map', array('toolid' => $tool->id));
+
         // Delete the lti tool record.
         $DB->delete_records('enrol_lti_tools', array('id' => $tool->id));
 
@@ -195,6 +228,17 @@ class enrol_lti_plugin extends enrol_plugin {
     public function edit_instance_form($instance, MoodleQuickForm $mform, $context) {
         global $DB;
 
+        $versionoptions = [
+            'LTI-1p3' => get_string('lti13', 'enrol_lti'),
+            'LTI-1p0/LTI-2p0' => get_string('ltilegacy', 'enrol_lti')
+        ];
+        $mform->addElement('select', 'ltiversion', get_string('ltiversion', 'enrol_lti'), $versionoptions);
+        $mform->addHelpButton('ltiversion', 'ltiversion', 'enrol_lti');
+        $legacy = optional_param('legacy', 0, PARAM_INT);
+        if (empty($instance->id)) {
+            $mform->setDefault('ltiversion', $legacy ? 'LTI-1p0/LTI-2p0' : 'LTI-1p3');
+        }
+
         $nameattribs = array('size' => '20', 'maxlength' => '255');
         $mform->addElement('text', 'name', get_string('custominstancename', 'enrol'), $nameattribs);
         $mform->setType('name', PARAM_TEXT);
@@ -241,13 +285,32 @@ class enrol_lti_plugin extends enrol_plugin {
         $mform->setDefault('rolelearner', '5');
         $mform->addHelpButton('rolelearner', 'rolelearner', 'enrol_lti');
 
+        if (!$legacy) {
+            global $CFG;
+            require_once($CFG->dirroot . '/auth/lti/auth.php');
+            $authmodes = [
+                auth_plugin_lti::PROVISIONING_MODE_AUTO_ONLY => get_string('provisioningmodeauto', 'auth_lti'),
+                auth_plugin_lti::PROVISIONING_MODE_PROMPT_NEW_EXISTING => get_string('provisioningmodenewexisting', 'auth_lti'),
+                auth_plugin_lti::PROVISIONING_MODE_PROMPT_EXISTING_ONLY => get_string('provisioningmodeexistingonly', 'auth_lti')
+            ];
+            $mform->addElement('select', 'provisioningmodeinstructor', get_string('provisioningmodeteacherlaunch', 'enrol_lti'),
+                $authmodes);
+            $mform->addHelpButton('provisioningmodeinstructor', 'provisioningmode', 'enrol_lti');
+            $mform->setDefault('provisioningmodeinstructor', auth_plugin_lti::PROVISIONING_MODE_PROMPT_NEW_EXISTING);
+
+            $mform->addElement('select', 'provisioningmodelearner', get_string('provisioningmodestudentlaunch', 'enrol_lti'),
+                $authmodes);
+            $mform->addHelpButton('provisioningmodelearner', 'provisioningmode', 'enrol_lti');
+            $mform->setDefault('provisioningmodelearner', auth_plugin_lti::PROVISIONING_MODE_AUTO_ONLY);
+        }
+
         $mform->addElement('header', 'remotesystem', get_string('remotesystem', 'enrol_lti'));
 
         $mform->addElement('text', 'secret', get_string('secret', 'enrol_lti'), 'maxlength="64" size="25"');
         $mform->setType('secret', PARAM_ALPHANUM);
         $mform->setDefault('secret', random_string(32));
         $mform->addHelpButton('secret', 'secret', 'enrol_lti');
-        $mform->addRule('secret', get_string('required'), 'required');
+        $mform->hideIf('secret', 'ltiversion', 'eq', 'LTI-1p3');
 
         $mform->addElement('selectyesno', 'gradesync', get_string('gradesync', 'enrol_lti'));
         $mform->setDefault('gradesync', 1);
@@ -280,6 +343,7 @@ class enrol_lti_plugin extends enrol_plugin {
         );
         $mform->addElement('select', 'maildisplay', get_string('emaildisplay'), $choices);
         $mform->setDefault('maildisplay', $emaildisplay);
+        $mform->addHelpButton('maildisplay', 'emaildisplay');
 
         $city = get_config('enrol_lti', 'city');
         $mform->addElement('text', 'city', get_string('city'), 'maxlength="100" size="25"');
@@ -318,6 +382,10 @@ class enrol_lti_plugin extends enrol_plugin {
             $mform->setType('toolid', PARAM_INT);
             $mform->setConstant('toolid', $ltitool->id);
 
+            $mform->addElement('hidden', 'uuid');
+            $mform->setType('uuid', PARAM_ALPHANUMEXT);
+            $mform->setConstant('uuid', $ltitool->uuid);
+
             $mform->setDefaults((array) $ltitool);
         }
     }
@@ -338,6 +406,11 @@ class enrol_lti_plugin extends enrol_plugin {
 
         $errors = array();
 
+        // Secret must be set.
+        if (empty($data['secret'])) {
+            $errors['secret'] = get_string('required');
+        }
+
         if (!empty($data['enrolenddate']) && $data['enrolenddate'] < $data['enrolstartdate']) {
             $errors['enrolenddate'] = get_string('enrolenddateerror', 'enrol_lti');
         }
@@ -359,32 +432,6 @@ class enrol_lti_plugin extends enrol_plugin {
         return $errors;
     }
 
-    /**
-     * Gets an array of the user enrolment actions.
-     *
-     * @param course_enrolment_manager $manager
-     * @param stdClass $ue A user enrolment object
-     * @return array An array of user_enrolment_actions
-     */
-    public function get_user_enrolment_actions(course_enrolment_manager $manager, $ue) {
-        $actions = array();
-        $context = $manager->get_context();
-        $instance = $ue->enrolmentinstance;
-        $params = $manager->get_moodlepage()->url->params();
-        $params['ue'] = $ue->id;
-        if ($this->allow_unenrol_user($instance, $ue) && has_capability("enrol/lti:unenrol", $context)) {
-            $url = new moodle_url('/enrol/unenroluser.php', $params);
-            $actions[] = new user_enrolment_action(new pix_icon('t/delete', ''), get_string('unenrol', 'enrol'), $url,
-                array('class' => 'unenrollink', 'rel' => $ue->id));
-        }
-        if ($this->allow_manage($instance) && has_capability("enrol/lti:manage", $context)) {
-            $url = new moodle_url('/enrol/editenrolment.php', $params);
-            $actions[] = new user_enrolment_action(new pix_icon('t/edit', ''), get_string('edit'), $url,
-                array('class' => 'editenrollink', 'rel' => $ue->id));
-        }
-        return $actions;
-    }
-
     /**
      * Restore instance and map settings.
      *
@@ -414,11 +461,20 @@ function enrol_lti_extend_navigation_course($navigation, $course, $context) {
         // Check that they can add an instance.
         $ltiplugin = enrol_get_plugin('lti');
         if ($ltiplugin->can_add_instance($course->id)) {
-            $url = new moodle_url('/enrol/lti/index.php', array('courseid' => $course->id));
+            $url = new moodle_url('/enrol/lti/index.php', ['courseid' => $course->id]);
             $settingsnode = navigation_node::create(get_string('sharedexternaltools', 'enrol_lti'), $url,
-                navigation_node::TYPE_SETTING, null, null, new pix_icon('i/settings', ''));
-
+                navigation_node::TYPE_SETTING, null, 'publishedtools', new pix_icon('i/settings', ''));
             $navigation->add_node($settingsnode);
         }
     }
 }
+
+/**
+ * Get icon mapping for font-awesome.
+ */
+function enrol_lti_get_fontawesome_icon_map() {
+    return [
+        'enrol_lti:managedeployments' => 'fa-sitemap',
+        'enrol_lti:platformdetails' => 'fa-pencil-square-o'
+    ];
+}
