@@ -552,6 +552,7 @@ class externallib_test extends externallib_advanced_testcase {
                 $this->assertTrue($courseenrol['hidden']);
                 $this->assertTrue($courseenrol['isfavourite']);
                 $this->assertEquals(2, $courseenrol['enrolledusercount']);
+                $this->assertEquals($course1->timemodified, $courseenrol['timemodified']);
             } else {
                 // Check language pack. Should be empty since an incorrect one was used when creating the course.
                 $this->assertEmpty($courseenrol['lang']);
@@ -565,6 +566,7 @@ class externallib_test extends externallib_advanced_testcase {
                 $this->assertFalse($courseenrol['hidden']);
                 $this->assertFalse($courseenrol['isfavourite']);
                 $this->assertEquals(1, $courseenrol['enrolledusercount']);
+                $this->assertEquals($course2->timemodified, $courseenrol['timemodified']);
             }
         }
 
@@ -1099,89 +1101,6 @@ class externallib_test extends externallib_advanced_testcase {
         $this->assertEquals($timenow, $expecteduserlist['users'][1]['lastcourseaccess']);
     }
 
-    /**
-     * Test for core_enrol_external::edit_user_enrolment().
-     */
-    public function test_edit_user_enrolment() {
-        global $DB;
-
-        $this->resetAfterTest(true);
-        $datagen = $this->getDataGenerator();
-
-        /** @var enrol_manual_plugin $manualplugin */
-        $manualplugin = enrol_get_plugin('manual');
-        $this->assertNotNull($manualplugin);
-
-        $studentroleid = $DB->get_field('role', 'id', ['shortname' => 'student'], MUST_EXIST);
-        $teacherroleid = $DB->get_field('role', 'id', ['shortname' => 'editingteacher'], MUST_EXIST);
-        $course = $datagen->create_course();
-        $user = $datagen->create_user();
-        $teacher = $datagen->create_user();
-
-        $instanceid = null;
-        $instances = enrol_get_instances($course->id, true);
-        foreach ($instances as $inst) {
-            if ($inst->enrol == 'manual') {
-                $instanceid = (int)$inst->id;
-                break;
-            }
-        }
-        if (empty($instanceid)) {
-            $instanceid = $manualplugin->add_default_instance($course);
-            if (empty($instanceid)) {
-                $instanceid = $manualplugin->add_instance($course);
-            }
-        }
-        $this->assertNotNull($instanceid);
-
-        $instance = $DB->get_record('enrol', ['id' => $instanceid], '*', MUST_EXIST);
-        $manualplugin->enrol_user($instance, $user->id, $studentroleid, 0, 0, ENROL_USER_ACTIVE);
-        $manualplugin->enrol_user($instance, $teacher->id, $teacherroleid, 0, 0, ENROL_USER_ACTIVE);
-        $ueid = (int)$DB->get_field(
-            'user_enrolments',
-            'id',
-            ['enrolid' => $instance->id, 'userid' => $user->id],
-            MUST_EXIST
-        );
-
-        // Login as teacher.
-        $this->setUser($teacher);
-
-        $now = new \DateTime();
-        $nowtime = $now->getTimestamp();
-
-        // Invalid data.
-        $data = core_enrol_external::edit_user_enrolment($course->id, $ueid, ENROL_USER_ACTIVE, $nowtime, $nowtime);
-        $data = \external_api::clean_returnvalue(core_enrol_external::edit_user_enrolment_returns(), $data);
-        $this->assertFalse($data['result']);
-        $this->assertNotEmpty($data['errors']);
-
-        // Valid data.
-        $nextmonth = clone($now);
-        $nextmonth->add(new \DateInterval('P1M'));
-        $nextmonthtime = $nextmonth->getTimestamp();
-        $data = core_enrol_external::edit_user_enrolment($course->id, $ueid, ENROL_USER_ACTIVE, $nowtime, $nextmonthtime);
-        $data = \external_api::clean_returnvalue(core_enrol_external::edit_user_enrolment_returns(), $data);
-        $this->assertTrue($data['result']);
-        $this->assertEmpty($data['errors']);
-
-        // Check updated user enrolment.
-        $ue = $DB->get_record('user_enrolments', ['id' => $ueid], '*', MUST_EXIST);
-        $this->assertEquals(ENROL_USER_ACTIVE, $ue->status);
-        $this->assertEquals($nowtime, $ue->timestart);
-        $this->assertEquals($nextmonthtime, $ue->timeend);
-
-        // Suspend user.
-        $data = core_enrol_external::edit_user_enrolment($course->id, $ueid, ENROL_USER_SUSPENDED);
-        $data = \external_api::clean_returnvalue(core_enrol_external::edit_user_enrolment_returns(), $data);
-        $this->assertTrue($data['result']);
-        $this->assertEmpty($data['errors']);
-
-        // Check updated user enrolment.
-        $ue = $DB->get_record('user_enrolments', ['id' => $ueid], '*', MUST_EXIST);
-        $this->assertEquals(ENROL_USER_SUSPENDED, $ue->status);
-    }
-
     /**
      * dataProvider for test_submit_user_enrolment_form().
      */
