@@ -75,6 +75,37 @@ class behat_grade extends behat_base {
 
         $gradeitem = behat_context_helper::escape($gradeitem);
 
+        if ($this->running_javascript()) {
+            $xpath = "//tr[contains(.,$gradeitem)]//*[contains(@class,'moodle-actionmenu')]";
+            if ($this->getSession()->getPage()->findAll('xpath', $xpath)) {
+                $this->execute("behat_action_menu::i_open_the_action_menu_in",
+                        array("//tr[contains(.,$gradeitem)]",
+                                "xpath_element"));
+            }
+        }
+
+        $savechanges = get_string('savechanges', 'grades');
+        $edit = behat_context_helper::escape(get_string('edit') . '  ');
+        $linkxpath = "//a[./*[contains(concat(' ', normalize-space(@class), ' '), ' icon ') " .
+                "and starts-with(@title,$edit) and contains(@title,$gradeitem)]]";
+
+        $this->execute("behat_general::i_click_on", array($this->escape($linkxpath), "xpath_element"));
+        $this->execute("behat_forms::i_set_the_following_fields_to_these_values", $data);
+        $this->execute('behat_forms::press_button', $this->escape($savechanges));
+    }
+
+    /**
+     * Hids a grade item or category.
+     *
+     * Teacher must be on the grade setup page.
+     *
+     * @Given /^I hide the grade item "(?P<grade_item_string>(?:[^"]|\\")*)"$/
+     * @param string $gradeitem
+     */
+    public function i_hide_the_grade_item($gradeitem) {
+
+        $gradeitem = behat_context_helper::escape($gradeitem);
+
         if ($this->running_javascript()) {
             $xpath = "//tr[contains(.,$gradeitem)]//*[contains(@class,'moodle-actionmenu')]//a[contains(@class,'toggle-display')]";
             if ($this->getSession()->getPage()->findAll('xpath', $xpath)) {
@@ -82,13 +113,31 @@ class behat_grade extends behat_base {
             }
         }
 
-        $savechanges = get_string('savechanges', 'grades');
-        $edit = behat_context_helper::escape(get_string('edit') . '  ');
-        $linkxpath = "//a[./img[starts-with(@title,$edit) and contains(@title,$gradeitem)]]";
+        $this->execute("behat_general::i_click_on_in_the", array(get_string('hide'), 'link',
+            "//tr[descendant::*[text() = " . $this->escape($gradeitem) . "]]", 'xpath_element'));
+    }
 
-        $this->execute("behat_general::i_click_on", array($this->escape($linkxpath), "xpath_element"));
-        $this->execute("behat_forms::i_set_the_following_fields_to_these_values", $data);
-        $this->execute('behat_forms::press_button', $this->escape($savechanges));
+    /**
+     * Duplicates a grade item or category.
+     *
+     * Teacher must be on the grade setup page.
+     *
+     * @Given /^I duplicate the grade item "(?P<grade_item_string>(?:[^"]|\\")*)"$/
+     * @param string $gradeitem
+     */
+    public function i_duplicate_the_grade_item($gradeitem) {
+
+        $gradeitem = behat_context_helper::escape($gradeitem);
+
+        if ($this->running_javascript()) {
+            $xpath = "//tr[contains(.,$gradeitem)]//*[contains(@class,'moodle-actionmenu')]//a[contains(@class,'toggle-display')]";
+            if ($this->getSession()->getPage()->findAll('xpath', $xpath)) {
+                $this->execute("behat_general::i_click_on", array($this->escape($xpath), "xpath_element"));
+            }
+        }
+
+        $this->execute("behat_general::i_click_on_in_the", array(get_string('duplicate'), 'link',
+            "//tr[descendant::*[text() = " . $this->escape($gradeitem) . "]]", 'xpath_element'));
     }
 
     /**
@@ -105,16 +154,19 @@ class behat_grade extends behat_base {
         $gradeitem = behat_context_helper::escape($gradeitem);
 
         if ($this->running_javascript()) {
-            $xpath = "//tr[contains(.,$gradeitem)]//*[contains(@class,'moodle-actionmenu')]//a[contains(@class,'toggle-display')]";
+            $xpath = "//tr[contains(.,$gradeitem)]//*[contains(@class,'moodle-actionmenu')]";
             if ($this->getSession()->getPage()->findAll('xpath', $xpath)) {
-                $this->execute("behat_general::i_click_on", array($this->escape($xpath), "xpath_element"));
+                $this->execute("behat_action_menu::i_open_the_action_menu_in",
+                        array("//tr[contains(.,$gradeitem)]",
+                                "xpath_element"));
             }
         }
 
         // Going to edit calculation.
         $savechanges = get_string('savechanges', 'grades');
         $edit = behat_context_helper::escape(get_string('editcalculation', 'grades'));
-        $linkxpath = "//a[./img[starts-with(@title,$edit) and contains(@title,$gradeitem)]]";
+        $linkxpath = "//a[./*[contains(concat(' ', normalize-space(@class), ' '), ' icon ') " .
+                "and starts-with(@title,$edit) and contains(@title,$gradeitem)]]";
         $this->execute("behat_general::i_click_on", array($this->escape($linkxpath), "xpath_element"));
 
         // Mapping names to idnumbers.
@@ -122,11 +174,12 @@ class behat_grade extends behat_base {
         foreach ($datahash as $gradeitem => $idnumber) {
             // This xpath looks for course, categories and items with the provided name.
             // Grrr, we can't equal in categoryitem and courseitem because there is a line jump...
-            $inputxpath ="//input[@class='idnumber'][" .
-                "parent::li[@class='item'][text()='" . $gradeitem . "']" .
-                " or " .
-                "parent::li[@class='categoryitem' or @class='courseitem']/parent::ul/parent::li[starts-with(text(),'" . $gradeitem . "')]" .
-            "]";
+            $inputxpath = "//input[@class='idnumber'][" .
+                    "parent::li[@class='item'][text()='" . $gradeitem . "']" .
+                    " or " .
+                    "parent::li[@class='categoryitem' or @class='courseitem']" .
+                    "/parent::ul/parent::li[starts-with(text(),'" . $gradeitem . "')]" .
+                    "]";
             $this->execute('behat_forms::i_set_the_field_with_xpath_to', array($inputxpath, $idnumber));
         }
 
@@ -151,17 +204,18 @@ class behat_grade extends behat_base {
         $gradeitem = behat_context_helper::escape($gradeitem);
 
         if ($this->running_javascript()) {
-            $xpath = "//tr[contains(.,$gradecategorytotal)]//*[contains(@class,'moodle-actionmenu')]" .
-                "//a[contains(@class,'toggle-display')]";
+            $xpath = "//tr[contains(.,$gradecategorytotal)]//*[contains(@class,'moodle-actionmenu')]";
             if ($this->getSession()->getPage()->findAll('xpath', $xpath)) {
-                $this->execute("behat_general::i_click_on", array($this->escape($xpath), "xpath_element"));
+                $xpath = "//tr[contains(.,$gradecategorytotal)]";
+                $this->execute("behat_action_menu::i_open_the_action_menu_in", array($xpath, "xpath_element"));
             }
         }
 
         // Going to edit calculation.
         $savechanges = get_string('savechanges', 'grades');
         $edit = behat_context_helper::escape(get_string('editcalculation', 'grades'));
-        $linkxpath = "//a[./img[starts-with(@title,$edit) and contains(@title,$gradeitem)]]";
+        $linkxpath = "//a[./*[contains(concat(' ', normalize-space(@class), ' '), ' icon ') " .
+                "and starts-with(@title,$edit) and contains(@title,$gradeitem)]]";
         $this->execute("behat_general::i_click_on", array($this->escape($linkxpath), "xpath_element"));
 
         // Mapping names to idnumbers.
@@ -170,11 +224,11 @@ class behat_grade extends behat_base {
             // This xpath looks for course, categories and items with the provided name.
             // Grrr, we can't equal in categoryitem and courseitem because there is a line jump...
             $inputxpath = "//input[@class='idnumber'][" .
-                "parent::li[@class='item'][text()='" . $gradeitem . "']" .
-                " | " .
-                "parent::li[@class='categoryitem' | @class='courseitem']" .
-                "/parent::ul/parent::li[starts-with(text(),'" . $gradeitem . "')]" .
-            "]";
+                    "parent::li[@class='item'][text()='" . $gradeitem . "']" .
+                    " | " .
+                    "parent::li[@class='categoryitem' or @class='courseitem']" .
+                    "/parent::ul/parent::li[starts-with(text(),'" . $gradeitem . "')]" .
+                    "]";
             $this->execute('behat_forms::i_set_the_field_with_xpath_to', array($inputxpath, $idnumber));
         }
 
@@ -198,9 +252,10 @@ class behat_grade extends behat_base {
 
         if ($this->running_javascript()) {
             $gradeitemliteral = behat_context_helper::escape($gradeitem);
-            $xpath = "//tr[contains(.,$gradeitemliteral)]//*[contains(@class,'moodle-actionmenu')]//a[contains(@class,'toggle-display')]";
+            $xpath = "//tr[contains(.,$gradeitemliteral)]//*[contains(@class,'moodle-actionmenu')]";
             if ($this->getSession()->getPage()->findAll('xpath', $xpath)) {
-                $this->execute("behat_general::i_click_on", array($this->escape($xpath), "xpath_element"));
+                $xpath = "//tr[contains(.,$gradeitemliteral)]";
+                $this->execute("behat_action_menu::i_open_the_action_menu_in", array($xpath, "xpath_element"));
             }
         }
 
@@ -222,48 +277,144 @@ class behat_grade extends behat_base {
     }
 
     /**
-     * Navigates to the course gradebook and selects a specified item from the grade navigation tabs.
+     * Select the tab in the gradebook. We must be on one of the gradebook pages already.
+     *
+     * @deprecated since 4.0 - use select_in_gradebook_navigation_selector() instead.
+     * @param string $gradepath examples: "View > User report", "Letters > View", "Scales"
+     */
+    protected function select_in_gradebook_tabs($gradepath) {
+        debugging('The function select_in_gradebook_tabs() is deprecated, please use ' .
+            'select_in_gradebook_navigation_selector() instead.', DEBUG_DEVELOPER);
+
+        $gradepath = preg_split('/\s*>\s*/', trim($gradepath));
+        if (count($gradepath) > 2) {
+            throw new coding_exception('Grade path is too long (must have no more than two items separated with ">")');
+        }
+
+        $xpath = '//div[contains(@class,\'grade-navigation\')]';
+
+        // If the first row of the grade-navigation tabs does not have $gradepath[0] as active tab, click on it.
+        $link = '\'' . $this->escape($gradepath[0]) . '\'';
+        $xpathrow1 = $xpath . '//ul[1]//*[contains(@class,\'active\') and contains(normalize-space(.), ' . $link . ')]';
+        if (!$this->getSession()->getPage()->findAll('xpath', $xpathrow1)) {
+            $this->find('xpath', $xpath . '//ul[1]/li/a[text()=' . $link . ']')->click();
+            $this->wait_for_pending_js();
+        }
+
+        if (isset($gradepath[1])) {
+            // If the second row of the grade-navigation tabs does not have $gradepath[1] as active tab, click on it.
+            $link = '\'' . $this->escape($gradepath[1]) . '\'';
+            $xpathrow2 = $xpath . '//ul[2]//*[contains(@class,\'active\') and contains(normalize-space(.), ' . $link . ')]';
+            if (!$this->getSession()->getPage()->findAll('xpath', $xpathrow2)) {
+                $this->find('xpath', $xpath . '//ul[2]/li/a[text()=' . $link . ']')->click();
+                $this->wait_for_pending_js();
+            }
+        }
+    }
+
+    /**
+     * Navigates to the course gradebook and selects the specified item from the general grade navigation selector.
      *
      * Examples:
      * - I navigate to "Setup > Gradebook setup" in the course gradebook
      * - I navigate to "Scales" in the course gradebook
-     * - I navigate to "Letters > View" in the course gradebook
-     * - I navigate to "View > User report" in the course gradebook // for teachers
-     * - I navigate to "User report" in the course gradebook // for students
+     * - I navigate to "More > Grade letters" in the course gradebook
      *
      * @Given /^I navigate to "(?P<gradepath_string>(?:[^"]|\\")*)" in the course gradebook$/
-     * @param string $gradepath
+     * @param string $gradepath The path string. If the path has two items (ex. "More > Grade letters"), the first item
+     *                          ("More") will be used to identify an option group in the navigation selector, while the
+     *                          second ("Grade letters") will be used to identify an option within that option group.
+     *                          Otherwise, a single item in a path (ex. "Scales") will be used to identify an option in
+     *                          the navigation selector regardless of the option group.
      */
     public function i_navigate_to_in_the_course_gradebook($gradepath) {
-        $gradeadmin = get_string('gradeadministration', 'grades');
-
-        // If we are not on one of the gradebook pages already, follow "Grades" link in the administration block.
-        $xpath = '//div[contains(@class,\'block_settings\')]//div[@id=\'settingsnav\']/ul/li[1]/p[1]/span[string(.)=' .
-            behat_context_helper::escape($gradeadmin) . ']';
+        // If we are not on one of the gradebook pages already, follow "Grades" link in the navigation drawer.
+        $xpath = '//div[contains(@class,\'grade-navigation\')]';
         if (!$this->getSession()->getPage()->findAll('xpath', $xpath)) {
-            $this->execute("behat_general::i_click_on_in_the", array(get_string('grades'), 'link',
-                get_string('pluginname', 'block_settings'), 'block'));
+            $this->execute('behat_navigation::i_select_from_secondary_navigation', get_string('grades'));
         }
 
-        $parentnodes = preg_split('/\s*>\s*/', trim($gradepath));
-        if ($parentnodes[0] === 'Letters' && count($parentnodes) > 1) {
-            // Make Letters navigation steps compatible with tabs in Moodle 3.2.
-            if ($parentnodes[1] === 'Edit') {
-                $this->execute("behat_navigation::i_navigate_to_node_in", [$parentnodes[0], $gradeadmin]);
-                $this->execute("behat_general::click_link", "Edit grade letters");
-                return;
-            } else {
-                array_pop($parentnodes);
-            }
+        $this->select_in_gradebook_navigation_selector($gradepath, 'gradesactionselect');
+    }
+
+    /**
+     * Navigates to the imports page in the course gradebook and selects the specified import type from the grade
+     * imports navigation selector.
+     *
+     * Examples:
+     * - I navigate to "CSV file" import page in the course gradebook
+     *
+     * @Given /^I navigate to "(?P<importoption_string>(?:[^"]|\\")*)" import page in the course gradebook$/
+     * @param string $gradeimportoption The name of an existing grade import option.
+     */
+    public function i_navigate_to_import_page_in_the_course_gradebook($gradeimportoption) {
+        $this->i_navigate_to_in_the_course_gradebook("More > Import");
+        $this->select_in_gradebook_navigation_selector($gradeimportoption, 'gradesimportactionselect');
+    }
+
+    /**
+     * Navigates to the exports page in the course gradebook and selects the specified export type from the grade
+     * exports navigation selector.
+     *
+     * Examples:
+     * - I navigate to "XML file" export page in the course gradebook
+     *
+     * @Given /^I navigate to "(?P<exportoption_string>(?:[^"]|\\")*)" export page in the course gradebook$/
+     * @param string $gradeexportoption The name of an existing grade export option.
+     */
+    public function i_navigate_to_export_page_in_the_course_gradebook($gradeexportoption) {
+        $this->i_navigate_to_in_the_course_gradebook("More > Export");
+        $this->select_in_gradebook_navigation_selector($gradeexportoption, 'gradesexportactionselect');
+    }
+
+    /**
+     * Select a given option from a navigation URL selector in the gradebook. We must be on one of the gradebook pages
+     * already.
+     *
+     * @param string $path The string path that is used to identify an item within the navigation selector. If the path
+     *                     has two items (ex. "More > Grade letters"), the first item ("More") will be used to identify
+     *                     an option group in the navigation selector, while the second ("Grade letters") will be used to
+     *                     identify an option within that option group. Otherwise, a single item in a path (ex. "Scales")
+     *                     will be used to identify an option in the navigation selector regardless of the option group.
+     * @param string $formid The ID of the form element which contains the navigation URL selector element.
+     */
+    protected function select_in_gradebook_navigation_selector(string $path, string $formid) {
+        // Split the path string by ">".
+        $path = preg_split('/\s*>\s*/', trim($path));
+
+        // Make sure that the path does not have more than two items separated with ">".
+        if (count($path) > 2) {
+            throw new coding_exception('The path is too long (must have no more than two items separated with ">")');
         }
 
-        $lastitem = array_pop($parentnodes);
-        if ($parentnodes && $parentnodes[0] === 'View') {
-            // When viewing reports (grader, user, single view), ignore the "View" in the path,
-            // it is only needed in Moodle 3.2 gradebook navigation.
-            array_shift($parentnodes);
+        // Get the select element.
+        $selectxpath = "//form[contains(@id,'{$formid}')]//select";
+        $select = $this->find('xpath', $selectxpath);
+
+        // Define the xpath to the option element depending on the provided path.
+        // If two items are provided in the path, the first item will be considered as an identifier of an existing
+        // option group in the select select element, while the second item will identify an existing option within
+        // that option group.
+        // If one item is provided in the path, this item will identify any existing option in the select element
+        // regardless of the option group. Also, this is useful when option elements are not a part of an option group
+        // which is possible.
+        if (count($path) === 2) {
+            $optionxpath = $selectxpath . '/optgroup[@label="' . $this->escape($path[0]) . '"]' .
+                '/option[contains(.,"' . $this->escape($path[1]) . '")]';
+        } else {
+            $optionxpath = $selectxpath . '//option[contains(.,"' . $this->escape($path[0]) . '")]';
+        }
+
+        // Get the option element that we are looking to select.
+        $option = $this->find('xpath', $optionxpath);
+
+        // Select the given option in the select element.
+        $field = behat_field_manager::get_field_instance('select', $select, $this->getSession());
+        $field->set_value($this->escape($option->getValue()));
+
+        if (!$this->running_javascript()) {
+            $this->execute('behat_general::i_click_on_in_the', [get_string('go'), 'button',
+                "#{$formid}", 'css_element']);
         }
-        array_unshift($parentnodes, $gradeadmin);
-        $this->execute("behat_navigation::i_navigate_to_node_in", [$lastitem, join('>', $parentnodes)]);
     }
 }
