@@ -14,10 +14,14 @@
 // You should have received a copy of the GNU General Public License
 // along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
 
-namespace core_grades;
-
-use grade_plugin_return;
-use grade_report_grader;
+/**
+ * Unit tests for grade/report/user/lib.php.
+ *
+ * @package  core_grades
+ * @category phpunit
+ * @copyright 2012 Andrew Davis
+ * @license  http://www.gnu.org/copyleft/gpl.html GNU Public License
+ */
 
 defined('MOODLE_INTERNAL') || die();
 
@@ -27,13 +31,8 @@ require_once($CFG->dirroot.'/grade/report/grader/lib.php');
 
 /**
  * Tests grade_report_grader (the grader report)
- *
- * @package  core_grades
- * @category test
- * @copyright 2012 Andrew Davis
- * @license  http://www.gnu.org/copyleft/gpl.html GNU Public License
  */
-class report_graderlib_test extends \advanced_testcase {
+class core_grade_report_graderlib_testcase extends advanced_testcase {
 
     /**
      * Tests grade_report_grader::process_data()
@@ -58,12 +57,12 @@ class report_graderlib_test extends \advanced_testcase {
         $forummax = 80;
         $forum1 = $this->getDataGenerator()->create_module('forum', array('assessed' => 1, 'scale' => $forummax, 'course' => $course->id));
         // Switch the stdClass instance for a grade item instance.
-        $forum1 = \grade_item::fetch(array('itemtype' => 'mod', 'itemmodule' => 'forum', 'iteminstance' => $forum1->id, 'courseid' => $course->id));
+        $forum1 = grade_item::fetch(array('itemtype' => 'mod', 'itemmodule' => 'forum', 'iteminstance' => $forum1->id, 'courseid' => $course->id));
 
         $report = $this->create_report($course);
         $testgrade = 60.00;
 
-        $data = new \stdClass();
+        $data = new stdClass();
         $data->id = $course->id;
         $data->report = 'grader';
         $data->timepageload = time();
@@ -75,7 +74,7 @@ class report_graderlib_test extends \advanced_testcase {
         $warnings = $report->process_data($data);
         $this->assertEquals(count($warnings), 0);
 
-        $studentgrade = \grade_grade::fetch(array('itemid' => $forum1->id, '' => $student->id));
+        $studentgrade = grade_grade::fetch(array('itemid' => $forum1->id, '' => $student->id));
         $this->assertEquals($studentgrade->finalgrade, $testgrade);
 
         // Grade above max. Should be pulled down to max.
@@ -85,7 +84,7 @@ class report_graderlib_test extends \advanced_testcase {
         $warnings = $report->process_data($data);
         $this->assertEquals(count($warnings), 1);
 
-        $studentgrade = \grade_grade::fetch(array('itemid' => $forum1->id, '' => $student->id));
+        $studentgrade = grade_grade::fetch(array('itemid' => $forum1->id, '' => $student->id));
         $this->assertEquals($studentgrade->finalgrade, $forummax);
 
         // Grade below min. Should be pulled up to min.
@@ -95,7 +94,7 @@ class report_graderlib_test extends \advanced_testcase {
         $warnings = $report->process_data($data);
         $this->assertEquals(count($warnings), 1);
 
-        $studentgrade = \grade_grade::fetch(array('itemid' => $forum1->id, '' => $student->id));
+        $studentgrade = grade_grade::fetch(array('itemid' => $forum1->id, '' => $student->id));
         $this->assertEquals($studentgrade->finalgrade, 0);
 
         // Test unlimited grades so we can give a student a grade about max.
@@ -106,7 +105,7 @@ class report_graderlib_test extends \advanced_testcase {
         $warnings = $report->process_data($data);
         $this->assertEquals(count($warnings), 0);
 
-        $studentgrade = \grade_grade::fetch(array('itemid' => $forum1->id, '' => $student->id));
+        $studentgrade = grade_grade::fetch(array('itemid' => $forum1->id, '' => $student->id));
         $this->assertEquals($studentgrade->finalgrade, $toobig);
     }
 
@@ -482,7 +481,7 @@ class report_graderlib_test extends \advanced_testcase {
 
         // Set current user.
         $this->setUser($manager);
-        $USER->gradeediting[$course->id] = false;
+        $USER->editing = false;
 
         // Get the report.
         $report = $this->create_report($course);
@@ -505,7 +504,7 @@ class report_graderlib_test extends \advanced_testcase {
 
         // Supposing the user cannot view hidden grades, this shouldn't make any difference (due
         // to a bug, it previously did).
-        $context = \context_course::instance($course->id);
+        $context = context_course::instance($course->id);
         $managerroleid = $DB->get_field('role', 'id', array('shortname' => 'manager'));
         assign_capability('moodle/grade:viewhidden', CAP_PROHIBIT, $managerroleid, $context->id, true);
         $this->assertFalse(has_capability('moodle/grade:viewhidden', $context));
@@ -521,16 +520,16 @@ class report_graderlib_test extends \advanced_testcase {
     private function create_grade_category($course) {
         static $cnt = 0;
         $cnt++;
-        $gradecat = new \grade_category(array('courseid' => $course->id, 'fullname' => 'Cat '.$cnt), false);
-        $gradecat->apply_default_settings();
-        $gradecat->apply_forced_settings();
-        $gradecat->insert();
-        return $gradecat;
+        $grade_category = new grade_category(array('courseid' => $course->id, 'fullname' => 'Cat '.$cnt), false);
+        $grade_category->apply_default_settings();
+        $grade_category->apply_forced_settings();
+        $grade_category->insert();
+        return $grade_category;
     }
 
     private function create_report($course) {
 
-        $coursecontext = \context_course::instance($course->id);
+        $coursecontext = context_course::instance($course->id);
         $gpr = new grade_plugin_return(array('type' => 'report', 'plugin'=>'grader', 'courseid' => $course->id));
         $report = new grade_report_grader($course->id, $gpr, $coursecontext);
 
