@@ -28,21 +28,25 @@ $string['addevent'] = 'Add events';
 $string['annually'] = 'Annually';
 $string['activityevent'] = 'Activity event';
 $string['calendar'] = 'Calendar';
+$string['calendarexporturl'] = 'Calendar URL';
 $string['calendarheading'] = '{$a} Calendar';
 $string['calendarpreferences'] = 'Calendar preferences';
 $string['calendartypes'] = 'Calendar types';
-$string['calendarurl'] = 'Calendar URL: {$a}';
+$string['calendarurlcopiedtoclipboard'] = 'Calendar URL copied to clipboard';
 $string['category'] = 'Category';
 $string['categoryevent'] = 'Category event';
 $string['clickhide'] = 'click to hide';
 $string['clickshow'] = 'click to show';
 $string['colcalendar'] = 'Calendar';
 $string['collastupdated'] = 'Last updated';
-$string['colpoll'] = 'Update';
+$string['colpoll'] = 'Refresh interval';
 $string['colactions'] = 'Actions';
 $string['commontasks'] = 'Options';
 $string['confirmeventdelete'] = 'Are you sure you want to delete the "{$a}" event?';
 $string['confirmeventseriesdelete'] = 'The "{$a->name}" event is part of a series. Do you want to delete just this event, or all {$a->count} events in the series?';
+$string['confirmsubscriptiondelete'] = 'Are you sure you want to delete the "{$a}" calendar subscription?';
+$string['copycalendarurl'] = 'Copy calendar URL';
+$string['copyurl'] = 'Copy URL';
 $string['course'] = 'Course';
 $string['courseevent'] = 'Course event';
 $string['courseevents'] = 'Course events';
@@ -52,6 +56,9 @@ $string['courses'] = 'Courses';
 $string['customexport'] = 'Custom range ({$a->timestart} - {$a->timeend})';
 $string['daily'] = 'Daily';
 $string['day'] = 'Day';
+$string['dayeventsmany'] = '{$a->num} events, {$a->day}';
+$string['dayeventsnone'] = 'No events, {$a}';
+$string['dayeventsone'] = '1 event, {$a}';
 $string['daynext'] = 'Next day';
 $string['dayprev'] = 'Previous day';
 $string['dayviewfor'] = 'Day view for:';
@@ -110,11 +117,11 @@ $string['eventnone'] = 'No events';
 $string['eventrepeat'] = 'Repeats';
 $string['events'] = 'Events';
 $string['eventsall'] = 'All events';
-$string['eventsdeleted'] = 'Events deleted';
-$string['eventsimported'] = 'Events imported: {$a}';
+$string['eventsdeleted'] = '{$a} events were deleted';
+$string['eventsimported'] = '{$a} events were imported';
 $string['eventsource'] = 'Event source';
-$string['eventsskipped'] = 'Events skipped: {$a}';
-$string['eventsupdated'] = 'Events updated: {$a}';
+$string['eventsskipped'] = '{$a} events were skipped';
+$string['eventsupdated'] = '{$a} events were updated';
 $string['eventsfor'] = '{$a} events';
 $string['eventskey'] = 'Events key';
 $string['eventspersonal'] = 'My personal events';
@@ -136,12 +143,14 @@ $string['eventsubscriptioneditwarning'] = 'This calendar event is part of a subs
 $string['expired'] = 'Expired';
 $string['explain_site_timeformat'] = 'You can choose to see times in either 12 or 24 hour format for the whole site. If you choose "default", then the format will be automatically chosen according to the language you use in the site. This setting can be overridden by user preferences.';
 $string['export'] = 'Export';
-$string['exporthelp'] = 'How do I subscribe to this calendar from a calendar application (Google/Outlook/Other)?';
+$string['exporthelp'] = '<p>The calendar URL provides a dynamic link for importing events into other calendars. Any new, changed or deleted events in the source calendar <strong>will</strong> be reflected in the other calendars.</p>
+<p>The calendar export allows you to create a backup copy of events, which may be imported into other calendars. Updates made in the source calendar <strong>will not</strong> be reflected in the other calendars.</p>';
 $string['exportbutton'] = 'Export';
 $string['exportcalendar'] = 'Export calendar';
 $string['forcecalendartype'] = 'Force calendar';
 $string['fri'] = 'Fri';
 $string['friday'] = 'Friday';
+$string['fullcalendar'] = 'Full calendar';
 $string['generateurlbutton'] = 'Get calendar URL';
 $string['gotoactivity'] = 'Go to activity';
 $string['gotocalendar'] = 'Go to calendar';
@@ -158,33 +167,33 @@ $string['eventtypeuser'] = 'user';
 $string['hideeventtype'] = 'Hide {$a} events';
 $string['showeventtype'] = 'Show {$a} events';
 $string['hourly'] = 'Hourly';
+$string['importcalendarexternal'] = 'Import an external calendar?';
 $string['importcalendar'] = 'Import calendar';
 $string['importcalendarheading'] = 'Import calendar...';
 $string['importcalendarfrom'] = 'Import from';
 $string['importfromfile'] = 'Calendar file (.ics)';
 $string['importfromurl'] = 'Calendar URL';
-$string['importfrominstructions'] = 'Please provide either a URL to a remote calendar, or upload a file.';
 $string['invalidtimedurationminutes'] = 'The duration in minutes you have entered is invalid. Please enter the duration in minutes greater than 0 or select no duration.';
 $string['invalidtimedurationuntil'] = 'The date and time you selected for duration until is before the start time of the event. Please correct this before proceeding.';
 $string['invalideventtype'] = 'The event type you have selected is invalid.';
 $string['iwanttoexport'] = 'Export';
 $string['less'] = 'Less';
-$string['managesubscriptions'] = 'Manage subscriptions';
+$string['managesubscriptions'] = 'Import or export calendars';
 $string['manyevents'] = '{$a} events';
 $string['mon'] = 'Mon';
 $string['monday'] = 'Monday';
 $string['monthly'] = 'Monthly';
-$string['monthlyview'] = 'Monthly view';
 $string['monthnext'] = 'Next month';
 $string['monthprev'] = 'Previous month';
 $string['monththis'] = 'This month';
 $string['more'] = 'More';
+$string['moreevents'] = '{$a} more';
 $string['namewithsource'] = '{$a->name} ({$a->source})';
 $string['never'] = 'Never';
 $string['newevent'] = 'New event';
 $string['notitle'] = 'no title';
 $string['noupcomingevents'] = 'There are no upcoming events';
-$string['nocalendarsubscriptions'] = 'You have no calendar subscriptions.';
+$string['nocalendarsubscriptions'] = 'No calendar subscriptions yet. Do you want to {$a}';
 $string['oneevent'] = '1 event';
 $string['pollinterval'] = 'Update interval';
 $string['pollinterval_help'] = 'How often you would like the calendar to update with new events.';
@@ -220,6 +229,7 @@ $string['repeatevent'] = 'Repeat this event';
 $string['repeatnone'] = 'No repeats';
 $string['repeatweeksl'] = 'Repeat weekly, creating altogether';
 $string['repeatweeksr'] = 'events';
+$string['requiresaction'] = '{$a} requires action';
 $string['sat'] = 'Sat';
 $string['saturday'] = 'Saturday';
 $string['shown'] = 'shown';
@@ -231,7 +241,7 @@ $string['subscriptions'] = 'Subscriptions';
 $string['subscriptionname'] = 'Calendar name';
 $string['subscriptionremoved'] = 'Calendar subscription {$a} removed';
 $string['subscriptionsource'] = 'Event source: {$a}';
-$string['subscriptionupdated'] = 'Calendar subscription {$a} updated';
+$string['subscriptionupdated'] = 'The \'{$a}\' calendar subscription has been updated';
 $string['sun'] = 'Sun';
 $string['sunday'] = 'Sunday';
 $string['thu'] = 'Thu';
@@ -271,9 +281,9 @@ $string['when'] = 'When';
 $string['whendate'] = 'When: {$a}';
 $string['yesterday'] = 'Yesterday';
 $string['youcandeleteallrepeats'] = 'This event is part of a repeating event series. You can delete this event only, or all {$a} events in the series at once.';
+$string['yoursubscriptions'] = 'Imported calendars';
 
-// Deprecated since Moodle 3.8.
-$string['global'] = 'Global';
-$string['globalevent'] = 'Global event';
-$string['globalevents'] = 'Global events';
-$string['eventtypeglobal'] = 'global';
+// Deprecated since Moodle 4.0.
+$string['calendarurl'] = 'Calendar URL: {$a}';
+$string['importfrominstructions'] = 'Please provide either a URL to a remote calendar, or upload a file.';
+$string['monthlyview'] = 'Monthly view';
