@@ -120,7 +120,6 @@ $string['categorytotal'] = 'Category total';
 $string['categorytotalname'] = 'Category total name';
 $string['categorytotalfull'] = '{$a->category} total';
 $string['collapsecriterion'] = 'Collapse criterion';
-$string['combo'] = 'Tabs and drop-down menu';
 $string['compact'] = 'Compact';
 $string['componentcontrolsvisibility'] = 'Whether this grade item is hidden is controlled by the activity settings.';
 $string['contract'] = 'Contract category';
@@ -149,13 +148,13 @@ $string['decimalpoints_help'] = 'This setting determines the number of decimal p
 $string['default'] = 'Default';
 $string['defaultprev'] = 'Default ({$a})';
 $string['deletecategory'] = 'Delete category';
+$string['deletescale'] = 'Delete scale';
 $string['disablegradehistory'] = 'Disable grade history';
 $string['disablegradehistory_help'] = 'Disable history tracking of changes in grades related tables. This may speed up the server a little and conserve space in database.';
 $string['displaylettergrade'] = 'Display letter grades';
 $string['displaypercent'] = 'Display percents';
 $string['displaypoints'] = 'Display points';
 $string['displayweighted'] = 'Display weighted grades';
-$string['dropdown'] = 'Drop-down menu';
 $string['droplow'] = 'Drop the lowest';
 $string['droplow_help'] = 'This setting enables a specified number of the lowest grades to be excluded from the aggregation.';
 $string['droplowestvalue'] = 'Set drop lowest grade value';
@@ -477,6 +476,7 @@ $string['locktimedate'] = 'Locked after: {$a}';
 $string['lockverbose'] = 'Lock {$a->category} {$a->itemmodule} {$a->itemname}';
 $string['lowest'] = 'Lowest';
 $string['lowgradeletter'] = 'Low';
+$string['manageoutcomes'] = 'Manage outcomes';
 $string['manualitem'] = 'Manual item';
 $string['mapfrom'] = 'Map from';
 $string['mapfrom_help'] = 'Select the column in the spreadsheet containing data for identifying the user, such as username, user ID or email address.';
@@ -544,7 +544,6 @@ $string['mygrades_desc'] = 'This setting allows for the option of linking to an
 $string['mypreferences'] = 'My preferences';
 $string['myreportpreferences'] = 'My report preferences';
 $string['mustchooserescaleyesorno'] = 'You must choose whether to rescale existing grades or not.';
-$string['navmethod'] = 'Navigation method';
 $string['neverdeletehistory'] = 'Never delete history';
 $string['newcategory'] = 'New category';
 $string['newitem'] = 'New grade item';
@@ -554,6 +553,8 @@ $string['nocategories'] = 'Grade categories could not be added or found for this
 $string['nocategoryname'] = 'No category name was given.';
 $string['nocategoryview'] = 'No category to view by';
 $string['nocourses'] = 'There are no courses yet';
+$string['noexistingscales'] = 'There are no existing scales.';
+$string['noexistingoutcomes'] = 'There are no existing outcomes.';
 $string['noforce'] = 'Do not force';
 $string['nogradeletters'] = 'No grade letters set';
 $string['nogradesreturned'] = 'No grades returned';
@@ -737,6 +738,7 @@ $string['setgradeletters'] = 'Set grade letters';
 $string['setpreferences'] = 'Set preferences';
 $string['setting'] = 'Setting';
 $string['settings'] = 'Settings';
+$string['setup'] = 'Setup';
 $string['setweights'] = 'Set weights';
 $string['showanalysisicon'] = 'Show grade analysis icon';
 $string['showanalysisicon_desc'] = 'Whether to show grade analysis icon by default. If the activity module supports it, the grade analysis icon links to a page with more detailed explanation of the grade and how it was obtained.';
@@ -821,7 +823,6 @@ $string['aggregatesubcatsupgradedgrades'] = 'Note: The aggregation setting "Aggr
 $string['upgradedgradeshidemessage'] = 'Dismiss notice';
 $string['switchtofullview'] = 'Switch to full view';
 $string['switchtosimpleview'] = 'Switch to simple view';
-$string['tabs'] = 'Tabs';
 $string['topcategory'] = 'Super category';
 $string['total'] = 'Total';
 $string['totalweightnot100'] = 'The total weight is not equal to 100';
@@ -888,3 +889,8 @@ $string['xml'] = 'XML';
 $string['yes'] = 'Yes';
 $string['yourgrade'] = 'Your grade';
 
+// Deprecated since Moodle 4.0.
+$string['navmethod'] = 'Navigation method';
+$string['dropdown'] = 'Drop-down menu';
+$string['tabs'] = 'Tabs';
+$string['combo'] = 'Tabs and drop-down menu';
