@@ -43,7 +43,7 @@ $string['archetype'] = 'Role archetype';
 $string['archetype_help'] = 'The role archetype determines the permissions when a role is reset to default. It also determines any new permissions for the role when the site is upgraded.';
 $string['archetypecoursecreator'] = 'ARCHETYPE: Course creator';
 $string['archetypeeditingteacher'] = 'ARCHETYPE: Teacher (editing)';
-$string['archetypefrontpage'] = 'ARCHETYPE: Authenticated user on frontpage';
+$string['archetypefrontpage'] = 'ARCHETYPE: Authenticated user on site home';
 $string['archetypeguest'] = 'ARCHETYPE: Guest';
 $string['archetypemanager'] = 'ARCHETYPE: Manager';
 $string['archetypestudent'] = 'ARCHETYPE: Student';
@@ -160,7 +160,6 @@ $string['contentbank:viewunlistedcontent'] = 'View unlisted content from the con
 $string['contentbank:upload'] = 'Upload new content to the content bank';
 $string['contentbank:useeditor'] = 'Create or edit content using a content type editor';
 $string['context'] = 'Context';
-$string['contextrolenotallowed'] = 'Role {$a} not allowed in this context.';
 $string['course:activityvisibility'] = 'Hide/show activities';
 $string['course:bulkmessaging'] = 'Send a message to many people';
 $string['course:create'] = 'Create courses';
@@ -241,6 +240,7 @@ $string['eventroleallowoverrideupdated'] = 'Allow role override';
 $string['eventroleallowswitchupdated'] = 'Allow role switch';
 $string['eventroleallowviewupdated'] = 'Allow role view';
 $string['eventroleassigned'] = 'Role assigned';
+$string['eventrolecapabilitiesupdated'] = 'Role capabilities updated';
 $string['eventroledeleted'] = 'Role deleted';
 $string['eventroleunassigned'] = 'Role unassigned';
 $string['eventroleupdated'] = 'Role updated';
@@ -251,10 +251,10 @@ $string['export'] = 'Export';
 $string['extusers'] = 'Existing users';
 $string['extusersmatching'] = 'Existing users matching \'{$a}\'';
 $string['filter:manage'] = 'Manage local filter settings';
-$string['frontpageuser'] = 'Authenticated user on frontpage';
-$string['frontpageuserdescription'] = 'All logged-in users in the frontpage course.';
+$string['frontpageuser'] = 'Authenticated user on site home';
+$string['frontpageuserdescription'] = 'All logged-in users in the site home course.';
 $string['globalrole'] = 'System role';
-$string['globalroleswarning'] = 'Warning: Any roles you assign from this page will apply to users throughout the entire system, including the front page and all courses.';
+$string['globalroleswarning'] = 'Warning: Any roles you assign from this page will apply to users throughout the entire system, including the site home and all courses.';
 $string['gotoassignroles'] = 'Go to Assign roles for this {$a->contextlevel}';
 $string['gotoassignsystemroles'] = 'Go to Assign system roles';
 $string['grade:edit'] = 'Edit grades';
@@ -366,10 +366,16 @@ $string['question:viewall'] = 'View all questions';
 $string['question:viewmine'] = 'View your own questions';
 $string['question:tagall'] = 'Tag all questions';
 $string['question:tagmine'] = 'Tag your own questions';
+$string['question:commentall'] = 'Comment on all questions';
+$string['question:commentmine'] = 'Comment on your own questions';
 $string['rating:rate'] = 'Add ratings to items';
 $string['rating:view'] = 'View the total rating you received';
 $string['rating:viewany'] = 'View total ratings that anyone received';
 $string['rating:viewall'] = 'View all raw ratings given by individuals';
+$string['reportbuilder:edit'] = 'Edit your own custom reports';
+$string['reportbuilder:editall'] = 'Edit all custom reports';
+$string['reportbuilder:scheduleviewas'] = 'Schedule reports to be viewed as other users';
+$string['reportbuilder:view'] = 'View custom reports';
 $string['resetrole'] = 'Reset';
 $string['resettingrole'] = 'Resetting role \'{$a}\'';
 $string['restore:configure'] = 'Configure restore options';
@@ -512,6 +518,3 @@ $string['privacy:metadata:role_capabilities:tableexplanation'] = 'The capabiliti
 $string['privacy:metadata:role_capabilities:timemodified'] = 'The date when the capability was created or modified.';
 $string['privacy:metadata:role_cohortroles'] = 'Roles to cohort';
 $string['course:togglecompletion'] = 'Manually mark activities as complete';
-
-// Deprecated since Moodle 3.8.
-$string['eventrolecapabilitiesupdated'] = 'Role capabilities updated';
