@@ -156,24 +156,12 @@ class behat_core_generator extends behat_generator_base {
                 'datagenerator' => 'role',
                 'required' => ['shortname'],
             ],
-            'role capabilities' => [
-                'singular' => 'role capability',
-                'datagenerator' => 'role_capability',
-                'required' => ['role'],
-                'switchids' => ['role' => 'roleid'],
-            ],
             'grade categories' => [
                 'singular' => 'grade category',
                 'datagenerator' => 'grade_category',
                 'required' => ['fullname', 'course'],
                 'switchids' => ['course' => 'courseid', 'gradecategory' => 'parent'],
             ],
-            'grade grades' => [
-                'singular' => 'grade grade',
-                'datagenerator' => 'grade_grade',
-                'required' => ['gradeitem'],
-                'switchids' => ['user' => 'userid', 'gradeitem' => 'itemid'],
-            ],
             'grade items' => [
                 'singular' => 'grade item',
                 'datagenerator' => 'grade_item',
@@ -299,33 +287,11 @@ class behat_core_generator extends behat_generator_base {
                 'required' => ['user', 'course', 'lastaccess'],
                 'switchids' => ['user' => 'userid', 'course' => 'courseid'],
             ],
-            'notifications' => [
-                'singular' => 'notification',
-                'datagenerator' => 'notification',
-                'required' => ['subject', 'userfrom', 'userto'],
-                'switchids' => ['userfrom' => 'userfromid', 'userto' => 'usertoid'],
-            ],
         ];
 
         return $entities;
     }
 
-    /**
-     * Get the grade item id using a name.
-     *
-     * @param string $name
-     * @return int The grade item id
-     */
-    protected function get_gradeitem_id(string $name): int {
-        global $DB;
-
-        if (!$id = $DB->get_field('grade_items', 'id', ['itemname' => $name])) {
-            throw new Exception('The specified grade item with name "' . $name . '" could not be found.');
-        }
-
-        return $id;
-    }
-
     /**
      * Remove any empty custom fields, to avoid errors when creating the course.
      *
@@ -592,16 +558,6 @@ class behat_core_generator extends behat_generator_base {
 
         if (!isset($data['status'])) {
             $data['status'] = null;
-        } else {
-            $status = strtolower($data['status']);
-            switch ($status) {
-                case 'active':
-                    $data['status'] = ENROL_USER_ACTIVE;
-                    break;
-                case 'suspended':
-                    $data['status'] = ENROL_USER_SUSPENDED;
-                    break;
-            }
         }
 
         // If the provided course shortname is the site shortname we consider it a system role assign.
@@ -725,23 +681,6 @@ class behat_core_generator extends behat_generator_base {
         $this->datagenerator->create_role($data);
     }
 
-    /**
-     * Assign capabilities to a role.
-     *
-     * @param array $data
-     */
-    protected function process_role_capability($data): void {
-        // We require the user to fill the role shortname.
-        if (empty($data['roleid'])) {
-            throw new Exception('\'role capability\' requires the field \'roleid\' to be specified');
-        }
-
-        $roleid = $data['roleid'];
-        unset($data['roleid']);
-
-        $this->datagenerator->create_role_capability($roleid, $data, \context_system::instance());
-    }
-
     /**
      * Adds members to cohorts
      *
@@ -1101,39 +1040,6 @@ class behat_core_generator extends behat_generator_base {
         $DB->insert_record('badge_backpack', $backpack);
     }
 
-    /**
-     * Creates notifications to specific user.
-     *
-     * @param array $data
-     * @return void
-     */
-    protected function process_notification(array $data) {
-        global $DB;
-
-        $notification = new stdClass();
-        $notification->useridfrom = $data['userfromid'];
-        $notification->useridto = $data['usertoid'];
-        $notification->subject = $data['subject'];
-        $notification->fullmessage = $data['subject'] . ' description';
-        $notification->smallmessage = $data['subject'] . ' description';
-        $notification->fullmessagehtml = $data['subject'] . ' description';
-
-        if ($data['timecreated'] !== 'null') {
-            $notification->timecreated = $data['timecreated'];
-        }
-
-        if ($data['timeread'] !== 'null') {
-            $notification->timeread = $data['timeread'];
-        }
-
-        if (!empty($data)) {
-            $popupnotification = new stdClass();
-            $popupnotification->notificationid = $DB->insert_record('notifications', $notification);
-            $DB->insert_record('message_popup_notifications', $popupnotification);
-        }
-
-    }
-
     /**
      * Creates user last access data within given courses.
      *
