@@ -72,8 +72,12 @@ class core_date {
 
         if (is_numeric($currentvalue)) {
             // UTC offset.
-            $modifier = ($currentvalue > 0) ? '+' : '';
-            $a = 'UTC' . $modifier . number_format($currentvalue, 1);
+            if ($currentvalue == 0) {
+                $a = 'UTC';
+            } else {
+                $modifier = ($currentvalue > 0) ? '+' : '';
+                $a = 'UTC' . $modifier . number_format($currentvalue, 1);
+            }
             $timezones[$currentvalue] = get_string('timezoneinvalid', 'core_admin', $a);
         } else {
             // Some string we don't recognise.
@@ -134,7 +138,7 @@ class core_date {
             // Convert to known zone.
             $tz = self::$badzones[$tz];
             $fixed = true;
-        } else if (is_number($tz)) {
+        } else if (is_numeric($tz)) {
             // Half hour numeric offsets were already tested, try rounding to integers here.
             $roundedtz = (string)(int)$tz;
             if (isset(self::$badzones[$roundedtz])) {
@@ -360,7 +364,7 @@ class core_date {
             'Dateline Standard Time' => 'Etc/GMT+12',
             'Hawaiian Standard Time' => 'Pacific/Honolulu',
             'Alaskan Standard Time' => 'America/Anchorage',
-            'Pacific Standard Time (Mexico)' => 'America/Santa_Isabel',
+            'Pacific Standard Time (Mexico)' => 'America/Tijuana',
             'Pacific Standard Time' => 'America/Los_Angeles',
             'US Mountain Standard Time' => 'America/Phoenix',
             'Mountain Standard Time (Mexico)' => 'America/Chihuahua',
@@ -370,18 +374,24 @@ class core_date {
             'Central Standard Time (Mexico)' => 'America/Mexico_City',
             'Canada Central Standard Time' => 'America/Regina',
             'SA Pacific Standard Time' => 'America/Bogota',
+            'S.A. Pacific Standard Time' => 'America/Bogota',
             'Eastern Standard Time' => 'America/New_York',
-            'US Eastern Standard Time' => 'America/Indianapolis',
+            'US Eastern Standard Time' => 'America/Indiana/Indianapolis',
+            'U.S. Eastern Standard Time' => 'America/Indiana/Indianapolis',
             'Venezuela Standard Time' => 'America/Caracas',
             'Paraguay Standard Time' => 'America/Asuncion',
             'Atlantic Standard Time' => 'America/Halifax',
             'Central Brazilian Standard Time' => 'America/Cuiaba',
             'SA Western Standard Time' => 'America/La_Paz',
+            'S.A. Western Standard Time' => 'America/La_Paz',
             'Pacific SA Standard Time' => 'America/Santiago',
+            'Pacific S.A. Standard Time' => 'America/Santiago',
             'Newfoundland Standard Time' => 'America/St_Johns',
+            'Newfoundland and Labrador Standard Time' => 'America/St_Johns',
             'E. South America Standard Time' => 'America/Sao_Paulo',
-            'Argentina Standard Time' => 'America/Buenos_Aires',
+            'Argentina Standard Time' => 'America/Argentina/Buenos_Aires',
             'SA Eastern Standard Time' => 'America/Cayenne',
+            'S.A. Eastern Standard Time' => 'America/Cayenne',
             'Greenland Standard Time' => 'America/Godthab',
             'Montevideo Standard Time' => 'America/Montevideo',
             'Bahia Standard Time' => 'America/Bahia',
@@ -425,12 +435,13 @@ class core_date {
             'Pakistan Standard Time' => 'Asia/Karachi',
             'India Standard Time' => 'Asia/Kolkata', // PHP and Windows differ in spelling.
             'Sri Lanka Standard Time' => 'Asia/Colombo',
-            'Nepal Standard Time' => 'Asia/Katmandu',
+            'Nepal Standard Time' => 'Asia/Kathmandu',
             'Central Asia Standard Time' => 'Asia/Almaty',
             'Bangladesh Standard Time' => 'Asia/Dhaka',
             'N. Central Asia Standard Time' => 'Asia/Novosibirsk',
-            'Myanmar Standard Time' => 'Asia/Rangoon',
+            'Myanmar Standard Time' => 'Asia/Yangon',
             'SE Asia Standard Time' => 'Asia/Bangkok',
+            'S.E. Asia Standard Time' => 'Asia/Bangkok',
             'North Asia Standard Time' => 'Asia/Krasnoyarsk',
             'China Standard Time' => 'Asia/Shanghai',
             'North Asia East Standard Time' => 'Asia/Irkutsk',
@@ -443,8 +454,10 @@ class core_date {
             'Yakutsk Standard Time' => 'Asia/Yakutsk',
             'Cen. Australia Standard Time' => 'Australia/Adelaide',
             'AUS Central Standard Time' => 'Australia/Darwin',
+            'A.U.S. Central Standard Time' => 'Australia/Darwin',
             'E. Australia Standard Time' => 'Australia/Brisbane',
             'AUS Eastern Standard Time' => 'Australia/Sydney',
+            'A.U.S. Eastern Standard Time' => 'Australia/Sydney',
             'West Pacific Standard Time' => 'Pacific/Port_Moresby',
             'Tasmania Standard Time' => 'Australia/Hobart',
             'Magadan Standard Time' => 'Asia/Magadan',
@@ -454,38 +467,119 @@ class core_date {
             'Russia Time Zone 11' => 'Asia/Kamchatka',
             'New Zealand Standard Time' => 'Pacific/Auckland',
             'Fiji Standard Time' => 'Pacific/Fiji',
+            'Fiji Islands Standard Time' => 'Pacific/Fiji',
             'Tonga Standard Time' => 'Pacific/Tongatapu',
             'Samoa Standard Time' => 'Pacific/Apia',
             'Line Islands Standard Time' => 'Pacific/Kiritimati',
+            'Mexico Standard Time 2' => 'America/Chihuahua',
+            'Mexico Standard Time' => 'America/Mexico_City',
+            'U.S. Mountain Standard Time' => 'America/Phoenix',
+            'Mid-Atlantic Standard Time' => 'Atlantic/South_Georgia',
+            'E. Europe Standard Time' => 'Europe/Minsk',
+            'Transitional Islamic State of Afghanistan Standard Time' => 'Asia/Kabul',
+            'Armenian Standard Time' => 'Asia/Yerevan',
+            'Kamchatka Standard Time' => 'Asia/Kamchatka',
 
-            // A lot more bad legacy time zones.
+            // A lot more bad legacy (deprecated) time zones.
+            'Australia/ACT' => 'Australia/Sydney',
+            'Australia/LHI' => 'Australia/Lord_Howe',
+            'Australia/North' => 'Australia/Darwin',
+            'Australia/NSW' => 'Australia/Sydney',
+            'Australia/Queensland' => 'Australia/Brisbane',
+            'Australia/South' => 'Australia/Adelaide',
+            'Australia/Tasmania' => 'Australia/Hobart',
+            'Australia/Victoria' => 'Australia/Melbourne',
+            'Australia/West' => 'Australia/Perth',
+            'Brazil/Acre' => 'America/Rio_Branco',
+            'Brazil/DeNoronha' => 'America/Noronha',
+            'Brazil/East' => 'America/Sao_Paulo',
+            'Brazil/West' => 'America/Manaus',
+            'Canada/Atlantic' => 'America/Halifax',
+            'Canada/Central' => 'America/Winnipeg',
+            'Canada/Eastern' => 'America/Toronto',
+            'Canada/Mountain' => 'America/Edmonton',
+            'Canada/Newfoundland' => 'America/St_Johns',
+            'Canada/Pacific' => 'America/Vancouver',
+            'Canada/Saskatchewan' => 'America/Regina',
+            'Canada/Yukon' => 'America/Whitehorse',
+            'CDT' => 'America/Chicago',
             'CET' => 'Europe/Berlin',
             'Central European Time' => 'Europe/Berlin',
-            'CST' => 'America/Chicago',
             'Central Time' => 'America/Chicago',
-            'CST6CDT' => 'America/Chicago',
-            'CDT' => 'America/Chicago',
+            'Chile/Continental' => 'America/Santiago',
+            'Chile/EasterIsland' => 'Pacific/Easter',
             'China Time' => 'Asia/Shanghai',
+            'CST' => 'America/Chicago',
+            'CST6CDT' => 'America/Chicago',
+            'Cuba' => 'America/Havana',
             'EDT' => 'America/New_York',
-            'EST' => 'America/New_York',
+            'EET' => 'Europe/Kiev',
+            'Egypt' => 'Africa/Cairo',
+            'Eire' => 'Europe/Dublin',
+            'EST' => 'America/Cancun',
             'EST5EDT' => 'America/New_York',
             'Eastern Time' => 'America/New_York',
-            'IST' => 'Asia/Kolkata',
-            'India Time' => 'Asia/Kolkata',
-            'JST' => 'Asia/Tokyo',
-            'Japan Time' => 'Asia/Tokyo',
-            'Japan Standard Time' => 'Asia/Tokyo',
-            'MDT' => 'America/Denver',
-            'MST' => 'America/Denver',
-            'MST7MDT' => 'America/Denver',
-            'PDT' => 'America/Los_Angeles',
-            'PST' => 'America/Los_Angeles',
-            'Pacific Time' => 'America/Los_Angeles',
-            'PST8PDT' => 'America/Los_Angeles',
-            'HST' => 'Pacific/Honolulu',
-            'WET' => 'Europe/London',
-            'EET' => 'Europe/Kiev',
+            'Etc/Greenwich' => 'Etc/GMT',
+            'Etc/UCT' => 'Etc/UTC',
+            'Etc/Universal' => 'Etc/UTC',
+            'Etc/Zulu' => 'Etc/UTC',
             'FET' => 'Europe/Minsk',
+            'GB' => 'Europe/London',
+            'GB-Eire' => 'Europe/London',
+            'Greenwich' => 'Etc/GMT',
+            'Hongkong' => 'Asia/Hong_Kong',
+            'HST' => 'Pacific/Honolulu',
+            'Iceland' => 'Atlantic/Reykjavik',
+            'India Time' => 'Asia/Kolkata',
+            'Iran' => 'Asia/Tehran',
+            'Israel' => 'Asia/Jerusalem',
+            'IST' => 'Asia/Kolkata',
+            'Jamaica' => 'America/Jamaica',
+            'Japan' => 'Asia/Tokyo',
+            'Japan Standard Time' => 'Asia/Tokyo',
+            'Japan Time' => 'Asia/Tokyo',
+            'JST' => 'Asia/Tokyo',
+            'Kwajalein' => 'Pacific/Kwajalein',
+            'Libya' => 'Africa/Tripoli',
+            'MDT' => 'America/Denver',
+            'MET' => 'Europe/Paris',
+            'Mexico/BajaNorte' => 'America/Tijuana',
+            'Mexico/BajaSur' => 'America/Mazatlan',
+            'Mexico/General' => 'America/Mexico_City',
+            'MST' => 'America/Phoenix',
+            'MST7MDT' => 'America/Denver',
+            'Navajo' => 'America/Denver',
+            'NZ' => 'Pacific/Auckland',
+            'NZ-CHAT' => 'Pacific/Chatham',
+            'Pacific Time' => 'America/Los_Angeles',
+            'PDT' => 'America/Los_Angeles',
+            'Poland' => 'Europe/Warsaw',
+            'Portugal' => 'Europe/Lisbon',
+            'PRC' => 'Asia/Shanghai',
+            'PST' => 'America/Los_Angeles',
+            'PST8PDT' => 'America/Los_Angeles',
+            'ROC' => 'Asia/Taipei',
+            'ROK' => 'Asia/Seoul',
+            'Singapore' => 'Asia/Singapore',
+            'Turkey' => 'Europe/Istanbul',
+            'UCT' => 'Etc/UTC',
+            'Universal' => 'Etc/UTC',
+            'US/Alaska' => 'America/Anchorage',
+            'US/Aleutian' => 'America/Adak',
+            'US/Arizona' => 'America/Phoenix',
+            'US/Central' => 'America/Chicago',
+            'US/East-Indiana' => 'America/Indiana/Indianapolis',
+            'US/Eastern' => 'America/New_York',
+            'US/Hawaii' => 'Pacific/Honolulu',
+            'US/Indiana-Starke' => 'America/Indiana/Knox',
+            'US/Michigan' => 'America/Detroit',
+            'US/Mountain' => 'America/Denver',
+            'US/Pacific' => 'America/Los_Angeles',
+            'US/Pacific-New' => 'America/Los_Angeles',
+            'US/Samoa' => 'Pacific/Pago_Pago',
+            'W-SU' => 'Europe/Moscow',
+            'WET' => 'Europe/London',
+            'Zulu' => 'Etc/UTC',
 
             // Some UTC variations.
             'UTC-01' => 'Etc/GMT+1',
@@ -503,12 +597,61 @@ class core_date {
             'Etc/GMT-0' => 'Etc/GMT',
             'Etc/GMT0' => 'Etc/GMT',
 
-            // And lastly some alternative city spelling.
+            // Link old timezone names with their new names.
+            'Africa/Asmera' => 'Africa/Asmara',
+            'Africa/Timbuktu' => 'Africa/Abidjan',
+            'America/Argentina/ComodRivadavia' => 'America/Argentina/Catamarca',
+            'America/Atka' => 'America/Adak',
+            'America/Buenos_Aires' => 'America/Argentina/Buenos_Aires',
+            'America/Catamarca' => 'America/Argentina/Catamarca',
+            'America/Coral_Harbour' => 'America/Atikokan',
+            'America/Cordoba' => 'America/Argentina/Cordoba',
+            'America/Ensenada' => 'America/Tijuana',
+            'America/Fort_Wayne' => 'America/Indiana/Indianapolis',
+            'America/Indianapolis' => 'America/Indiana/Indianapolis',
+            'America/Jujuy' => 'America/Argentina/Jujuy',
+            'America/Knox_IN' => 'America/Indiana/Knox',
+            'America/Louisville' => 'America/Kentucky/Louisville',
+            'America/Mendoza' => 'America/Argentina/Mendoza',
+            'America/Montreal' => 'America/Toronto',
+            'America/Porto_Acre' => 'America/Rio_Branco',
+            'America/Rosario' => 'America/Argentina/Cordoba',
+            'America/Santa_Isabel' => 'America/Tijuana',
+            'America/Shiprock' => 'America/Denver',
+            'America/Virgin' => 'America/Port_of_Spain',
+            'Antarctica/South_Pole' => 'Pacific/Auckland',
+            'Asia/Ashkhabad' => 'Asia/Ashgabat',
             'Asia/Calcutta' => 'Asia/Kolkata',
+            'Asia/Chongqing' => 'Asia/Shanghai',
+            'Asia/Chungking' => 'Asia/Shanghai',
+            'Asia/Dacca' => 'Asia/Dhaka',
+            'Asia/Harbin' => 'Asia/Shanghai',
+            'Asia/Istanbul' => 'Europe/Istanbul',
+            'Asia/Kashgar' => 'Asia/Urumqi',
+            'Asia/Katmandu' => 'Asia/Kathmandu',
+            'Asia/Macao' => 'Asia/Macau',
+            'Asia/Rangoon' => 'Asia/Yangon',
+            'Asia/Saigon' => 'Asia/Ho_Chi_Minh',
+            'Asia/Tel_Aviv' => 'Asia/Jerusalem',
+            'Asia/Thimbu' => 'Asia/Thimphu',
+            'Asia/Ujung_Pandang' => 'Asia/Makassar',
+            'Asia/Ulan_Bator' => 'Asia/Ulaanbaatar',
+            'Atlantic/Faeroe' => 'Atlantic/Faroe',
+            'Atlantic/Jan_Mayen' => 'Europe/Oslo',
+            'Australia/Canberra' => 'Australia/Sydney',
+            'Australia/Yancowinna' => 'Australia/Broken_Hill',
+            'Europe/Belfast' => 'Europe/London',
+            'Europe/Nicosia' => 'Asia/Nicosia',
+            'Europe/Tiraspol' => 'Europe/Chisinau',
+            'Pacific/Johnston' => 'Pacific/Honolulu',
+            'Pacific/Ponape' => 'Pacific/Pohnpei',
+            'Pacific/Samoa' => 'Pacific/Pago_Pago',
+            'Pacific/Truk' => 'Pacific/Chuuk',
+            'Pacific/Yap' => 'Pacific/Chuuk',
         );
 
         // Legacy GMT fallback.
-        for ($i = -14; $i <= 13; $i++) {
+        for ($i = -12; $i <= 14; $i++) {
             $off = abs($i);
             if ($i < 0) {
                 $mapto = 'Etc/GMT+' . $off;
@@ -532,12 +675,10 @@ class core_date {
         }
 
         // Legacy Moodle half an hour offsets - pick any city nearby, ideally without DST.
-        self::$badzones['-4.5'] = 'America/Caracas';
         self::$badzones['4.5'] = 'Asia/Kabul';
         self::$badzones['5.5'] = 'Asia/Kolkata';
         self::$badzones['6.5'] = 'Asia/Rangoon';
         self::$badzones['9.5'] = 'Australia/Darwin';
-        self::$badzones['11.5'] = 'Pacific/Norfolk';
 
         // Remove bad zones that are elsewhere.
         foreach (self::$bczones as $zone => $unused) {
