@@ -84,11 +84,12 @@ class external_api {
                 $function->classpath = $CFG->dirroot.'/'.$function->classpath;
             }
             if (!file_exists($function->classpath)) {
-                throw new coding_exception('Cannot find file with external function implementation');
+                throw new coding_exception('Cannot find file ' . $function->classpath .
+                        ' with external function implementation');
             }
             require_once($function->classpath);
             if (!class_exists($function->classname)) {
-                throw new coding_exception('Cannot find external class');
+                throw new coding_exception('Cannot find external class ' . $function->classname);
             }
         }
 
@@ -99,13 +100,16 @@ class external_api {
 
         // Make sure the implementaion class is ok.
         if (!method_exists($function->classname, $function->methodname)) {
-            throw new coding_exception('Missing implementation method of '.$function->classname.'::'.$function->methodname);
+            throw new coding_exception('Missing implementation method ' .
+                    $function->classname . '::' . $function->methodname);
         }
         if (!method_exists($function->classname, $function->parameters_method)) {
-            throw new coding_exception('Missing parameters description');
+            throw new coding_exception('Missing parameters description method ' .
+                    $function->classname . '::' . $function->parameters_method);
         }
         if (!method_exists($function->classname, $function->returns_method)) {
-            throw new coding_exception('Missing returned values description');
+            throw new coding_exception('Missing returned values description method ' .
+                    $function->classname . '::' . $function->returns_method);
         }
         if (method_exists($function->classname, $function->deprecated_method)) {
             if (call_user_func(array($function->classname, $function->deprecated_method)) === true) {
@@ -117,14 +121,16 @@ class external_api {
         // Fetch the parameters description.
         $function->parameters_desc = call_user_func(array($function->classname, $function->parameters_method));
         if (!($function->parameters_desc instanceof external_function_parameters)) {
-            throw new coding_exception('Invalid parameters description');
+            throw new coding_exception($function->classname . '::' . $function->parameters_method .
+                    ' did not return a valid external_function_parameters object.');
         }
 
         // Fetch the return values description.
         $function->returns_desc = call_user_func(array($function->classname, $function->returns_method));
         // Null means void result or result is ignored.
         if (!is_null($function->returns_desc) and !($function->returns_desc instanceof external_description)) {
-            throw new coding_exception('Invalid return description');
+            throw new coding_exception($function->classname . '::' . $function->returns_method .
+                    ' did not return a valid external_description object');
         }
 
         // Now get the function description.
@@ -192,7 +198,7 @@ class external_api {
         // Eventually this should shift into the various handlers and not be handled via config.
         $readonlysession = $externalfunctioninfo->readonlysession ?? false;
         if (!$readonlysession || empty($CFG->enable_read_only_sessions)) {
-            \core\session\manager::restart_with_write_lock();
+            \core\session\manager::restart_with_write_lock($readonlysession);
         }
 
         $currentpage = $PAGE;
@@ -244,7 +250,7 @@ class external_api {
                 foreach ($plugins as $plugin => $callback) {
                     $result = $callback($externalfunctioninfo, $params);
                     if ($result !== false) {
-                        break;
+                        break 2;
                     }
                 }
             }
@@ -851,7 +857,7 @@ class external_warnings extends external_multiple_structure {
                     'item' => new external_value(PARAM_TEXT, $itemdesc, VALUE_OPTIONAL),
                     'itemid' => new external_value(PARAM_INT, $itemiddesc, VALUE_OPTIONAL),
                     'warningcode' => new external_value(PARAM_ALPHANUM, $warningcodedesc),
-                    'message' => new external_value(PARAM_TEXT,
+                    'message' => new external_value(PARAM_RAW,
                             'untranslated english message to explain the warning')
                 ), 'warning'),
             'list of warnings', VALUE_OPTIONAL);
@@ -1099,7 +1105,7 @@ function external_generate_token_for_current_user($service) {
             $unsettoken = true;
         }
 
-        // Remove token if its ip not in whitelist.
+        // Remove token if its IP is restricted.
         if (isset($token->iprestriction) and !address_in_subnet(getremoteaddr(), $token->iprestriction)) {
             $unsettoken = true;
         }
@@ -1166,7 +1172,7 @@ function external_generate_token_for_current_user($service) {
  * @since  Moodle 3.2
  */
 function external_log_token_request($token) {
-    global $DB;
+    global $DB, $USER;
 
     $token->privatetoken = null;
 
@@ -1179,6 +1185,25 @@ function external_log_token_request($token) {
     $event = \core\event\webservice_token_sent::create($params);
     $event->add_record_snapshot('external_tokens', $token);
     $event->trigger();
+
+    // Check if we need to notify the user about the new login via token.
+    $loginip = getremoteaddr();
+    if ($USER->lastip != $loginip &&
+            ((!WS_SERVER && !CLI_SCRIPT && NO_MOODLE_COOKIES) || PHPUNIT_TEST)) {
+
+        $logintime = time();
+        $useragent = \core_useragent::get_user_agent_string();
+        $ismoodleapp = \core_useragent::is_moodle_app();
+
+        // Schedule adhoc task to sent a login notification to the user.
+        $task = new \core\task\send_login_notifications();
+        $task->set_userid($USER->id);
+        $task->set_custom_data(compact('ismoodleapp', 'useragent', 'loginip', 'logintime'));
+        $task->set_component('core');
+        // We need sometime so the mobile app will send to Moodle the device information after login.
+        $task->set_next_run_time($logintime + (2 * MINSECS));
+        \core\task\manager::reschedule_or_queue_adhoc_task($task);
+    }
 }
 
 /**
@@ -1211,6 +1236,9 @@ class external_settings {
     /** @var string The session lang */
     private $lang = '';
 
+    /** @var string The timezone to use during this WS request */
+    private $timezone = '';
+
     /**
      * Constructor - protected - can not be instanciated
      */
@@ -1223,12 +1251,6 @@ class external_settings {
         }
     }
 
-    /**
-     * Clone - private - can not be cloned
-     */
-    private final function __clone() {
-    }
-
     /**
      * Return only one instance
      *
@@ -1331,6 +1353,24 @@ class external_settings {
     public function get_lang() {
         return $this->lang;
     }
+
+    /**
+     * Set timezone
+     *
+     * @param string $timezone
+     */
+    public function set_timezone($timezone) {
+        $this->timezone = $timezone;
+    }
+
+    /**
+     * Get timezone
+     *
+     * @return string
+     */
+    public function get_timezone() {
+        return $this->timezone;
+    }
 }
 
 /**
