@@ -442,12 +442,12 @@ class grade_grade extends grade_object {
     public function set_overridden($state, $refresh = true) {
         if (empty($this->overridden) and $state) {
             $this->overridden = time();
-            $this->update();
+            $this->update(null, true);
             return true;
 
         } else if (!empty($this->overridden) and !$state) {
             $this->overridden = 0;
-            $this->update();
+            $this->update(null, true);
 
             if ($refresh) {
                 //refresh when unlocking
@@ -718,14 +718,6 @@ class grade_grade extends grade_object {
     protected static function flatten_dependencies_array(&$dependson, &$dependencydepth) {
         // Flatten the nested dependencies - this will handle recursion bombs because it removes duplicates.
         $somethingchanged = true;
-        // First of all, delete any incorrect (not array or individual null) dependency, they aren't welcome.
-        // TODO: Maybe we should report about this happening, it shouldn't if all dependencies are correct and consistent.
-        foreach ($dependson as $itemid => $depends) {
-            $depends = is_array($depends) ? $depends : []; // Only arrays are accepted.
-            $dependson[$itemid] = array_filter($depends, function($val) { // Only not-null values are accepted.
-                return !is_null($val);
-            });
-        }
         while ($somethingchanged) {
             $somethingchanged = false;
 
@@ -733,7 +725,7 @@ class grade_grade extends grade_object {
                 // Make a copy so we can tell if it changed.
                 $before = $dependson[$itemid];
                 foreach ($depends as $subitemid => $subdepends) {
-                    $dependson[$itemid] = array_unique(array_merge($depends, $dependson[$subdepends] ?? []));
+                    $dependson[$itemid] = array_unique(array_merge($depends, $dependson[$subdepends]));
                     sort($dependson[$itemid], SORT_NUMERIC);
                 }
                 if ($before != $dependson[$itemid]) {
@@ -1029,31 +1021,20 @@ class grade_grade extends grade_object {
         return $this->finalgrade >= $this->grade_item->gradepass;
     }
 
-    /**
-     * Insert the grade_grade instance into the database.
-     *
-     * @param string $source From where was the object inserted (mod/forum, manual, etc.)
-     * @return int The new grade_grade ID if successful, false otherwise
-     */
-    public function insert($source=null) {
-        // TODO: dategraded hack - do not update times, they are used for submission and grading (MDL-31379)
-        //$this->timecreated = $this->timemodified = time();
-        return parent::insert($source);
-    }
-
     /**
      * In addition to update() as defined in grade_object rounds the float numbers using php function,
      * the reason is we need to compare the db value with computed number to skip updates if possible.
      *
      * @param string $source from where was the object inserted (mod/forum, manual, etc.)
+     * @param bool $isbulkupdate If bulk grade update is happening.
      * @return bool success
      */
-    public function update($source=null) {
+    public function update($source=null, $isbulkupdate = false) {
         $this->rawgrade = grade_floatval($this->rawgrade);
         $this->finalgrade = grade_floatval($this->finalgrade);
         $this->rawgrademin = grade_floatval($this->rawgrademin);
         $this->rawgrademax = grade_floatval($this->rawgrademax);
-        return parent::update($source);
+        return parent::update($source, $isbulkupdate);
     }
 
 
@@ -1146,8 +1127,9 @@ class grade_grade extends grade_object {
      * has changed, and clear up a possible score cache.
      *
      * @param bool $deleted True if grade was actually deleted
+     * @param bool $isbulkupdate If bulk grade update is happening.
      */
-    protected function notify_changed($deleted) {
+    protected function notify_changed($deleted, $isbulkupdate = false) {
         global $CFG;
 
         // Condition code may cache the grades for conditional availability of
@@ -1208,7 +1190,7 @@ class grade_grade extends grade_object {
         }
 
         // Pass information on to completion system
-        $completion->inform_grade_changed($cm, $this->grade_item, $this, $deleted);
+        $completion->inform_grade_changed($cm, $this->grade_item, $this, $deleted, $isbulkupdate);
     }
 
     /**
