@@ -179,6 +179,9 @@ function install_helpbutton($url, $title='') {
  * @return string
  */
 function install_db_validate($database, $dbhost, $dbuser, $dbpass, $dbname, $prefix, $dboptions) {
+    if (!preg_match('/^[a-z_]*$/', $prefix)) {
+        return get_string('invaliddbprefix', 'install');
+    }
     try {
         try {
             $database->connect($dbhost, $dbuser, $dbpass, $dbname, $prefix, $dboptions);
@@ -256,7 +259,16 @@ function install_generate_configphp($database, $cfg) {
     }
     $configphp .= '$CFG->directorypermissions = ' . $chmod . ';' . PHP_EOL . PHP_EOL;
 
-    $configphp .= 'require_once(dirname(__FILE__) . \'/lib/setup.php\');' . PHP_EOL . PHP_EOL;
+    if (isset($cfg->upgradekey) and $cfg->upgradekey !== '') {
+        $configphp .= '$CFG->upgradekey = ' . var_export($cfg->upgradekey, true) . ';' . PHP_EOL . PHP_EOL;
+    }
+
+    if (isset($cfg->setsitepresetduringinstall) and $cfg->setsitepresetduringinstall !== '') {
+        $configphp .= '$CFG->setsitepresetduringinstall = ' . var_export($cfg->setsitepresetduringinstall, true) .
+            ';' . PHP_EOL . PHP_EOL;
+    }
+
+    $configphp .= 'require_once(__DIR__ . \'/lib/setup.php\');' . PHP_EOL . PHP_EOL;
     $configphp .= '// There is no php closing tag in this file,' . PHP_EOL;
     $configphp .= '// it is intentional because it prevents trailing whitespace problems!' . PHP_EOL;
 
@@ -333,23 +345,19 @@ function install_print_header($config, $stagename, $heading, $stagetext, $stagec
 
     echo '<link rel="stylesheet" type="text/css" href="'.$CFG->wwwroot.'/install/css.php" />
           <title>'.get_string('installation','install').' - Moodle '.$CFG->target_release.'</title>
+          <meta name="robots" content="noindex">
           <meta http-equiv="content-type" content="text/html; charset=UTF-8" />
           <meta http-equiv="pragma" content="no-cache" />
           <meta http-equiv="expires" content="0" />';
 
     echo '</head><body class="notloggedin">
-            <div id="page" class="stage'.$config->stage.'">
+            <div id="page" class="mt-0 container stage'.$config->stage.'">
                 <div id="page-header">
                     <div id="header" class=" clearfix">
                         <h1 class="headermain">'.get_string('installation','install').'</h1>
                         <div class="headermenu">&nbsp;</div>
                     </div>
-                    <div class="navbar clearfix">
-                        <nav class="breadcrumb-nav">
-                            <ul class="breadcrumb"><li class="first">'.$stagename.'</li></ul>
-                        </nav>
-                        <div class="navbutton">&nbsp;</div>
-                    </div>
+                    <div class="bg-light p-3 mb-3"><h3 class="m-0">'.$stagename.'</h3></div>
                 </div>
           <!-- END OF HEADER -->
           <div id="installdiv">';
@@ -380,9 +388,9 @@ function install_print_footer($config, $reload=false) {
     global $CFG;
 
     if ($config->stage > INSTALL_WELCOME) {
-        $first = '<input type="submit" id="previousbutton" name="previous" value="&laquo; '.s(get_string('previous')).'" />';
+        $first = '<input type="submit" id="previousbutton" class="btn btn-secondary flex-grow-0 ml-auto" name="previous" value="&laquo; '.s(get_string('previous')).'" />';
     } else {
-        $first = '<input type="submit" id="previousbutton" name="next" value="'.s(get_string('reload')).'" />';
+        $first = '<input type="submit" id="previousbutton" class="btn btn-secondary flex-grow-0  ml-auto" name="next" value="'.s(get_string('reload')).'" />';
         $first .= '<script type="text/javascript">
 //<![CDATA[
     var first = document.getElementById("previousbutton");
@@ -393,16 +401,16 @@ function install_print_footer($config, $reload=false) {
     }
 
     if ($reload) {
-        $next = '<input type="submit" id="nextbutton" class="btn btn-primary" name="next" value="'.s(get_string('reload')).'" />';
+        $next = '<input type="submit" id="nextbutton" class="btn btn-primary ml-1 flex-grow-0 mr-auto" name="next" value="'.s(get_string('reload')).'" />';
     } else {
-        $next = '<input type="submit" id="nextbutton" class="btn btn-primary" name="next" value="'.s(get_string('next')).' &raquo;" />';
+        $next = '<input type="submit" id="nextbutton" class="btn btn-primary ml-1 flex-grow-0 mr-auto" name="next" value="'.s(get_string('next')).' &raquo;" />';
     }
 
-    echo '</fieldset><fieldset id="nav_buttons">'.$first.$next.'</fieldset>';
+    echo '</fieldset><div id="nav_buttons" class="mb-3 w-100 flex-row-reverse">'.$first.$next.'</div>';
 
     $homelink  = '<div class="sitelink">'.
        '<a title="Moodle '. $CFG->target_release .'" href="http://docs.moodle.org/en/Administrator_documentation" onclick="this.target=\'_blank\'">'.
-       '<img src="pix/moodlelogo.png" alt="moodlelogo" /></a></div>';
+       '<img src="pix/moodlelogo.png" alt="'.get_string('moodlelogo').'" /></a></div>';
 
     echo '</form></div>';
     echo '<div id="page-footer">'.$homelink.'</div>';
@@ -488,11 +496,21 @@ function install_cli_database(array $options, $interactive) {
     // set up admin user password
     $DB->set_field('user', 'password', hash_internal_user_password($options['adminpass']), array('username' => 'admin'));
 
+    // Set the admin email address if specified.
+    if (isset($options['adminemail'])) {
+        $DB->set_field('user', 'email', $options['adminemail'], array('username' => 'admin'));
+    }
+
     // rename admin username if needed
     if (isset($options['adminuser']) and $options['adminuser'] !== 'admin' and $options['adminuser'] !== 'guest') {
         $DB->set_field('user', 'username', $options['adminuser'], array('username' => 'admin'));
     }
 
+    // Set the support email address if specified.
+    if (isset($options['supportemail'])) {
+        set_config('supportemail', $options['supportemail']);
+    }
+
     // indicate that this site is fully configured
     set_config('rolesactive', 1);
     upgrade_finished();
@@ -500,8 +518,7 @@ function install_cli_database(array $options, $interactive) {
     // log in as admin - we need do anything when applying defaults
     \core\session\manager::set_user(get_admin());
 
-    // apply all default settings, do it twice to fill all defaults - some settings depend on other setting
-    admin_apply_default_settings(NULL, true);
+    // Apply all default settings.
     admin_apply_default_settings(NULL, true);
     set_config('registerauth', '');
 
@@ -512,4 +529,15 @@ function install_cli_database(array $options, $interactive) {
     if (isset($options['fullname']) and $options['fullname'] !== '') {
         $DB->set_field('course', 'fullname', $options['fullname'], array('format' => 'site'));
     }
+    if (isset($options['summary'])) {
+        $DB->set_field('course', 'summary', $options['summary'], array('format' => 'site'));
+    }
+
+    // Redirect to site registration on first login.
+    set_config('registrationpending', 1);
+
+    // Apply default preset, if it is defined in $CFG and has a valid value.
+    if (!empty($CFG->setsitepresetduringinstall)) {
+        \core_adminpresets\helper::change_default_preset($CFG->setsitepresetduringinstall);
+    }
 }
