@@ -129,6 +129,9 @@ Y.extend(DRAGDROP, Y.Base, {
         // Listen for all drag:start events.
         this.listeners.push(Y.DD.DDM.on('drag:start', this.global_drag_start, this));
 
+        // Listen for all drag:over events.
+        this.listeners.push(Y.DD.DDM.on('drag:over', this.globalDragOver, this));
+
         // Listen for all drag:end events.
         this.listeners.push(Y.DD.DDM.on('drag:end', this.global_drag_end, this));
 
@@ -224,8 +227,8 @@ Y.extend(DRAGDROP, Y.Base, {
         return ret;
     },
     /*
-        * Drag-dropping related functions
-        */
+     * Drag-dropping related functions
+     */
     global_drag_start: function(e) {
         // Get our drag object
         var drag = e.target;
@@ -246,6 +249,15 @@ Y.extend(DRAGDROP, Y.Base, {
         this.drag_start(e);
     },
 
+    /**
+     * Drag-dropping related functions
+     *
+     * @param {EventFacade} e
+     */
+    globalDragOver: function(e) {
+        this.dragOver(e);
+    },
+
     global_drag_end: function(e) {
         var drag = e.target;
         // Check that drag object belongs to correct group
@@ -687,6 +699,14 @@ Y.extend(DRAGDROP, Y.Base, {
      */
     drag_start: function() {},
 
+    /**
+     * Callback to use for the drag:over event.
+     *
+     * @method dragOver
+     * @param {EventFacade} e
+     */
+    dragOver: function() {},
+
     /**
      * Callback to use when dragging ends.
      *
