@@ -14,15 +14,22 @@
 // You should have received a copy of the GNU General Public License
 // along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
 
-namespace core_message;
-
-use core_message\tests\helper as testhelper;
+/**
+ * Test api's in message lib.
+ *
+ * @package core_message
+ * @category test
+ * @copyright 2014 Rajesh Taneja <rajesh@moodle.com>
+ * @license http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
+ */
 
 defined('MOODLE_INTERNAL') || die();
 
 global $CFG;
 require_once($CFG->dirroot . '/message/lib.php');
 
+use \core_message\tests\helper as testhelper;
+
 /**
  * Test api's in message lib.
  *
@@ -31,7 +38,7 @@ require_once($CFG->dirroot . '/message/lib.php');
  * @copyright 2014 Rajesh Taneja <rajesh@moodle.com>
  * @license http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
  */
-class messagelib_test extends \advanced_testcase {
+class core_message_messagelib_testcase extends advanced_testcase {
 
     /** @var phpunit_message_sink keep track of messages. */
     protected $messagesink = null;
@@ -54,8 +61,8 @@ class messagelib_test extends \advanced_testcase {
      * sent from a user to another. We should stop using it once {@link message_send()} will support
      * transactions. This is not clean at all, this is just used to add rows to the table.
      *
-     * @param \stdClass $userfrom user object of the one sending the message.
-     * @param \stdClass $userto user object of the one receiving the message.
+     * @param stdClass $userfrom user object of the one sending the message.
+     * @param stdClass $userto user object of the one receiving the message.
      * @param string $message message to send.
      * @param int $notification if the message is a notification.
      * @param int $time the time the message was sent
@@ -69,7 +76,7 @@ class messagelib_test extends \advanced_testcase {
         }
 
         if ($notification) {
-            $record = new \stdClass();
+            $record = new stdClass();
             $record->useridfrom = $userfrom->id;
             $record->useridto = $userto->id;
             $record->subject = 'No subject';
@@ -103,7 +110,7 @@ class messagelib_test extends \advanced_testcase {
         }
 
         // Ok, send the message.
-        $record = new \stdClass();
+        $record = new stdClass();
         $record->useridfrom = $userfrom->id;
         $record->conversationid = $conversationid;
         $record->subject = 'No subject';
@@ -256,6 +263,8 @@ class messagelib_test extends \advanced_testcase {
      * Test message_get_messages.
      */
     public function test_message_get_messages() {
+        global $DB;
+
         $this->resetAfterTest(true);
 
         // Set this user as the admin.
@@ -280,12 +289,35 @@ class messagelib_test extends \advanced_testcase {
         $im3 = testhelper::send_fake_message_to_conversation($user1, $ic1->id, 'Message 3');
         $im4 = testhelper::send_fake_message_to_conversation($user1, $ic2->id, 'Message 4');
 
-        // Retrieve all messages sent from user1 to user2.
-        $lastmessages = message_get_messages($user2->id, $user1->id, 0, false);
+        // Mark a message as read by user2.
+        $message = $DB->get_record('messages', ['id' => $im1]);
+        \core_message\api::mark_message_as_read($user2->id, $message);
+
+        // Retrieve unread messages sent from user1 to user2.
+        $lastmessages = message_get_messages($user2->id, $user1->id, 0, MESSAGE_GET_UNREAD);
+        $this->assertCount(1, $lastmessages);
+        $this->assertArrayHasKey($im3, $lastmessages);
+
+        // Get only read messages.
+        $lastmessages = message_get_messages($user2->id, $user1->id, 0, MESSAGE_GET_READ);
+        $this->assertCount(1, $lastmessages);
+        $this->assertArrayHasKey($im1, $lastmessages);
+
+        // Get both read and unread.
+        $lastmessages = message_get_messages($user2->id, $user1->id, 0, MESSAGE_GET_READ_AND_UNREAD);
         $this->assertCount(2, $lastmessages);
         $this->assertArrayHasKey($im1, $lastmessages);
         $this->assertArrayHasKey($im3, $lastmessages);
 
+        // Repeat retrieve read/unread messages but using a bool to test backwards compatibility.
+        $lastmessages = message_get_messages($user2->id, $user1->id, 0, false);
+        $this->assertCount(1, $lastmessages);
+        $this->assertArrayHasKey($im3, $lastmessages);
+
+        $lastmessages = message_get_messages($user2->id, $user1->id, 0, true);
+        $this->assertCount(1, $lastmessages);
+        $this->assertArrayHasKey($im1, $lastmessages);
+
         // Create some group conversations.
         $gc1 = \core_message\api::create_conversation(\core_message\api::MESSAGE_CONVERSATION_TYPE_GROUP,
             [$user1->id, $user2->id, $user3->id], 'Group chat');
@@ -295,7 +327,7 @@ class messagelib_test extends \advanced_testcase {
 
         // Retrieve all messages sent from user1 to user2 (the result should be the same as before, because only individual
         // conversations should be considered by the message_get_messages function).
-        $lastmessages = message_get_messages($user2->id, $user1->id, 0, false);
+        $lastmessages = message_get_messages($user2->id, $user1->id, 0, MESSAGE_GET_READ_AND_UNREAD);
         $this->assertCount(2, $lastmessages);
         $this->assertArrayHasKey($im1, $lastmessages);
         $this->assertArrayHasKey($im3, $lastmessages);
@@ -324,7 +356,7 @@ class messagelib_test extends \advanced_testcase {
 
         // Retrieve all messages sent from user1 to user2. There shouldn't be messages, because only individual
         // conversations should be considered by the message_get_messages function.
-        $lastmessages = message_get_messages($user2->id, $user1->id, 0, false);
+        $lastmessages = message_get_messages($user2->id, $user1->id, 0, MESSAGE_GET_READ_AND_UNREAD);
         $this->assertCount(0, $lastmessages);
     }
 
