@@ -259,7 +259,7 @@ function mnet_encrypt_message($message, $remote_certificate) {
     // Generate a key resource from the remote_certificate text string
     $publickey = openssl_get_publickey($remote_certificate);
 
-    if ($publickey === false) {
+    if ( gettype($publickey) != 'resource' ) {
         // Remote certificate is faulty.
         return false;
     }
@@ -318,6 +318,8 @@ function mnet_get_keypair() {
     if (!is_null($keypair)) return $keypair;
     if ($result = get_config('mnet', 'openssl')) {
         list($keypair['certificate'], $keypair['keypair_PEM']) = explode('@@@@@@@@', $result);
+        $keypair['privatekey'] = openssl_pkey_get_private($keypair['keypair_PEM']);
+        $keypair['publickey']  = openssl_pkey_get_public($keypair['certificate']);
         return $keypair;
     } else {
         $keypair = mnet_generate_keypair();
@@ -922,3 +924,15 @@ function mnet_strip_user($user, $fields) {
     }
     return $user;
 }
+
+/**
+ * Return the deprecation notice of the Mnet.
+ *
+ * @return \core\output\notification
+ */
+function mnet_get_deprecation_notice(): \core\output\notification {
+    $notice = new \core\output\notification(get_string('xmlrpcmnetenabled', 'admin'),
+        \core\output\notification::NOTIFY_WARNING);
+
+    return $notice;
+}
