@@ -27,7 +27,7 @@ $string['action'] = 'Action';
 $string['add'] = 'Add entry';
 $string['addcomment'] = 'Add comment';
 $string['addentries'] = 'Add entries';
-$string['addtemplate'] = 'Add template';
+$string['addtemplate'] = 'Add entry template';
 $string['advancedsearch'] = 'Advanced search';
 $string['allowcomments'] = 'Allow comments on entries';
 $string['alttext'] = 'Alternative text';
@@ -95,7 +95,7 @@ $string['data:exportuserinfo'] = 'Export user information';
 $string['data:managecomments'] = 'Manage comments';
 $string['data:manageentries'] = 'Manage entries';
 $string['data:managetemplates'] = 'Manage templates';
-$string['data:manageuserpresets'] = 'Manage presets';
+$string['data:manageuserpresets'] = 'Manage all template presets';
 $string['data:rate'] = 'Rate entries';
 $string['data:readentry'] = 'Read entries';
 $string['data:viewalluserpresets'] = 'View presets from all users';
@@ -115,7 +115,8 @@ $string['delete'] = 'Delete';
 $string['deleteallentries'] = 'Delete all entries';
 $string['deletecomment'] = 'Are you sure you want to delete this comment?';
 $string['deleted'] = 'deleted';
-$string['deletefield'] = 'Delete an existing field';
+$string['deleteentry'] = 'Delete entry';
+$string['deletefield'] = 'Delete field';
 $string['deletenotenrolled'] = 'Delete entries by users not enrolled';
 $string['deletewarning'] = 'Are you sure you want to delete this preset?';
 $string['descending'] = 'Descending';
@@ -125,6 +126,7 @@ $string['download'] = 'Download';
 $string['edit'] = 'Edit';
 $string['editcomment'] = 'Edit comment';
 $string['editentry'] = 'Edit entry';
+$string['editfield'] = 'Edit field';
 $string['editordisable'] = 'Disable editor';
 $string['editorenable'] = 'Enable editor';
 $string['emptyadd'] = 'The Add template is empty, generating a default form...';
@@ -157,6 +159,7 @@ $string['exportentries'] = 'Export entries';
 $string['exportformat'] = 'Export format';
 $string['exportoptions'] = 'Export options';
 $string['exportownentries'] = 'Export your own entries only? ({$a->mine}/{$a->all})';
+$string['exportpreset'] = 'Export preset';
 $string['failedpresetdelete'] = 'Error deleting a preset!';
 $string['fieldadded'] = 'Field added';
 $string['fieldallowautolink'] = 'Allow autolink';
@@ -176,6 +179,7 @@ $string['fieldnametype'] = '{$a->name} ({$a->type})';
 $string['fieldnotmatched'] = 'The following fields in your file are not known in this database: {$a}';
 $string['fieldoptions'] = 'Options (one per line)';
 $string['fields'] = 'Fields';
+$string['fieldsnavigation'] = 'Fields tertiary navigation';
 $string['fieldupdated'] = 'Field updated';
 $string['fieldwidth'] = 'Width';
 $string['fieldwidthlistview'] = 'Width in list view';
@@ -198,12 +202,13 @@ $string['headerjstemplate'] = 'Defines custom Javascript for manipulating the wa
 $string['headerlisttemplate'] = 'Defines browsing interface for multiple entries';
 $string['headerrsstemplate'] = 'Defines appearance of entries in RSS feeds';
 $string['headersingletemplate'] = 'Defines browsing interface for a single entry';
-$string['checkbox'] = 'Checkboxes';
+$string['checkbox'] = 'Checkbox';
 $string['chooseexportfields'] = 'Choose the fields you wish to export';
 $string['chooseexportformat'] = 'Choose the format you wish to export to';
 $string['chooseorupload'] = 'Choose file';
 $string['expired'] = 'Sorry, this activity closed on {$a} and is no longer available';
 $string['importentries'] = 'Import entries';
+$string['importpreset'] = 'Import a preset';
 $string['importsuccess'] = 'The preset has been successfully applied.';
 $string['includeapproval'] = 'Include approval status';
 $string['includetags'] = 'Include tags';
@@ -226,26 +231,23 @@ $string['invalidfieldid'] = 'Field ID is incorrect';
 $string['invalidfieldname'] = 'Please choose another name for this field';
 $string['invalidfieldtype'] = 'Field type is incorrect';
 $string['invalidid'] = 'Incorrect data ID';
-$string['missingfieldtype'] = 'Field type for {$a->name} not found';
-$string['missingfieldtypes'] = 'The following fields do not have their corresponding field types installed and will not be included in the forms when adding or editing entries.
- Their labels may still show on the form, so please update the "Add entry template" accordingly:';
-$string['missingfieldtypeimport'] = 'The following fields were not imported because their corresponding field types are not installed:';
-$string['unknown'] = 'Unknown field';
 $string['invalidpreset'] = '{$a} is not a preset.';
 $string['invalidrecord'] = 'Incorrect record';
 $string['invalidurl'] = 'The URL you just entered is not valid';
 $string['jstemplate'] = 'Javascript template';
 $string['latitude'] = 'Latitude';
-$string['latlong'] = 'Coordinates';
+$string['latlong'] = 'Latitude/longitude';
 $string['latlongboth'] = 'Both latitude and longitude are required.';
 $string['latlongdownloadallhint'] = 'Download link for all entries as KML';
 $string['latlongkmllabelling'] = 'How to label items in KML files (Google Earth)';
 $string['latlonglinkservicesdisplayed'] = 'Link-out services to display';
 $string['latlongotherfields'] = 'Other fields';
 $string['list'] = 'View list';
+$string['listview'] = 'List view';
 $string['listtemplate'] = 'List template';
 $string['longitude'] = 'Longitude';
 $string['manageapproved'] = 'Allow editing of approved entries';
+$string['managefields'] = 'Manage fields';
 $string['manageapproved_help'] = 'If disabled, approved entries are no longer editable or deletable by the user who added them. This setting has no effect unless \'Approval required\' is enabled.';
 $string['mapexistingfield'] = 'Map to {$a}';
 $string['mapnewfield'] = 'Create a new field';
@@ -258,18 +260,18 @@ $string['menuchoose'] = 'Choose...';
 $string['missingdata'] = 'Data id or object must be provided to field class';
 $string['missingfield'] = 'Programmer error: You must specify field and/or data when defining field class.';
 $string['modulename'] = 'Database';
-$string['modulename_help'] = 'Database enables participants to create, maintain, and search a collection of entries.
+$string['modulename_help'] = 'The database activity module enables participants to create, maintain and search a collection of entries (i.e. records).  The structure of the entries is defined by the teacher as a number of fields. Field types include checkbox, radio buttons, drop-down menu, text area, URL, picture and uploaded file.
 
-The structure of the entries is defined using different types of fields, such as short text, dropdown list, text area, URL, image, or file.
+The visual layout of information when listing, viewing or editing database entries may be controlled by database templates. Database activities may be shared between courses as presets and a teacher may also import and export database entries.
 
-As a teacher, you can comment and rate entries. You can also allow students to rate and comment on entries (peer evaluation). Ratings can be aggregated to form a final grade, which is recorded in the gradebook.
+If the database auto-linking filter is enabled, any entries in a database will be automatically linked where the words or phrases appear within the course.
 
-If the Database auto-linking filter is enabled, any entries in a database will be automatically linked where the words or phrases appear within the course.
+A teacher can allow comments on entries. Entries can also be rated by teachers or students (peer evaluation). Ratings can be aggregated to form a final grade which is recorded in the gradebook.
 
-You can use Database to:
+Database activities have many uses, such as
 
-* Create a collaborative collection of links, books, book reviews, journal references, etc.
-* Display student-created photos, posters, websites or poems for peer comment and review.';
+* A collaborative collection of web links, books, book reviews, journal references etc
+* For displaying student-created photos, posters, websites or poems for peer comment and review.';
 $string['modulename_link'] = 'mod/data/view';
 $string['modulenameplural'] = 'Databases';
 $string['more'] = 'More';
@@ -316,6 +318,7 @@ $string['pluginadministration'] = 'Database activity administration';
 $string['pluginname'] = 'Database';
 $string['portfolionotfile'] = 'Export to a portfolio rather than a file (csv and leap2a only)';
 $string['presetinfo'] = 'Saving as a preset will publish this template. Other users may be able to use it in their databases.';
+$string['presetnotselected'] = 'No preset has been selected.';
 $string['presets'] = 'Presets';
 $string['privacy:metadata:commentpurpose'] = 'Comments on database records';
 $string['privacy:metadata:data_content'] = 'Represents one answer to one field in database activity module';
@@ -369,6 +372,8 @@ $string['saveandadd'] = 'Save and add another';
 $string['saveandview'] = 'Save and view';
 $string['saveaspreset'] = 'Save as preset';
 $string['saveaspreset_help'] = 'The save as preset feature publishes the templates and fields as a preset which others on the site can then use. (You may delete it from the list of presets at any time.)';
+$string['savedataaspreset'] = 'Save all fields and templates as preset';
+$string['saveaspresetmissingcapability'] = 'The user does not have permission to save the database as a preset.';
 $string['savesettings'] = 'Save settings';
 $string['savesuccess'] = 'Saved successfully. Your preset will now be available across the site.';
 $string['savetemplate'] = 'Save template';
@@ -380,6 +385,7 @@ $string['selectfields'] = 'Select fields';
 $string['selectexportoptions'] = 'Select export options';
 $string['showall'] = 'Show all entries';
 $string['single'] = 'View single';
+$string['singleview'] = 'Single view';
 $string['singletemplate'] = 'Single template';
 $string['subplugintype_datafield'] = 'Database field type';
 $string['subplugintype_datafield_plural'] = 'Database field types';
@@ -390,9 +396,10 @@ $string['tags'] = 'Tags';
 $string['tagsdeleted'] = 'Database tags have been deleted';
 $string['teachersandstudents'] = '{$a->teachers} and {$a->students}';
 $string['templates'] = 'Templates';
+$string['templatesnavigation'] = 'Templates tertiary navigation';
 $string['templatesaved'] = 'Template saved';
 $string['text'] = 'Text';
-$string['textarea'] = 'Text area';
+$string['textarea'] = 'Textarea';
 $string['timeadded'] = 'Time added';
 $string['timemodified'] = 'Time modified';
 $string['todatabase'] = 'to this database.';
@@ -413,9 +420,12 @@ The field enclosure is a character that surrounds each field in each record. It
 $string['uploadrecords_link'] = 'mod/data/import';
 $string['url'] = 'URL';
 $string['usedate'] = 'Include in search.';
+$string['usepredefinedset'] = 'Use predefined set';
+$string['usepreset'] = 'Use preset';
 $string['usestandard'] = 'Use a preset';
 $string['usestandard_help'] = 'To use a preset available to the whole site, select it from the list. (If you have added a preset to the list using the save as preset feature then you have the option of deleting it.)';
 $string['viewfromdate'] = 'Read only from';
+$string['viewnavigation'] = 'View mode tertiary navigation';
 $string['viewtodate'] = 'Read only to';
 $string['viewtodatevalidation'] = 'The read only to date cannot be before the read only from date.';
 $string['wrongdataid'] = 'Wrong data id provided';
