@@ -46,12 +46,17 @@
  * @author     Chris Scribner
  * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
  */
+namespace mod_lti;
+
+use mod_lti_external;
+use mod_lti_testcase;
 
 defined('MOODLE_INTERNAL') || die;
 
 global $CFG;
 require_once($CFG->dirroot . '/mod/lti/locallib.php');
 require_once($CFG->dirroot . '/mod/lti/servicelib.php');
+require_once($CFG->dirroot . '/mod/lti/tests/mod_lti_testcase.php');
 
 /**
  * Local library tests
@@ -60,12 +65,12 @@ require_once($CFG->dirroot . '/mod/lti/servicelib.php');
  * @copyright  Copyright (c) 2012 Moodlerooms Inc. (http://www.moodlerooms.com)
  * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
  */
-class mod_lti_locallib_testcase extends advanced_testcase {
+class locallib_test extends mod_lti_testcase {
 
     public function test_split_custom_parameters() {
         $this->resetAfterTest();
 
-        $tool = new stdClass();
+        $tool = new \stdClass();
         $tool->enabledcapability = '';
         $tool->parameter = '';
         $tool->ltiversion = 'LTI-1p0';
@@ -279,8 +284,8 @@ class mod_lti_locallib_testcase extends advanced_testcase {
      * Tests lti_prepare_type_for_save's handling of the "Force SSL" configuration.
      */
     public function test_lti_prepare_type_for_save_forcessl() {
-        $type = new stdClass();
-        $config = new stdClass();
+        $type = new \stdClass();
+        $config = new \stdClass();
 
         // Try when the forcessl config property is not set.
         lti_prepare_type_for_save($type, $config);
@@ -307,7 +312,7 @@ class mod_lti_locallib_testcase extends advanced_testcase {
      * Tests lti_load_type_from_cartridge and lti_load_type_if_cartridge
      */
     public function test_lti_load_type_from_cartridge() {
-        $type = new stdClass();
+        $type = new \stdClass();
         $type->lti_toolurl = $this->getExternalTestFileUrl('/ims_cartridge_basic_lti_link.xml');
 
         lti_load_type_if_cartridge($type);
@@ -323,7 +328,7 @@ class mod_lti_locallib_testcase extends advanced_testcase {
      * Tests lti_load_tool_from_cartridge and lti_load_tool_if_cartridge
      */
     public function test_lti_load_tool_from_cartridge() {
-        $lti = new stdClass();
+        $lti = new \stdClass();
         $lti->toolurl = $this->getExternalTestFileUrl('/ims_cartridge_basic_lti_link.xml');
 
         lti_load_tool_if_cartridge($lti);
@@ -347,8 +352,8 @@ class mod_lti_locallib_testcase extends advanced_testcase {
         $proxy = mod_lti_external::create_tool_proxy('Test proxy', $this->getExternalTestFileUrl('/test.html'), array(), array());
 
         // Create a tool type, associated with that proxy.
-        $type = new stdClass();
-        $data = new stdClass();
+        $type = new \stdClass();
+        $data = new \stdClass();
         $data->lti_contentitem = true;
         $type->state = LTI_TOOL_STATE_CONFIGURED;
         $type->name = "Test tool";
@@ -361,7 +366,7 @@ class mod_lti_locallib_testcase extends advanced_testcase {
         $typeconfig = lti_get_type_config($typeid);
 
         $course = $this->getDataGenerator()->create_course();
-        $returnurl = new moodle_url('/');
+        $returnurl = new \moodle_url('/');
 
         // Default parameters.
         $result = lti_build_content_item_selection_request($typeid, $course, $returnurl);
@@ -433,7 +438,7 @@ class mod_lti_locallib_testcase extends advanced_testcase {
 
         $this->setAdminUser();
         $course = $this->getDataGenerator()->create_course();
-        $returnurl = new moodle_url('/');
+        $returnurl = new \moodle_url('/');
 
         // Should throw Exception on non-existent tool type.
         $this->expectException('moodle_exception');
@@ -449,8 +454,8 @@ class mod_lti_locallib_testcase extends advanced_testcase {
         $this->setAdminUser();
 
         // Create a tool type, associated with that proxy.
-        $type = new stdClass();
-        $data = new stdClass();
+        $type = new \stdClass();
+        $data = new \stdClass();
         $data->lti_contentitem = true;
         $type->state = LTI_TOOL_STATE_CONFIGURED;
         $type->name = "Test tool";
@@ -459,7 +464,7 @@ class mod_lti_locallib_testcase extends advanced_testcase {
 
         $typeid = lti_add_type($type, $data);
         $course = $this->getDataGenerator()->create_course();
-        $returnurl = new moodle_url('/');
+        $returnurl = new \moodle_url('/');
 
         // Should throw coding_exception on non-array media types.
         $mediatypes = 'image/*,video/*';
@@ -476,8 +481,8 @@ class mod_lti_locallib_testcase extends advanced_testcase {
         $this->setAdminUser();
 
         // Create a tool type, associated with that proxy.
-        $type = new stdClass();
-        $data = new stdClass();
+        $type = new \stdClass();
+        $data = new \stdClass();
         $data->lti_contentitem = true;
         $type->state = LTI_TOOL_STATE_CONFIGURED;
         $type->name = "Test tool";
@@ -486,7 +491,7 @@ class mod_lti_locallib_testcase extends advanced_testcase {
 
         $typeid = lti_add_type($type, $data);
         $course = $this->getDataGenerator()->create_course();
-        $returnurl = new moodle_url('/');
+        $returnurl = new \moodle_url('/');
 
         // Should throw coding_exception on non-array presentation targets.
         $targets = 'frame,iframe';
@@ -934,7 +939,7 @@ class mod_lti_locallib_testcase extends advanced_testcase {
                 'claim' => 'url',
                 'isarray' => false
             ],
-            'custom_context_memberships_url' => [
+            'custom_context_memberships_v2_url' => [
                 'suffix' => 'nrps',
                 'group' => 'namesroleservice',
                 'claim' => 'context_memberships_url',
@@ -1066,13 +1071,13 @@ class mod_lti_locallib_testcase extends advanced_testcase {
         $this->setAdminUser();
 
         // Create a tool type, associated with that proxy.
-        $type = new stdClass();
+        $type = new \stdClass();
         $type->state = LTI_TOOL_STATE_CONFIGURED;
         $type->name = "Test tool";
         $type->description = "Example description";
         $type->baseurl = $this->getExternalTestFileUrl('/test.html');
 
-        $config = new stdClass();
+        $config = new \stdClass();
         $config->lti_publickey = '-----BEGIN PUBLIC KEY-----
 MIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAnzyis1ZjfNB0bBgKFMSv
 vkTtwlvBsaJq7S5wA+kzeVOVpVWwkWdVha4s38XM/pa/yr47av7+z3VTmvDRyAHc
@@ -1103,13 +1108,13 @@ MwIDAQAB
         $this->setAdminUser();
 
         // Create a tool type, associated with that proxy.
-        $type = new stdClass();
+        $type = new \stdClass();
         $type->state = LTI_TOOL_STATE_CONFIGURED;
         $type->name = "Test tool";
         $type->description = "Example description";
         $type->baseurl = $this->getExternalTestFileUrl('/test.html');
 
-        $config = new stdClass();
+        $config = new \stdClass();
         $config->lti_publickeyset = dirname(__FILE__) . '/fixtures/test_keyset';
 
         $config->lti_keytype = LTI_JWK_KEYSET;
@@ -1146,14 +1151,14 @@ MwIDAQAB
         $proxy = mod_lti_external::create_tool_proxy('Test proxy', $this->getExternalTestFileUrl('/test.html'), array(), array());
 
         // Create a tool type, associated with that proxy.
-        $type = new stdClass();
+        $type = new \stdClass();
         $type->state = LTI_TOOL_STATE_CONFIGURED;
         $type->name = "Test tool";
         $type->description = "Example description";
         $type->toolproxyid = $proxy->id;
         $type->baseurl = $this->getExternalTestFileUrl('/test.html');
 
-        $data = new stdClass();
+        $data = new \stdClass();
         $data->lti_contentitem = true;
 
         $typeid = lti_add_type($type, $data);
@@ -1171,14 +1176,14 @@ MwIDAQAB
         $this->setAdminUser();
 
         // Create a tool type, associated with that proxy.
-        $type = new stdClass();
+        $type = new \stdClass();
         $type->state = LTI_TOOL_STATE_CONFIGURED;
         $type->name = "Test tool";
         $type->description = "Example description";
         $type->clientid = 'consumerkey';
         $type->baseurl = $this->getExternalTestFileUrl('/test.html');
 
-        $config = new stdClass();
+        $config = new \stdClass();
         $typeid = lti_add_type($type, $config);
 
         $this->expectExceptionMessage(get_string('errorincorrectconsumerkey', 'mod_lti'));
@@ -1193,14 +1198,14 @@ MwIDAQAB
         $this->setAdminUser();
 
         // Create a tool type, associated with that proxy.
-        $type = new stdClass();
+        $type = new \stdClass();
         $type->state = LTI_TOOL_STATE_CONFIGURED;
         $type->name = "Test tool";
         $type->description = "Example description";
         $type->clientid = 'consumerkey';
         $type->baseurl = $this->getExternalTestFileUrl('/test.html');
 
-        $config = new stdClass();
+        $config = new \stdClass();
         $config->lti_keytype = LTI_RSA_KEY;
         $typeid = lti_add_type($type, $config);
 
@@ -1252,38 +1257,38 @@ MwIDAQAB
         $strgraph = '@graph';
         $strtype = '@type';
 
-        $objgraph = new stdClass();
+        $objgraph = new \stdClass();
         $objgraph->url = 'http://example.com/messages/launch';
         $objgraph->title = 'Test title';
         $objgraph->text = 'Test text';
-        $objgraph->placementAdvice = new stdClass();
+        $objgraph->placementAdvice = new \stdClass();
         $objgraph->placementAdvice->presentationDocumentTarget = 'iframe';
         $objgraph->{$strtype} = 'LtiLinkItem';
         $objgraph->mediaType = 'application\/vnd.ims.lti.v1.ltilink';
 
-        $objgraph2 = new stdClass();
+        $objgraph2 = new \stdClass();
         $objgraph2->url = 'http://example.com/messages/launch2';
         $objgraph2->title = 'Test title2';
         $objgraph2->text = 'Test text2';
-        $objgraph2->placementAdvice = new stdClass();
+        $objgraph2->placementAdvice = new \stdClass();
         $objgraph2->placementAdvice->presentationDocumentTarget = 'iframe';
         $objgraph2->placementAdvice->displayHeight = 200;
         $objgraph2->placementAdvice->displayWidth = 300;
         $objgraph2->{$strtype} = 'LtiLinkItem';
         $objgraph2->mediaType = 'application\/vnd.ims.lti.v1.ltilink';
 
-        $objgraph3 = new stdClass();
+        $objgraph3 = new \stdClass();
         $objgraph3->url = 'http://example.com/messages/launch3';
         $objgraph3->title = 'Test title3';
         $objgraph3->text = 'Test text3';
-        $objgraph3->placementAdvice = new stdClass();
+        $objgraph3->placementAdvice = new \stdClass();
         $objgraph3->placementAdvice->presentationDocumentTarget = 'window';
         $objgraph3->placementAdvice->displayHeight = 400;
         $objgraph3->placementAdvice->windowTarget = 'test-win';
         $objgraph3->{$strtype} = 'LtiLinkItem';
         $objgraph3->mediaType = 'application\/vnd.ims.lti.v1.ltilink';
 
-        $expected = new stdClass();
+        $expected = new \stdClass();
         $expected->{$strcontext} = 'http://purl.imsglobal.org/ctx/lti/v1/ContentItem';
         $expected->{$strgraph} = [];
         $expected->{$strgraph}[] = $objgraph;
@@ -1300,10 +1305,10 @@ MwIDAQAB
         $this->resetAfterTest();
         $this->setAdminUser();
 
-        $type = new stdClass();
+        $type = new \stdClass();
         $type->name = "Test tool";
         $type->baseurl = "http://example.com";
-        $config = new stdClass();
+        $config = new \stdClass();
         $config->lti_acceptgrades = LTI_SETTING_DELEGATE;
         $typeid = lti_add_type($type, $config);
 
@@ -1343,10 +1348,10 @@ MwIDAQAB
         $this->resetAfterTest();
         $this->setAdminUser();
 
-        $type = new stdClass();
+        $type = new \stdClass();
         $type->name = "Test tool";
         $type->baseurl = "http://example.com";
-        $config = new stdClass();
+        $config = new \stdClass();
         $config->lti_acceptgrades = LTI_SETTING_DELEGATE;
         $typeid = lti_add_type($type, $config);
 
@@ -1401,10 +1406,10 @@ MwIDAQAB
         $this->resetAfterTest();
         $this->setAdminUser();
 
-        $type = new stdClass();
+        $type = new \stdClass();
         $type->name = "Test tool";
         $type->baseurl = "http://example.com";
-        $config = new stdClass();
+        $config = new \stdClass();
         $typeid = lti_add_type($type, $config);
 
         $generator = $this->getDataGenerator()->get_plugin_generator('mod_lti');
@@ -1445,14 +1450,14 @@ MwIDAQAB
         $this->setAdminUser();
 
         // Create a tool type, associated with that proxy.
-        $type = new stdClass();
+        $type = new \stdClass();
         $type->state = LTI_TOOL_STATE_CONFIGURED;
         $type->name = "Test tool";
         $type->description = "Example description";
         $type->clientid = 'consumerkey';
         $type->baseurl = $this->getExternalTestFileUrl('/test.html');
 
-        $config = new stdClass();
+        $config = new \stdClass();
         $typeid = lti_add_type($type, $config);
 
         $params = [];
@@ -1491,14 +1496,14 @@ MwIDAQAB
         $this->setAdminUser();
 
         // Create a tool type, associated with that proxy.
-        $type = new stdClass();
+        $type = new \stdClass();
         $type->state = LTI_TOOL_STATE_CONFIGURED;
         $type->name = "Test tool";
         $type->description = "Example description";
         $type->clientid = 'sso.example.com';
         $type->baseurl = $this->getExternalTestFileUrl('/test.html');
 
-        $config = new stdClass();
+        $config = new \stdClass();
         $config->lti_publickey = '-----BEGIN PUBLIC KEY-----
 MIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAnzyis1ZjfNB0bBgKFMSv
 vkTtwlvBsaJq7S5wA+kzeVOVpVWwkWdVha4s38XM/pa/yr47av7+z3VTmvDRyAHc
@@ -1531,13 +1536,13 @@ MwIDAQAB
         $this->setAdminUser();
 
         // Create a tool type, associated with that proxy.
-        $type = new stdClass();
+        $type = new \stdClass();
         $type->state = LTI_TOOL_STATE_CONFIGURED;
         $type->name = "Test tool";
         $type->description = "Example description";
         $type->baseurl = $this->getExternalTestFileUrl('/test.html');
 
-        $typeconfig = new stdClass();
+        $typeconfig = new \stdClass();
         $typeconfig->lti_acceptgrades = true;
 
         $typeid = lti_add_type($type, $typeconfig);
@@ -1562,14 +1567,14 @@ MwIDAQAB
         $this->setAdminUser();
 
         // Create a tool type, associated with that proxy.
-        $type = new stdClass();
+        $type = new \stdClass();
         $type->state = LTI_TOOL_STATE_CONFIGURED;
         $type->name = "Test tool";
         $type->description = "Example description";
         $type->clientid = "Test client ID";
         $type->baseurl = $this->getExternalTestFileUrl('/test.html');
 
-        $config = new stdClass();
+        $config = new \stdClass();
 
         $typeid = lti_add_type($type, $config);
 
@@ -1596,14 +1601,14 @@ MwIDAQAB
         $this->setAdminUser();
 
         // Create a tool type, associated with that proxy.
-        $type = new stdClass();
+        $type = new \stdClass();
         $type->state = LTI_TOOL_STATE_CONFIGURED;
         $type->name = "Test tool";
         $type->description = "Example description";
         $type->clientid = "Test client ID";
         $type->baseurl = $this->getExternalTestFileUrl('/test.html');
 
-        $config = new stdClass();
+        $config = new \stdClass();
 
         $typeid = lti_add_type($type, $config);
 
@@ -1639,7 +1644,7 @@ MwIDAQAB
             ]
         );
 
-        $config = new stdClass();
+        $config = new \stdClass();
         $config->lti_clientid = 'some-client-id';
         $config->typeid = 'some-type-id';
         $config->lti_toolurl = 'some-lti-tool-url';
@@ -1661,7 +1666,7 @@ MwIDAQAB
         global $DB;
         $this->resetAfterTest();
         $this->setAdminUser();
-        $config = new stdClass();
+        $config = new \stdClass();
         $config->lti_organizationid = '';
         $course = $this->getDataGenerator()->create_course();
         $type = $this->create_type($config);
@@ -1677,7 +1682,7 @@ MwIDAQAB
         global $DB;
         $this->resetAfterTest();
         $this->setAdminUser();
-        $config = new stdClass();
+        $config = new \stdClass();
         $config->lti_organizationid = '';
         $config->lti_organizationid_default = LTI_DEFAULT_ORGID_SITEHOST;
         $course = $this->getDataGenerator()->create_course();
@@ -1694,7 +1699,7 @@ MwIDAQAB
         global $DB;
         $this->resetAfterTest();
         $this->setAdminUser();
-        $config = new stdClass();
+        $config = new \stdClass();
         $config->lti_organizationid = '';
         $config->lti_organizationid_default = LTI_DEFAULT_ORGID_SITEID;
         $course = $this->getDataGenerator()->create_course();
@@ -1711,7 +1716,7 @@ MwIDAQAB
         global $DB;
         $this->resetAfterTest();
         $this->setAdminUser();
-        $config = new stdClass();
+        $config = new \stdClass();
         $config->lti_organizationid = 'overridden!';
         $config->lti_organizationid_default = LTI_DEFAULT_ORGID_SITEID;
         $course = $this->getDataGenerator()->create_course();
@@ -1740,6 +1745,220 @@ MwIDAQAB
         $this->assertEquals(get_course_history($course), [38903]);
     }
 
+    /**
+     * Test the lti_get_ims_role helper function.
+     *
+     * @dataProvider lti_get_ims_role_provider
+     * @covers ::lti_get_ims_role()
+     *
+     * @param bool $islti2 whether the method is called with LTI 2.0 role names or not.
+     * @param string $rolename the name of the role (student, teacher, admin)
+     * @param null|string $switchedto the role to switch to, or false if not using the 'switch to' functionality.
+     * @param string $expected the expected role name.
+     */
+    public function test_lti_get_ims_role(bool $islti2, string $rolename, ?string $switchedto, string $expected) {
+        global $DB;
+        $this->resetAfterTest();
+
+        $course = $this->getDataGenerator()->create_course();
+        $user = $rolename == 'admin' ? get_admin() : $this->getDataGenerator()->create_and_enrol($course, $rolename);
+
+        if ($switchedto) {
+            $this->setUser($user);
+            $role = $DB->get_record('role', array('shortname' => $switchedto));
+            role_switch($role->id, \context_course::instance($course->id));
+        }
+
+        $this->assertEquals($expected, lti_get_ims_role($user, 0, $course->id, $islti2));
+    }
+
+    /**
+     * Data provider for testing lti_get_ims_role.
+     *
+     * @return array[] the test case data.
+     */
+    public function lti_get_ims_role_provider() {
+        return [
+            'Student, LTI 1.1, no role switch' => [
+                'islti2' => false,
+                'rolename' => 'student',
+                'switchedto' => null,
+                'expected' => 'Learner'
+            ],
+            'Student, LTI 2.0, no role switch' => [
+                'islti2' => true,
+                'rolename' => 'student',
+                'switchedto' => null,
+                'expected' => 'Learner'
+            ],
+            'Teacher, LTI 1.1, no role switch' => [
+                'islti2' => false,
+                'rolename' => 'editingteacher',
+                'switchedto' => null,
+                'expected' => 'Instructor'
+            ],
+            'Teacher, LTI 2.0, no role switch' => [
+                'islti2' => true,
+                'rolename' => 'editingteacher',
+                'switchedto' => null,
+                'expected' => 'Instructor'
+            ],
+            'Admin, LTI 1.1, no role switch' => [
+                'islti2' => false,
+                'rolename' => 'admin',
+                'switchedto' => null,
+                'expected' => 'Instructor,urn:lti:sysrole:ims/lis/Administrator,urn:lti:instrole:ims/lis/Administrator'
+            ],
+            'Admin, LTI 2.0, no role switch' => [
+                'islti2' => true,
+                'rolename' => 'admin',
+                'switchedto' => null,
+                'expected' => 'Instructor,http://purl.imsglobal.org/vocab/lis/v2/person#Administrator'
+            ],
+            'Admin, LTI 1.1, role switch student' => [
+                'islti2' => false,
+                'rolename' => 'admin',
+                'switchedto' => 'student',
+                'expected' => 'Learner'
+            ],
+            'Admin, LTI 2.0, role switch student' => [
+                'islti2' => true,
+                'rolename' => 'admin',
+                'switchedto' => 'student',
+                'expected' => 'Learner'
+            ],
+            'Admin, LTI 1.1, role switch teacher' => [
+                'islti2' => false,
+                'rolename' => 'admin',
+                'switchedto' => 'editingteacher',
+                'expected' => 'Instructor'
+            ],
+            'Admin, LTI 2.0, role switch teacher' => [
+                'islti2' => true,
+                'rolename' => 'admin',
+                'switchedto' => 'editingteacher',
+                'expected' => 'Instructor'
+            ],
+        ];
+    }
+
+    /**
+     * Test lti_get_lti_types_and_proxies with no limit or offset.
+     */
+    public function test_lti_get_lti_types_and_proxies_with_no_limit() {
+        $this->resetAfterTest();
+        $this->setAdminUser();
+        $this->generate_tool_types_and_proxies(10);
+        list($proxies, $types) = lti_get_lti_types_and_proxies();
+
+        $this->assertCount(10, $proxies);
+        $this->assertCount(10, $types);
+    }
+
+    /**
+     * Test lti_get_lti_types_and_proxies with limits.
+     */
+    public function test_lti_get_lti_types_and_proxies_with_limit() {
+        $this->resetAfterTest();
+        $this->setAdminUser();
+        $this->generate_tool_types_and_proxies(10);
+
+        // Get the middle 10 data sets (of 20 total).
+        list($proxies, $types) = lti_get_lti_types_and_proxies(10, 5);
+
+        $this->assertCount(5, $proxies);
+        $this->assertCount(5, $types);
+
+        // Get the last 5 data sets with large limit (of 20 total).
+        list($proxies, $types) = lti_get_lti_types_and_proxies(50, 15);
+
+        $this->assertCount(0, $proxies);
+        $this->assertCount(5, $types);
+
+        // Get the last 13 data sets with large limit (of 20 total).
+        list($proxies, $types) = lti_get_lti_types_and_proxies(50, 7);
+
+        $this->assertCount(3, $proxies);
+        $this->assertCount(10, $types);
+    }
+
+    /**
+     * Test lti_get_lti_types_and_proxies with limits and only fetching orphaned proxies.
+     */
+    public function test_lti_get_lti_types_and_proxies_with_limit_and_orphaned_proxies() {
+        $this->resetAfterTest();
+        $this->setAdminUser();
+        $this->generate_tool_types_and_proxies(10, 5);
+
+        // Get the first 10 data sets (of 15 total).
+        list($proxies, $types) = lti_get_lti_types_and_proxies(10, 0, true);
+
+        $this->assertCount(5, $proxies);
+        $this->assertCount(5, $types);
+
+        // Get the middle 10 data sets with large limit (of 15 total).
+        list($proxies, $types) = lti_get_lti_types_and_proxies(10, 2, true);
+
+        $this->assertCount(3, $proxies);
+        $this->assertCount(7, $types);
+
+        // Get the last 5 data sets with large limit (of 15 total).
+        list($proxies, $types) = lti_get_lti_types_and_proxies(50, 10, true);
+
+        $this->assertCount(0, $proxies);
+        $this->assertCount(5, $types);
+    }
+
+    /**
+     * Test lti_get_lti_types_and_proxies_count.
+     */
+    public function test_lti_get_lti_types_and_proxies_count_with_no_filters() {
+        $this->resetAfterTest();
+        $this->setAdminUser();
+        $this->generate_tool_types_and_proxies(10, 5);
+
+        $totalcount = lti_get_lti_types_and_proxies_count();
+        $this->assertEquals(25, $totalcount); // 10 types, 15 proxies.
+    }
+
+    /**
+     * Test lti_get_lti_types_and_proxies_count only counting orphaned proxies.
+     */
+    public function test_lti_get_lti_types_and_proxies_count_with_only_orphaned_proxies() {
+        $this->resetAfterTest();
+        $this->setAdminUser();
+        $this->generate_tool_types_and_proxies(10, 5);
+
+        $orphanedcount = lti_get_lti_types_and_proxies_count(true);
+        $this->assertEquals(15, $orphanedcount); // 10 types, 5 proxies.
+    }
+
+    /**
+     * Test lti_get_lti_types_and_proxies_count only matching tool type with toolproxyid.
+     */
+    public function test_lti_get_lti_types_and_proxies_count_type_with_proxyid() {
+        $this->resetAfterTest();
+        $this->setAdminUser();
+        ['proxies' => $proxies, 'types' => $types] = $this->generate_tool_types_and_proxies(10, 5);
+
+        $countwithproxyid = lti_get_lti_types_and_proxies_count(false, $proxies[0]->id);
+        $this->assertEquals(16, $countwithproxyid); // 1 type, 15 proxies.
+    }
+
+    /**
+     * Verify that empty curl responses lead to the proper moodle_exception, not to XML ValueError.
+     *
+     * @covers ::lti_load_cartridge()
+     */
+    public function test_empty_reponse_lti_load_cartridge() {
+        // Mock the curl response to empty string, this is hardly
+        // reproducible in real life (only Windows + GHA).
+        \curl::mock_response('');
+
+        $this->expectException(\moodle_exception::class);
+        lti_load_cartridge('http://example.com/mocked/empty/response', []);
+    }
+
     /**
      * Create an LTI Tool.
      *
@@ -1748,14 +1967,14 @@ MwIDAQAB
      * @return object tool.
      */
     private function create_type(object $config) {
-        $type = new stdClass();
+        $type = new \stdClass();
         $type->state = LTI_TOOL_STATE_CONFIGURED;
         $type->name = "Test tool";
         $type->description = "Example description";
         $type->clientid = "Test client ID";
         $type->baseurl = $this->getExternalTestFileUrl('/test.html');
 
-        $configbase = new stdClass();
+        $configbase = new \stdClass();
         $configbase->lti_acceptgrades = LTI_SETTING_NEVER;
         $configbase->lti_sendname = LTI_SETTING_NEVER;
         $configbase->lti_sendemailaddr = LTI_SETTING_NEVER;
@@ -1779,4 +1998,27 @@ MwIDAQAB
                   'typeid' => $type->id
                   ), array());
     }
+
+    /**
+     * Generate a number of LTI tool types and proxies.
+     *
+     * @param int $toolandproxycount How many tool types and associated proxies to create. E.g. Value of 10 will create 10 types
+     * and 10 proxies.
+     * @param int $orphanproxycount How many orphaned proxies to create.
+     * @return array[]
+     */
+    private function generate_tool_types_and_proxies(int $toolandproxycount = 0, int $orphanproxycount = 0) {
+        $proxies = [];
+        $types = [];
+        for ($i = 0; $i < $toolandproxycount; $i++) {
+            $proxies[$i] = $this->generate_tool_proxy($i);
+            $types[$i] = $this->generate_tool_type($i, $proxies[$i]->id);
+
+        }
+        for ($i = $toolandproxycount; $i < ($toolandproxycount + $orphanproxycount); $i++) {
+            $proxies[$i] = $this->generate_tool_proxy($i);
+        }
+
+        return ['proxies' => $proxies, 'types' => $types];
+    }
 }
