@@ -75,7 +75,7 @@ class mod_scorm_mod_form extends moodleform_mod {
             $mform->addElement('text', 'packageurl', get_string('packageurl', 'scorm'), array('size' => 60));
             $mform->setType('packageurl', PARAM_RAW);
             $mform->addHelpButton('packageurl', 'packageurl', 'scorm');
-            $mform->disabledIf('packageurl', 'scormtype', 'eq', SCORM_TYPE_LOCAL);
+            $mform->hideIf('packageurl', 'scormtype', 'eq', SCORM_TYPE_LOCAL);
         } else {
             $mform->addElement('hidden', 'scormtype', SCORM_TYPE_LOCAL);
             $mform->setType('scormtype', PARAM_ALPHA);
@@ -90,7 +90,7 @@ class mod_scorm_mod_form extends moodleform_mod {
 
         $mform->addElement('filemanager', 'packagefile', get_string('package', 'scorm'), null, $filemanageroptions);
         $mform->addHelpButton('packagefile', 'package', 'scorm');
-        $mform->disabledIf('packagefile', 'scormtype', 'noteq', SCORM_TYPE_LOCAL);
+        $mform->hideIf('packagefile', 'scormtype', 'noteq', SCORM_TYPE_LOCAL);
 
         // Update packages timing.
         $mform->addElement('select', 'updatefreq', get_string('updatefreq', 'scorm'), scorm_get_updatefreq_array());
@@ -111,14 +111,14 @@ class mod_scorm_mod_form extends moodleform_mod {
         $mform->setDefault('width', $cfgscorm->framewidth);
         $mform->setType('width', PARAM_INT);
         $mform->setAdvanced('width', $cfgscorm->framewidth_adv);
-        $mform->disabledIf('width', 'popup', 'eq', 0);
+        $mform->hideIf('width', 'popup', 'eq', 0);
 
         // Height.
         $mform->addElement('text', 'height', get_string('height', 'scorm'), 'maxlength="5" size="5"');
         $mform->setDefault('height', $cfgscorm->frameheight);
         $mform->setType('height', PARAM_INT);
         $mform->setAdvanced('height', $cfgscorm->frameheight_adv);
-        $mform->disabledIf('height', 'popup', 'eq', 0);
+        $mform->hideIf('height', 'popup', 'eq', 0);
 
         // Window Options.
         $winoptgrp = array();
@@ -127,22 +127,11 @@ class mod_scorm_mod_form extends moodleform_mod {
             $mform->setDefault($key, $value);
         }
         $mform->addGroup($winoptgrp, 'winoptgrp', get_string('options', 'scorm'), '<br />', false);
-        $mform->disabledIf('winoptgrp', 'popup', 'eq', 0);
+        $mform->hideIf('winoptgrp', 'popup', 'eq', 0);
         $mform->setAdvanced('winoptgrp', $cfgscorm->winoptgrp_adv);
 
-        // Display activity name.
-        $mform->addElement('advcheckbox', 'displayactivityname', get_string('displayactivityname', 'scorm'));
-        $mform->addHelpButton('displayactivityname', 'displayactivityname', 'scorm');
-        $mform->setDefault('displayactivityname', $cfgscorm->displayactivityname);
-
         // Skip view page.
         $skipviewoptions = scorm_get_skip_view_array();
-        if ($COURSE->format == 'singleactivity') { // Remove option that would cause a constant redirect.
-            unset($skipviewoptions[SCORM_SKIPVIEW_ALWAYS]);
-            if ($cfgscorm->skipview == SCORM_SKIPVIEW_ALWAYS) {
-                $cfgscorm->skipview = SCORM_SKIPVIEW_FIRST;
-            }
-        }
         $mform->addElement('select', 'skipview', get_string('skipview', 'scorm'), $skipviewoptions);
         $mform->addHelpButton('skipview', 'skipview', 'scorm');
         $mform->setDefault('skipview', $cfgscorm->skipview);
@@ -172,23 +161,23 @@ class mod_scorm_mod_form extends moodleform_mod {
         $mform->addHelpButton('nav', 'nav', 'scorm');
         $mform->setDefault('nav', $cfgscorm->nav);
         $mform->setAdvanced('nav', $cfgscorm->nav_adv);
-        $mform->disabledIf('nav', 'hidetoc', 'noteq', SCORM_TOC_SIDE);
+        $mform->hideIf('nav', 'hidetoc', 'noteq', SCORM_TOC_SIDE);
 
         // Navigation panel position from left.
         $mform->addElement('text', 'navpositionleft', get_string('fromleft', 'scorm'), 'maxlength="5" size="5"');
         $mform->setDefault('navpositionleft', $cfgscorm->navpositionleft);
         $mform->setType('navpositionleft', PARAM_INT);
         $mform->setAdvanced('navpositionleft', $cfgscorm->navpositionleft_adv);
-        $mform->disabledIf('navpositionleft', 'hidetoc', 'noteq', SCORM_TOC_SIDE);
-        $mform->disabledIf('navpositionleft', 'nav', 'noteq', SCORM_NAV_FLOATING);
+        $mform->hideIf('navpositionleft', 'hidetoc', 'noteq', SCORM_TOC_SIDE);
+        $mform->hideIf('navpositionleft', 'nav', 'noteq', SCORM_NAV_FLOATING);
 
         // Navigation panel position from top.
         $mform->addElement('text', 'navpositiontop', get_string('fromtop', 'scorm'), 'maxlength="5" size="5"');
         $mform->setDefault('navpositiontop', $cfgscorm->navpositiontop);
         $mform->setType('navpositiontop', PARAM_INT);
         $mform->setAdvanced('navpositiontop', $cfgscorm->navpositiontop_adv);
-        $mform->disabledIf('navpositiontop', 'hidetoc', 'noteq', SCORM_TOC_SIDE);
-        $mform->disabledIf('navpositiontop', 'nav', 'noteq', SCORM_NAV_FLOATING);
+        $mform->hideIf('navpositiontop', 'hidetoc', 'noteq', SCORM_TOC_SIDE);
+        $mform->hideIf('navpositiontop', 'nav', 'noteq', SCORM_NAV_FLOATING);
 
         // Display attempt status.
         $mform->addElement('select', 'displayattemptstatus', get_string('displayattemptstatus', 'scorm'),
@@ -204,7 +193,7 @@ class mod_scorm_mod_form extends moodleform_mod {
         $mform->addElement('date_time_selector', 'timeclose', get_string("scormclose", "scorm"), array('optional' => true));
 
         // Grade Settings.
-        $mform->addElement('header', 'gradesettings', get_string('grade'));
+        $mform->addElement('header', 'gradesettings', get_string('gradenoun'));
 
         // Grade Method.
         $mform->addElement('select', 'grademethod', get_string('grademethod', 'scorm'), scorm_get_grade_method_array());
@@ -217,7 +206,7 @@ class mod_scorm_mod_form extends moodleform_mod {
         }
         $mform->addElement('select', 'maxgrade', get_string('maximumgrade'), $grades);
         $mform->setDefault('maxgrade', $cfgscorm->maxgrade);
-        $mform->disabledIf('maxgrade', 'grademethod', 'eq', GRADESCOES);
+        $mform->hideIf('maxgrade', 'grademethod', 'eq', GRADESCOES);
 
         // Attempts management.
         $mform->addElement('header', 'attemptsmanagementhdr', get_string('attemptsmanagement', 'scorm'));
@@ -229,13 +218,14 @@ class mod_scorm_mod_form extends moodleform_mod {
 
         // What Grade.
         $mform->addElement('select', 'whatgrade', get_string('whatgrade', 'scorm'),  scorm_get_what_grade_array());
-        $mform->disabledIf('whatgrade', 'maxattempt', 'eq', 1);
+        $mform->hideIf('whatgrade', 'maxattempt', 'eq', 1);
         $mform->addHelpButton('whatgrade', 'whatgrade', 'scorm');
         $mform->setDefault('whatgrade', $cfgscorm->whatgrade);
 
         // Force new attempt.
-        $mform->addElement('selectyesno', 'forcenewattempt', get_string('forcenewattempt', 'scorm'));
-        $mform->addHelpButton('forcenewattempt', 'forcenewattempt', 'scorm');
+        $newattemptselect = scorm_get_forceattempt_array();
+        $mform->addElement('select', 'forcenewattempt', get_string('forcenewattempts', 'scorm'), $newattemptselect);
+        $mform->addHelpButton('forcenewattempt', 'forcenewattempts', 'scorm');
         $mform->setDefault('forcenewattempt', $cfgscorm->forcenewattempt);
 
         // Last attempt lock - lock the enter button after the last available attempt has been made.
@@ -280,12 +270,17 @@ class mod_scorm_mod_form extends moodleform_mod {
 
         $this->standard_coursemodule_elements();
 
+        // A SCORM module should define this within itself and is not needed here.
+        if ($mform->elementExists('completionpassgrade')) {
+            $mform->removeElement('completionpassgrade');
+        }
+
         // Buttons.
         $this->add_action_buttons();
     }
 
     public function data_preprocessing(&$defaultvalues) {
-        global $COURSE;
+        global $CFG, $COURSE;
 
         if (isset($defaultvalues['popup']) && ($defaultvalues['popup'] == 1) && isset($defaultvalues['options'])) {
             if (!empty($defaultvalues['options'])) {
@@ -318,10 +313,10 @@ class mod_scorm_mod_form extends moodleform_mod {
 
         if (($COURSE->format == 'singleactivity') && ((count($scorms) == 0) || ($defaultvalues['instance'] == $coursescorm->id))) {
             $defaultvalues['redirect'] = 'yes';
-            $defaultvalues['redirecturl'] = '../course/view.php?id='.$defaultvalues['course'];
+            $defaultvalues['redirecturl'] = $CFG->wwwroot.'/course/view.php?id='.$defaultvalues['course'];
         } else {
             $defaultvalues['redirect'] = 'no';
-            $defaultvalues['redirecturl'] = '../mod/scorm/view.php?id='.$defaultvalues['coursemodule'];
+            $defaultvalues['redirecturl'] = $CFG->wwwroot.'/mod/scorm/view.php?id='.$defaultvalues['coursemodule'];
         }
         if (isset($defaultvalues['version'])) {
             $defaultvalues['pkgtype'] = (substr($defaultvalues['version'], 0, 5) == 'SCORM') ? 'scorm' : 'aicc';
@@ -337,15 +332,19 @@ class mod_scorm_mod_form extends moodleform_mod {
         }
 
         // Set some completion default data.
-        if (!empty($defaultvalues['completionstatusrequired']) && !is_array($defaultvalues['completionstatusrequired'])) {
+        $cvalues = array();
+        if (empty($this->_instance)) {
+            // When in add mode, set a default completion rule that requires the SCORM's status be set to "Completed".
+            $cvalues[4] = 1;
+        } else if (!empty($defaultvalues['completionstatusrequired']) && !is_array($defaultvalues['completionstatusrequired'])) {
             // Unpack values.
-            $cvalues = array();
             foreach (scorm_status_options() as $key => $value) {
                 if (($defaultvalues['completionstatusrequired'] & $key) == $key) {
                     $cvalues[$key] = 1;
                 }
             }
-
+        }
+        if (!empty($cvalues)) {
             $defaultvalues['completionstatusrequired'] = $cvalues;
         }
 
@@ -487,7 +486,6 @@ class mod_scorm_mod_form extends moodleform_mod {
             }
         }
 
-        $this->data_preprocessing($defaultvalues);
         parent::set_data($defaultvalues);
     }
 
@@ -519,10 +517,6 @@ class mod_scorm_mod_form extends moodleform_mod {
                 $firstkey = $key;
             }
             $mform->addElement('checkbox', $key, $name, $value);
-            // Default completion rule that requires the SCORM's status be set to "Completed".
-            if ($key === 'completionstatusrequired[4]') {
-                $mform->setDefault($key, 1);
-            }
             $mform->setType($key, PARAM_BOOL);
             $items[] = $key;
         }
@@ -557,8 +551,13 @@ class mod_scorm_mod_form extends moodleform_mod {
         // Convert completionstatusrequired to a proper integer, if any.
         $total = 0;
         if (isset($data->completionstatusrequired) && is_array($data->completionstatusrequired)) {
-            foreach (array_keys($data->completionstatusrequired) as $state) {
-                $total |= $state;
+            foreach ($data->completionstatusrequired as $state => $value) {
+                if ($value) {
+                    $total |= $state;
+                }
+            }
+            if (!$total) {
+                $total  = null;
             }
             $data->completionstatusrequired = $total;
         }
