@@ -22,6 +22,8 @@
  * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
  */
 
+use core\report_helper;
+
 require('../../config.php');
 require_once($CFG->dirroot.'/course/lib.php');
 require_once($CFG->dirroot.'/report/log/locallib.php');
@@ -48,8 +50,7 @@ $params = array();
 if (!empty($id)) {
     $params['id'] = $id;
 } else {
-    $site = get_site();
-    $id = $site->id;
+    $id = $SITE->id;
 }
 if ($group !== 0) {
     $params['group'] = $group;
@@ -104,12 +105,12 @@ $PAGE->set_url('/report/log/index.php', array('id' => $id));
 $PAGE->set_pagelayout('report');
 
 // Get course details.
-$course = null;
-if ($id) {
+if ($id != $SITE->id) {
     $course = $DB->get_record('course', array('id' => $id), '*', MUST_EXIST);
     require_login($course);
     $context = context_course::instance($course->id);
 } else {
+    $course = $SITE;
     require_login();
     $context = context_system::instance();
     $PAGE->set_context($context);
@@ -140,9 +141,10 @@ if ($PAGE->user_allowed_editing() && $adminediting != -1) {
     $USER->editing = $adminediting;
 }
 
-if (empty($course) || ($course->id == $SITE->id)) {
+if ($course->id == $SITE->id) {
     admin_externalpage_setup('reportlog', '', null, '', array('pagelayout' => 'report'));
     $PAGE->set_title($SITE->shortname .': '. $strlogs);
+    $PAGE->set_primary_active_tab('siteadminnode');
 } else {
     $PAGE->set_title($course->shortname .': '. $strlogs);
     $PAGE->set_heading($course->fullname);
@@ -163,6 +165,9 @@ if (empty($readers)) {
 
         if (empty($logformat)) {
             echo $output->header();
+            // Print selector dropdown.
+            $pluginname = get_string('pluginname', 'report_log');
+            report_helper::print_report_selector($pluginname);
             $userinfo = get_string('allparticipants');
             $dateinfo = get_string('alldays');
 
@@ -184,7 +189,10 @@ if (empty($readers)) {
         }
     } else {
         echo $output->header();
-        echo $output->heading(get_string('chooselogs') .':');
+        // Print selector dropdown.
+        $pluginname = get_string('pluginname', 'report_log');
+        report_helper::print_report_selector($pluginname);
+        echo $output->heading(get_string('chooselogs') .':', 3);
         echo $output->render($reportlog);
     }
 }
