@@ -17,8 +17,7 @@
 /**
  * Lang strings
  *
- * @package    report
- * @subpackage security
+ * @package    report_security
  * @copyright  2008 petr Skoda
  * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
  */
@@ -26,13 +25,14 @@
 $string['configuration'] = 'Configuration';
 $string['description'] = 'Description';
 $string['details'] = 'Details';
-$string['check_configrw_details'] = '<p>It is recommended that the file permissions of config.php are changed after installation so that the file cannot be modified by the web server.
+$string['check_configrw_details'] = '<p>It is recommended that the file permissions of <code>config.php</code> are changed after installation so that the file cannot be modified by the web server.
 Please note that this measure does not improve security of the server significantly, though it may slow down or limit general exploits.</p>';
 $string['check_configrw_name'] = 'Writable config.php';
 $string['check_configrw_ok'] = 'config.php can not be modified by PHP scripts.';
 $string['check_configrw_warning'] = 'PHP scripts may modify config.php.';
 $string['check_cookiesecure_details'] = '<p>If https communication is enabled, it is recommended to enable sending of secure cookies. You should have permanent redirection from http to https and ideally serve HSTS headers as well.</p>';
 $string['check_cookiesecure_error'] = 'Please enable secure cookies';
+$string['check_cookiesecure_http'] = 'You must turn on https in order to use secure cookies';
 $string['check_cookiesecure_name'] = 'Secure cookies';
 $string['check_cookiesecure_ok'] = 'Secure cookies enabled.';
 $string['check_defaultuserrole_details'] = '<p>All logged in users are given capabilities of the default user role. Please make sure no risky capabilities are allowed in this role.</p>
@@ -55,32 +55,34 @@ $string['check_embed_details'] = '<p>Unlimited object embedding is very dangerou
 $string['check_embed_error'] = 'Unlimited object embedding enabled - this is very dangerous for the majority of servers.';
 $string['check_embed_name'] = 'Allow EMBED and OBJECT';
 $string['check_embed_ok'] = 'Unlimited object embedding is not allowed.';
-$string['check_frontpagerole_details'] = '<p>The default frontpage role is given to all registered users for frontpage activities. Please make sure no risky capabilities are allowed for this role.</p>
+$string['check_frontpagerole_details'] = '<p>The default site home role is given to all authenticated users for site home activities. Please make sure no risky capabilities are allowed for this role.</p>
 <p>It is recommended that a special role is created for this purpose and a legacy type role is not used.</p>';
-$string['check_frontpagerole_error'] = 'Incorrectly defined frontpage role "{$a}" detected!';
-$string['check_frontpagerole_name'] = 'Frontpage role';
-$string['check_frontpagerole_notset'] = 'Frontpage role is not set.';
-$string['check_frontpagerole_ok'] = 'Frontpage role definition is OK.';
+$string['check_frontpagerole_error'] = 'Incorrectly defined site home role "{$a}" detected!';
+$string['check_frontpagerole_name'] = 'Site home role';
+$string['check_frontpagerole_notset'] = 'Site home role is not set.';
+$string['check_frontpagerole_ok'] = 'Site home role definition is OK.';
 $string['check_crawlers_details'] = '<p>The "Open to search engines" setting enables search engines to enter courses with guest access. There is no point in enabling this setting if guest login is not allowed.</p>';
 $string['check_crawlers_error'] = 'Search engine access is allowed but guest access is disabled.';
 $string['check_crawlers_info'] = 'Search engines may enter as guests.';
 $string['check_crawlers_name'] = 'Open to search engines';
 $string['check_crawlers_ok'] = 'Search engine access is not enabled.';
+
+$string['check_antivirus_details'] = 'This status checks whether or not there has been a recent error detected based on the threshold set in the main antivirus settings.';
+$string['check_antivirus_error'] = '{$a->errors} errors have been detected within the last {$a->lookback}';
+$string['check_antivirus_info'] = 'No antivirus scanners are currently enabled';
+$string['check_antivirus_name'] = 'Antivirus';
+$string['check_antivirus_ok'] = '{$a->scanners} antivirus scanner(s) enabled, no issues have been detected in the last {$a->lookback}';
+$string['check_antivirus_logstore_not_supported'] = 'Unable to verify state of antivirus scanners due to the type of log store chosen';
+
+$string['check_dotfiles_info'] = 'All dotfiles except /.well-known/* should not be public';
+$string['check_dirindex_info'] = 'Directory index should not be enabled';
 $string['check_guestrole_details'] = '<p>The guest role is used for guests, not logged in users and temporary guest course access. Please make sure no risky capabilities are allowed in this role.</p>
 <p>The only supported legacy type for guest role is <em>Guest</em>.</p>';
 $string['check_guestrole_error'] = 'The guest role "{$a}" is incorrectly defined!';
 $string['check_guestrole_name'] = 'Guest role';
 $string['check_guestrole_notset'] = 'Guest role is not set.';
 $string['check_guestrole_ok'] = 'Guest role definition is OK.';
-$string['check_mediafilterswf_details'] = '<p>Automatic swf embedding is very dangerous - any registered user may launch an XSS attack against other server users. Please disable it on production servers.</p>';
-$string['check_mediafilterswf_error'] = 'Flash media filter is enabled - this is very dangerous for the majority of servers.';
-$string['check_mediafilterswf_name'] = 'Enabled .swf media filter';
-$string['check_mediafilterswf_ok'] = 'Flash media filter is not enabled.';
-$string['check_noauth_details'] = '<p>The <em>No authentication</em> plugin is not intended for production sites. Please disable it unless this is a development test site.</p>';
-$string['check_noauth_error'] = 'The No authentication plugin cannot be used on production sites.';
-$string['check_noauth_name'] = 'No authentication';
-$string['check_noauth_ok'] = 'No authentication plugin is disabled.';
-$string['check_nodemodules_details'] = '<p>The directory <em>{$a->path}</em> contains Node.js modules and their dependencies, typically installed by the NPM utility. These modules may be needed for local Moodle development, such as for using the grunt framework. They are not needed to run a Moodle site in production and they can contain potentially dangerous code exposing your site to remote attacks.</p><p>It is strongly recommended to remove the directory if the site is available via a public URL, or at least prohibit web access to it in your webserver configuration.</p>';
+$string['check_nodemodules_details'] = '<p>The directory <code>{$a->path}</code> contains Node.js modules and their dependencies, typically installed by the NPM utility. These modules may be needed for local Moodle development, such as for using the grunt framework. They are not needed to run a Moodle site in production and they can contain potentially dangerous code exposing your site to remote attacks.</p><p>It is strongly recommended to remove the directory if the site is available via a public URL, or at least prohibit web access to it in your webserver configuration.</p>';
 $string['check_nodemodules_info'] = 'The node_modules directory should not be present on public sites.';
 $string['check_nodemodules_name'] = 'Node.js modules directory';
 $string['check_openprofiles_details'] = 'Open user profiles can be abused by spammers. It is recommended that either <code>Force users to log in for profiles</code> or <code>Force users to log in</code> are enabled.';
@@ -95,8 +97,12 @@ $string['check_passwordpolicy_ok'] = 'Password policy enabled.';
 $string['check_preventexecpath_name'] = 'Executable paths';
 $string['check_preventexecpath_ok'] = 'Executable paths only settable in config.php.';
 $string['check_preventexecpath_warning'] = 'Executable paths can be set in the Admin GUI.';
-$string['check_preventexecpath_details'] = '<p>Allowing executable paths to be set via the Admin GUI is a vector for privilege escalation.</p>';
-
+$string['check_preventexecpath_details'] = '<p>Allowing executable paths to be set via the Admin GUI is a vector for privilege escalation. This must be forced in config.php:</p><p><code>$CFG->preventexecpath = true;<code></p>';
+$string['check_publicpaths_name'] = 'Check all public / private paths';
+$string['check_publicpaths_ok'] = 'All internal paths are not publicly accessible';
+$string['check_publicpaths_warning'] = 'Some internal paths are publicly accessible';
+$string['check_publicpaths_generic'] = '{$a} files should not be public';
+$string['check_publicpaths_403'] = ' (Returned a 403, ideally should be 404)';
 $string['check_riskadmin_detailsok'] = '<p>Please verify the following list of system administrators:</p>{$a}';
 $string['check_riskadmin_detailswarning'] = '<p>Please verify the following list of system administrators:</p>{$a->admins}
 <p>It is recommended to assign administrator role in the system context only. The following users have (unsupported) admin role assignments in other contexts:</p>{$a->unsupported}';
@@ -124,21 +130,22 @@ $string['check_unsecuredataroot_error'] = 'Your dataroot directory <code>{$a}</c
 $string['check_unsecuredataroot_name'] = 'Insecure dataroot';
 $string['check_unsecuredataroot_ok'] = 'Dataroot directory must not be accessible via the web.';
 $string['check_unsecuredataroot_warning'] = 'Your dataroot directory <code>{$a}</code> is in the wrong location and might be exposed to the web.';
-$string['check_vendordir_details'] = '<p>The directory <em>{$a->path}</em> contains various third-party libraries and their dependencies, typically installed by the PHP Composer. These libraries may be needed for local Moodle development, such as for installing the PHPUnit framework. They are not needed to run a Moodle site in production and they can contain potentially dangerous code exposing your site to remote attacks.</p><p>It is strongly recommended to remove the directory if the site is available via a public URL, or at least prohibit web access to it in your webserver configuration.</p>';
+$string['check_vendordir_details'] = '<p>The directory <code>{$a->path}</code> contains various third-party libraries and their dependencies, typically installed by the PHP Composer. These libraries may be needed for local Moodle development, such as for installing the PHPUnit framework. They are not needed to run a Moodle site in production and they can contain potentially dangerous code exposing your site to remote attacks.</p><p>It is strongly recommended to remove the directory if the site is available via a public URL, or at least prohibit web access to it in your webserver configuration.</p>';
 $string['check_vendordir_info'] = 'The vendor directory should not be present on public sites.';
 $string['check_vendordir_name'] = 'Vendor directory';
 $string['check_webcron_details'] = '<p>Running the cron from a web browser can expose privileged information to anonymous users. It is recommended to only run the cron from the command line or set a cron password for remote access.</p>';
 $string['check_webcron_warning'] = 'Anonymous users can access cron.';
 $string['check_webcron_name'] = 'Web cron';
 $string['check_webcron_ok'] = 'Anonymous users can not access cron.';
+$string['eventreportviewed'] = 'Viewed security check report';
 $string['issue'] = 'Issue';
-$string['pluginname'] = 'Security overview';
+$string['pluginname'] = 'Security checks';
 $string['security:view'] = 'View security report';
-$string['status'] = 'Status';
-$string['statuscritical'] = 'Critical';
-$string['statusinfo'] = 'Information';
-$string['statusok'] = 'OK';
-$string['statusserious'] = 'Serious';
-$string['statuswarning'] = 'Warning';
 $string['timewarning'] = 'Data processing may take a long time, please be patient...';
 $string['privacy:metadata'] = 'The Security overview plugin does not store any personal data.';
+
+// Deprecated since Moodle 4.0.
+$string['check_mediafilterswf_details'] = '<p>Automatic swf embedding is very dangerous - any registered user may launch an XSS attack against other server users. Please disable it on production servers.</p>';
+$string['check_mediafilterswf_error'] = 'Flash media filter is enabled - this is very dangerous for the majority of servers.';
+$string['check_mediafilterswf_name'] = 'Enabled .swf media filter';
+$string['check_mediafilterswf_ok'] = 'Flash media filter is not enabled.';
