@@ -16,6 +16,8 @@
 
 namespace core_user;
 
+use core_text;
+
 /**
  * Class for retrieving information about user fields that are needed for displaying user identity.
  *
@@ -345,7 +347,8 @@ class fields {
      * showing lists of users (in addition to the user's name which is included as standard).
      *
      * The results include basic field names (columns from the 'user' database table) and, unless
-     * turned off, custom profile field names in the format 'profile_field_myfield'.
+     * turned off, custom profile field names in the format 'profile_field_myfield', note these
+     * fields will always be returned lower cased to match how they are returned by the DML library.
      *
      * This function does all the required capability checks to see if the current user is allowed
      * to see them in the specified context. You can pass context null to get all the fields
@@ -380,6 +383,10 @@ class fields {
                         case PROFILE_VISIBLE_PRIVATE:
                             $allowed = !$context || has_capability('moodle/user:viewalldetails', $context);
                             break;
+                        case PROFILE_VISIBLE_TEACHERS:
+                            // This is actually defined (in user/profile/lib.php) based on whether
+                            // you have moodle/site:viewuseridentity in context. We already checked
+                            // that, so treat it as visible (fall through).
                         case PROFILE_VISIBLE_ALL:
                             $allowed = true;
                             break;
@@ -415,7 +422,7 @@ class fields {
 
         // Re-index the entries and return.
         $extra = array_values($extra);
-        return $extra;
+        return array_map([core_text::class, 'strtolower'], $extra);
     }
 
     /**
@@ -532,7 +539,8 @@ class fields {
                     $placeholder = '?';
                     $params[] = $shortname;
                 }
-                $joins .= " JOIN {user_info_field} $fieldalias ON $fieldalias.shortname = $placeholder
+                $joins .= " JOIN {user_info_field} $fieldalias ON " .
+                                 $DB->sql_equal($fieldalias . '.shortname', $placeholder, false) . "
                        LEFT JOIN {user_info_data} $dataalias ON $dataalias.fieldid = $fieldalias.id
                                  AND $dataalias.userid = {$usertable}id";
                 // For Oracle we need to convert the field into a usable format.
@@ -567,6 +575,65 @@ class fields {
                 'mappings' => $mappings];
     }
 
+    /**
+     * Similar to {@see \moodle_database::sql_fullname} except it returns all user name fields as defined by site config, in a
+     * single select statement suitable for inclusion in a query/filter for a users fullname, e.g.
+     *
+     * [$select, $params] = fields::get_sql_fullname('u');
+     * $users = $DB->get_records_sql_menu("SELECT u.id, {$select} FROM {user} u", $params);
+     *
+     * @param string|null $tablealias User table alias, if set elsewhere in the query, null if not required
+     * @param bool $override If true then the alternativefullnameformat format rather than fullnamedisplay format will be used
+     * @return array SQL select snippet and parameters
+     */
+    public static function get_sql_fullname(?string $tablealias = 'u', bool $override = false): array {
+        global $DB;
+
+        $unique = self::$uniqueidentifier++;
+
+        $namefields = self::get_name_fields();
+
+        // Create a dummy user object containing all name fields.
+        $dummyuser = (object) array_combine($namefields, $namefields);
+        $dummyfullname = fullname($dummyuser, $override);
+
+        // Extract any name fields from the fullname format in the order that they appear.
+        $matchednames = array_values(order_in_string($namefields, $dummyfullname));
+        $namelookup = $namepattern = $elements = $params = [];
+
+        foreach ($namefields as $index => $namefield) {
+            $namefieldwithalias = $tablealias ? "{$tablealias}.{$namefield}" : $namefield;
+
+            // Coalesce the name fields to ensure we don't return null.
+            $emptyparam = "uf{$unique}ep_{$index}";
+            $namelookup[$namefield] = "COALESCE({$namefieldwithalias}, :{$emptyparam})";
+            $params[$emptyparam] = '';
+
+            $namepattern[] = '\b' . preg_quote($namefield) . '\b';
+        }
+
+        // Grab any content between the name fields, inserting them after each name field.
+        $chunks = preg_split('/(' . implode('|', $namepattern) . ')/', $dummyfullname);
+        foreach ($chunks as $index => $chunk) {
+            if ($index > 0) {
+                $elements[] = $namelookup[$matchednames[$index - 1]];
+            }
+
+            if (core_text::strlen($chunk) > 0) {
+                // If content is just whitespace, add to elements directly (also Oracle doesn't support passing ' ' as param).
+                if (preg_match('/^\s+$/', $chunk)) {
+                    $elements[] = "'$chunk'";
+                } else {
+                    $elementparam = "uf{$unique}fp_{$index}";
+                    $elements[] = ":{$elementparam}";
+                    $params[$elementparam] = $chunk;
+                }
+            }
+        }
+
+        return [$DB->sql_concat(...$elements), $params];
+    }
+
     /**
      * Gets the display name of a given user field.
      *
@@ -583,31 +650,13 @@ class fields {
         // Custom fields have special handling.
         if (preg_match(self::PROFILE_FIELD_REGEX, $field, $matches)) {
             require_once($CFG->dirroot . '/user/profile/lib.php');
-            $fieldinfo = profile_get_custom_field_data_by_shortname($matches[1]);
+            $fieldinfo = profile_get_custom_field_data_by_shortname($matches[1], false);
             // Use format_string so it can be translated with multilang filter if necessary.
             return $fieldinfo ? format_string($fieldinfo->name) : $field;
         }
 
         // Some fields have language strings which are not the same as field name.
         switch ($field) {
-            case 'url' : {
-                return get_string('webpage');
-            }
-            case 'icq' : {
-                return get_string('icqnumber');
-            }
-            case 'skype' : {
-                return get_string('skypeid');
-            }
-            case 'aim' : {
-                return get_string('aimid');
-            }
-            case 'yahoo' : {
-                return get_string('yahooid');
-            }
-            case 'msn' : {
-                return get_string('msnid');
-            }
             case 'picture' : {
                 return get_string('pictureofuser');
             }
